@@ -33,6 +33,11 @@ void Allocate_Arrays(int wherefrom)
         SpeVPS[i] = (char*)malloc(sizeof(char)*YOUSO10);
       }  
 
+      Spe_AtomicMass = (double*)malloc(sizeof(double)*SpeciesNum);
+      for (i=0; i<SpeciesNum; i++){
+        Spe_AtomicMass[i] = -1.0;
+      }    
+
       Spe_MaxL_Basis = (int*)malloc(sizeof(int)*SpeciesNum);
 
       Spe_Num_Basis = (int**)malloc(sizeof(int*)*SpeciesNum);
@@ -67,10 +72,18 @@ void Allocate_Arrays(int wherefrom)
       Rcv_DS_NL_Size = (int*)malloc(sizeof(int)*Num_Procs);
       Snd_HFS_Size = (int*)malloc(sizeof(int)*Num_Procs);
       Rcv_HFS_Size = (int*)malloc(sizeof(int)*Num_Procs);
-      Start_Grid1 = (int*)malloc(sizeof(int)*Num_Procs);
-      End_Grid1   = (int*)malloc(sizeof(int)*Num_Procs);
-      Start_Grid2 = (int*)malloc(sizeof(int)*Num_Procs);
-      End_Grid2   = (int*)malloc(sizeof(int)*Num_Procs);
+
+      Num_Snd_Grid_A2B = (int*)malloc(sizeof(int)*Num_Procs);
+      Num_Rcv_Grid_A2B = (int*)malloc(sizeof(int)*Num_Procs);
+      Num_Snd_Grid_B2C = (int*)malloc(sizeof(int)*Num_Procs);
+      Num_Rcv_Grid_B2C = (int*)malloc(sizeof(int)*Num_Procs);
+      Num_Snd_Grid_B2D = (int*)malloc(sizeof(int)*Num_Procs);
+      Num_Rcv_Grid_B2D = (int*)malloc(sizeof(int)*Num_Procs);
+      Num_Snd_Grid_B_AB2CA = (int*)malloc(sizeof(int)*Num_Procs);
+      Num_Rcv_Grid_B_AB2CA = (int*)malloc(sizeof(int)*Num_Procs);
+      Num_Snd_Grid_B_CA2CB = (int*)malloc(sizeof(int)*Num_Procs);
+      Num_Rcv_Grid_B_CA2CB = (int*)malloc(sizeof(int)*Num_Procs);
+
       VPS_j_dependency = (int*)malloc(sizeof(int)*SpeciesNum);
 
       EH0_scaling = (double**)malloc(sizeof(double*)*SpeciesNum);
@@ -154,6 +167,10 @@ void Allocate_Arrays(int wherefrom)
 
       time_per_atom = (double*)malloc(sizeof(double)*(atomnum+1));
 
+      if (Solver==1 || Solver==5 || Solver==6 || Solver==8){
+        orderN_FNAN_SNAN = (int*)malloc(sizeof(int)*(atomnum+1)); 
+      }
+
       /* spin non-collinear */
       if (SpinP_switch==3){
         Angle0_Spin = (double*)malloc(sizeof(double)*(atomnum+1));
@@ -263,12 +280,32 @@ void Allocate_Arrays(int wherefrom)
       Spe_Total_CNO = (int*)malloc(sizeof(int)*SpeciesNum);
       FNAN = (int*)malloc(sizeof(int)*(atomnum+1));
       SNAN = (int*)malloc(sizeof(int)*(atomnum+1));
-      SNAN_GDC = (int*)malloc(sizeof(int)*(atomnum+1));
-      True_SNAN = (int*)malloc(sizeof(int)*(atomnum+1));
       zp = (dcomplex*)malloc(sizeof(dcomplex)*POLES);
       Ep = (dcomplex*)malloc(sizeof(dcomplex)*POLES);
       Rp = (dcomplex*)malloc(sizeof(dcomplex)*POLES);
 
+      /* Cluster2 */
+
+      if (Solver==9){
+
+        ON2_zp = (dcomplex*)malloc(sizeof(dcomplex)*(ON2_Npoles+2));
+        ON2_Rp = (dcomplex*)malloc(sizeof(dcomplex)*(ON2_Npoles+2));
+        ON2_method = (int*)malloc(sizeof(int)*(ON2_Npoles+2));
+
+        ON2_zp_f = (dcomplex*)malloc(sizeof(dcomplex)*(ON2_Npoles+2));
+        ON2_Rp_f = (dcomplex*)malloc(sizeof(dcomplex)*(ON2_Npoles+2));
+        ON2_method_f = (int*)malloc(sizeof(int)*(ON2_Npoles+2));
+
+ 	for (i=0; i<(ON2_Npoles+2); i++){
+          ON2_zp[i] = Complex(0.0,0.0);
+          ON2_Rp[i] = Complex(0.0,0.0);
+          ON2_zp_f[i] = Complex(0.0,0.0);
+          ON2_Rp_f[i] = Complex(0.0,0.0);
+          ON2_method[i] = 0;
+          ON2_method_f[i] = 0;
+	}
+      }
+
     break;
 
     case 3:
@@ -300,12 +337,12 @@ void Allocate_Arrays(int wherefrom)
 
       ncn  = (int**)malloc(sizeof(int*)*(atomnum+1));
       for (ct_AN=0; ct_AN<=atomnum; ct_AN++){
-        ncn[ct_AN]  = (int*)malloc(sizeof(int)*((int)(Max_FSNAN*ScaleSize)+1));
+        ncn[ct_AN] = (int*)malloc(sizeof(int)*((int)(Max_FSNAN*ScaleSize)+1));
       }
 
       Dis  = (double**)malloc(sizeof(double*)*(atomnum+1));
       for (ct_AN=0; ct_AN<=atomnum; ct_AN++){
-        Dis[ct_AN]  = (double*)malloc(sizeof(double)*((int)(Max_FSNAN*ScaleSize)+1));
+        Dis[ct_AN] = (double*)malloc(sizeof(double)*((int)(Max_FSNAN*ScaleSize)+1));
       }
 
       alloc_first[8] = 0;
@@ -372,8 +409,14 @@ void Allocate_Arrays(int wherefrom)
 
       Spe_Atomic_Den = (double**)malloc(sizeof(double*)*List_YOUSO[18]);
       for (i=0; i<List_YOUSO[18]; i++){
-        Spe_Atomic_Den[i] = (double*)malloc(sizeof(double)*List_YOUSO[21]);
-        for (j=0; j<List_YOUSO[21]; j++) Spe_Atomic_Den[i][j] = 0.0;
+        Spe_Atomic_Den[i] = (double*)malloc(sizeof(double)*(List_YOUSO[21]+2));
+        for (j=0; j<(List_YOUSO[21]+2); j++) Spe_Atomic_Den[i][j] = 0.0;
+      }
+
+      Spe_Atomic_Den2 = (double**)malloc(sizeof(double*)*List_YOUSO[18]);
+      for (i=0; i<List_YOUSO[18]; i++){
+        Spe_Atomic_Den2[i] = (double*)malloc(sizeof(double)*(List_YOUSO[21]+2));
+        for (j=0; j<(List_YOUSO[21]+2); j++) Spe_Atomic_Den2[i][j] = 0.0;
       }
 
       Spe_PAO_RWF = (double****)malloc(sizeof(double***)*List_YOUSO[18]);
@@ -424,14 +467,14 @@ void Allocate_Arrays(int wherefrom)
 
       Spe_VH_Atom = (double**)malloc(sizeof(double*)*List_YOUSO[18]);
       for (i=0; i<List_YOUSO[18]; i++){
-        Spe_VH_Atom[i] = (double*)malloc(sizeof(double)*List_YOUSO[22]);
-        for (j=0; j<List_YOUSO[22]; j++) Spe_VH_Atom[i][j] = 0.0;
+        Spe_VH_Atom[i] = (double*)malloc(sizeof(double)*(List_YOUSO[22]+2));
+        for (j=0; j<(List_YOUSO[22]+2); j++) Spe_VH_Atom[i][j] = 0.0;
       }
 
       Spe_Atomic_PCC = (double**)malloc(sizeof(double*)*List_YOUSO[18]);
       for (i=0; i<List_YOUSO[18]; i++){
-        Spe_Atomic_PCC[i] = (double*)malloc(sizeof(double)*List_YOUSO[22]);
-        for (j=0; j<List_YOUSO[22]; j++) Spe_Atomic_PCC[i][j] = 0.0;
+        Spe_Atomic_PCC[i] = (double*)malloc(sizeof(double)*(List_YOUSO[22]+2));
+        for (j=0; j<(List_YOUSO[22]+2); j++) Spe_Atomic_PCC[i][j] = 0.0;
       }
 
       Spe_VNL = (double****)malloc(sizeof(double***)*(SO_switch+1));
@@ -633,9 +676,17 @@ void Allocate_Arrays(int wherefrom)
 
       break;
 
+
+  case 10:
+
+    /* MD_VS4 */
+
+    AtomGr = (int*)malloc(sizeof(int)*(atomnum+1));
+    atnum_AtGr = (int*)malloc(sizeof(int)*(num_AtGr+1));
+    Temp_AtGr = (double*)malloc(sizeof(double)*(num_AtGr+1));
+
+    break;
+
   }
 
 }
-
-
-
