@@ -15,20 +15,25 @@
 #include <math.h>
 #include <time.h>
 #include "openmx_common.h"
+#include "mpi.h"
+#include <omp.h>
 
 #define  measure_time   0
 
-#ifdef nompi
-#include "mimic_mpi.h"
-#else
-#include "mpi.h"
-#endif
 
-#ifdef noomp
-#include "mimic_omp.h"
-#else
-#include <omp.h>
-#endif
+
+/*---------- added by TOYODA, 08/JAN/2010 */
+#include "exx_debug.h"
+/*---------- until here */
+
+void EXX_Fock_Cluster(
+  double **H,
+  double *Ux,
+  EXX_t *exx,
+  dcomplex ***exx_CDM, 
+  const int *MP
+  );
+
 
 
 static double Cluster_collinear(
@@ -41,6 +46,9 @@ static double Cluster_collinear(
                    double ****CntOLP,
                    double *****CDM,
                    double *****EDM,
+                   EXX_t *exx,
+                   dcomplex ****exx_CDM,
+                   double *Uexx,
                    double Eele0[2], double Eele1[2]);
 
 static double Cluster_non_collinear(
@@ -69,6 +77,9 @@ double Cluster_DFT(char *mode,
                    double ****CntOLP,
                    double *****CDM,
                    double *****EDM,
+                   EXX_t *exx, 
+                   dcomplex ****exx_CDM,
+                   double *Uexx,
                    double Eele0[2], double Eele1[2])
 {
   static double time0;
@@ -80,7 +91,7 @@ double Cluster_DFT(char *mode,
   if ( (SpinP_switch==0 || SpinP_switch==1) && SO_switch==0 ){
 
     time0 = Cluster_collinear(mode,SCF_iter,SpinP_switch,Cluster_ReCoes,Cluster_ko,
-                              nh,CntOLP,CDM,EDM,Eele0,Eele1);
+                              nh,CntOLP,CDM,EDM,exx,exx_CDM,Uexx,Eele0,Eele1);
   }
 
   /****************************************************
@@ -106,23 +117,27 @@ double Cluster_DFT(char *mode,
 
 
 
-double Cluster_collinear(
-                   char *mode,
-                   int SCF_iter,
-                   int SpinP_switch,
-                   double ***C,
-                   double **ko,
-                   double *****nh, double ****CntOLP,
-                   double *****CDM,
-                   double *****EDM,
-                   double Eele0[2], double Eele1[2])
+
+static double Cluster_collinear(
+				char *mode,
+				int SCF_iter,
+				int SpinP_switch,
+				double ***C,
+				double **ko,
+				double *****nh, double ****CntOLP,
+				double *****CDM,
+				double *****EDM,
+				EXX_t *exx,
+				dcomplex ****exx_CDM,
+				double *Uexx,
+				double Eele0[2], double Eele1[2])
 {
   static int firsttime=1;
   int i,j,l,n,n2,n1,i1,i1s,j1,k1,l1;
   int wan,HOMO0,HOMO1;
   int *MP;
   int step,wstep,nstep,istart,iend;
-  int spin,po,num0,num1;
+  int spin,po,num0,num1,ires;
   int ct_AN,k,wanA,tnoA,wanB,tnoB;
   int GA_AN,Anum,loopN,Gc_AN;
   int MA_AN,LB_AN,GB_AN,Bnum,MaxN;
@@ -131,14 +146,16 @@ double Cluster_collinear(
   double time0,lumos,av_num;
   double *OneD_Mat1;
   double ***H;
-  double TZ,my_sum,sum,sumE,max_x=50.0;
+  double TZ,my_sum,sum,sumE,max_x=60.0;
   double sum0,sum1,sum2,sum3;
   double My_Eele1[2],tmp1,tmp2;
   double Num_State,x,FermiF,Dnum,Dnum2;
+  double FermiF2,x2,diffF;
   double dum,ChemP_MAX,ChemP_MIN,spin_degeneracy;
   double TStime,TEtime;
   double FermiEps = 1.0e-13;
   double EV_cut0;
+  double res;
   int numprocs0,myid0;
   int numprocs1,myid1;
   int Num_Comm_World1;
@@ -162,6 +179,11 @@ double Cluster_collinear(
   MPI_Status *stat_send;
   MPI_Request *request_send;
   MPI_Request *request_recv;
+
+  /*---------- added by TOYODA, 08/JAN/2010 */
+  int exx_nep, iep;
+  const int *exx_atm1, *exx_atm2;
+  /*---------- until here */
  
   /* for time */
   MPI_Barrier(mpi_comm_level1);
@@ -316,6 +338,24 @@ double Cluster_collinear(
     Hamiltonian_Cluster(nh[spin],H[spin],MP);
   } 
 
+  /*---------- added by TOYODA 14/JAN/2010 */
+  if (g_exx_switch) { 
+#if !EXX_MIX_DM
+    EXX_Debug_Copy_DM(MaxN, CDM, exx, exx_CDM, (SCF_iter<10));
+#endif
+
+    for (spin=0; spin<=SpinP_switch; spin++){
+      EXX_Fock_Cluster(H[spin], &Uexx[spin], exx, exx_CDM[spin], MP); 
+    }
+    if (0==SpinP_switch) {
+      Uexx[1] = Uexx[0]; 
+      EXX_Log_Print("EXX: U= %20.12f\n", Uexx[0]);
+    } else {
+      EXX_Log_Print("EXX: U= %20.12f %20.12f\n", Uexx[0], Uexx[1]);
+    }
+  }
+  /*---------- until here */
+
   if (SCF_iter==1){
 
     if (measure_time) dtime(&stime);
@@ -324,11 +364,35 @@ double Cluster_collinear(
 
     bcast_flag = 1;
 
-    if (numprocs0<n)
+    /*
+    if(myid0==0){
+      printf("S before\n");
+      for(i=0;i<Size_Total_Matrix+2;i++){
+	for(j=0;j<Size_Total_Matrix+2;j++){
+	  printf("%2.1f ",S[i][j]);
+	  if(j==Size_Total_Matrix+1) printf("\n");
+	}
+      }
+    }
+    */
+
+    if (numprocs0<n && 1<numprocs0)
       Eigen_PReHH(mpi_comm_level1,S,ko[0],n,n,bcast_flag);
     else 
       Eigen_lapack(S,ko[0],n,n);
 
+    /*
+    if(myid0==0){
+      printf("S after\n");
+      for(i=0;i<Size_Total_Matrix+2;i++){
+	for(j=0;j<Size_Total_Matrix+2;j++){
+	  printf("%2.1f ",S[i][j]);
+	  if(j==Size_Total_Matrix+1) printf("\n");
+	}
+      }
+    }
+    */
+
     /* minus eigenvalues to 1.0e-14 */
 
     for (l=1; l<=n; l++){
@@ -560,13 +624,15 @@ double Cluster_collinear(
   else {
 
     if ( strcasecmp(mode,"scf")==0 ) 
-      lumos = (double)n*0.050;      
-    else if ( strcasecmp(mode,"dos")==0 )
       lumos = (double)n*0.100;      
+    else if ( strcasecmp(mode,"dos")==0 )
+      lumos = (double)n*0.200;      
 
-    if (lumos<60.0) lumos = 60.0;
+    if (lumos<400.0) lumos = 400.0;
     MaxN = (TZ-system_charge)/2 + (int)lumos;
     if (n<MaxN) MaxN = n;
+
+    if (cal_partial_charge) MaxN = n; 
   }
 
   if ( numprocs1<=MaxN ){
@@ -604,11 +670,48 @@ double Cluster_collinear(
 
   bcast_flag = 0;
 
-  if (numprocs1<n)
+  /*
+
+  printf("n=%d, MaxN=%d\n",n,MaxN);
+
+    if(myid0==0){
+      printf("C before\n");
+      for(i=0;i<Size_Total_Matrix+2;i++){
+	for(j=0;j<Size_Total_Matrix+2;j++){
+	  printf("%2.1f ",C[spin][i][j]);
+	  if(j==Size_Total_Matrix+1) printf("\n");
+	}
+      }
+    }
+  */
+
+
+  if (numprocs1<n && 1<numprocs1)
     Eigen_PReHH(MPI_CommWD1[myworld1],C[spin],ko[spin],n,MaxN,bcast_flag);
   else 
     Eigen_lapack(C[spin],ko[spin],n,n);
 
+  /*
+    if(myid0==0){
+      printf("C after\n");
+      for(i=0;i<Size_Total_Matrix+2;i++){
+	for(j=0;j<Size_Total_Matrix+2;j++){
+	  printf("%2.1f ",C[spin][i][j]);
+	  if(j==Size_Total_Matrix+1) printf("\n");
+	}
+      }
+    }
+
+
+  if(myid0==0){
+    printf("ko after eigen\n");
+    for(i=1;i<=MaxN;i++){
+      printf("%d %2.1f\n",i,ko[spin][i]);
+      }
+    }
+  */
+
+
   /*  The H matrix is distributed by row */
   for (i1=1; i1<=n; i1++){
     for (j1=is2[myid1]; j1<=ie2[myid1]; j1++){
@@ -721,9 +824,11 @@ double Cluster_collinear(
 
   nstep = 8*MaxN*n/(1000000*100) + 1;
   wstep = MaxN/nstep;
+  res = ((double)MaxN/(double)nstep - (double)wstep)*(double)nstep + 3.0;
+  ires = (int)res;
 
- /* allocate */
-  OneD_Mat1 = (double*)malloc(sizeof(double)*(wstep+2)*(n+1));
+  /* allocate */
+  OneD_Mat1 = (double*)malloc(sizeof(double)*(wstep+ires)*(n+1));
 
   for (spin=0; spin<=SpinP_switch; spin++){
 
@@ -768,10 +873,6 @@ double Cluster_collinear(
 
   if ( strcasecmp(mode,"scf")==0 ){
 
-    /****************************************************
-              searching of chemical potential
-    ****************************************************/
-
     if (2<=level_stdout){
       for (i1=1; i1<=MaxN; i1++){
 	if (SpinP_switch==0)
@@ -783,6 +884,10 @@ double Cluster_collinear(
       }
     }
 
+    /****************************************************
+              searching of chemical potential
+    ****************************************************/
+
     po = 0;
     loopN = 0;
 
@@ -808,7 +913,7 @@ double Cluster_collinear(
       Dnum = (TZ - Num_State) - system_charge;
       if (0.0<=Dnum) ChemP_MIN = ChemP;
       else           ChemP_MAX = ChemP;
-      if (fabs(Dnum)<1.0e-13) po = 1;
+      if (fabs(Dnum)<1.0e-14) po = 1;
 
       if (myid1==Host_ID && 2<=level_stdout){
 	printf("ChemP=%15.12f TZ=%15.12f Num_state=%15.12f\n",ChemP,TZ,Num_State); 
@@ -818,7 +923,7 @@ double Cluster_collinear(
 
     } 
     while (po==0 && loopN<1000); 
-  
+
     if (2<=level_stdout){
       printf("  ChemP=%15.12f\n",ChemP);
     }
@@ -881,7 +986,13 @@ double Cluster_collinear(
 
       for (spin=0; spin<=SpinP_switch; spin++){
 	for (j=0; j<num_HOMOs; j++){
+
 	  j1 = Cluster_HOMO[spin] - j;
+
+          /* store eigenvalue */
+          HOMOs_Coef[0][spin][j][0][0].r = ko[spin][j1] ;
+
+          /* store eigenvector */
 	  for (GA_AN=1; GA_AN<=atomnum; GA_AN++){
 	    wanA = WhatSpecies[GA_AN];
 	    tnoA = Spe_Total_CNO[wanA];
@@ -897,7 +1008,13 @@ double Cluster_collinear(
    
       for (spin=0; spin<=SpinP_switch; spin++){
 	for (j=0; j<num_LUMOs; j++){
+
 	  j1 = Cluster_HOMO[spin] + 1 + j;
+
+          /* store eigenvalue */
+	  LUMOs_Coef[0][spin][j][0][0].r = ko[spin][j1];
+
+          /* store eigenvector */
 	  for (GA_AN=1; GA_AN<=atomnum; GA_AN++){
 	    wanA = WhatSpecies[GA_AN];
 	    tnoA = Spe_Total_CNO[wanA];
@@ -932,6 +1049,10 @@ double Cluster_collinear(
 	    for (j=0; j<tnoB; j++){
 	      CDM[spin][MA_AN][LB_AN][i][j] = 0.0;
 	      EDM[spin][MA_AN][LB_AN][i][j] = 0.0;
+
+              if (cal_partial_charge){
+ 	        Partial_DM[spin][MA_AN][LB_AN][i][j] = 0.0;
+              }
 	    }
 	  }
 	}
@@ -946,9 +1067,17 @@ double Cluster_collinear(
 	if (max_x<=x)  x = max_x;
 	FermiF = 1.0/(1.0 + exp(x));
 
-	if (FermiF>FermiEps) {
+        if (cal_partial_charge){
+	  x2 = (ko[spin][k] - (ChemP+ene_win_partial_charge))*Beta;
+	  if (x2<=-max_x) x2 = -max_x;
+	  if (max_x<=x2)  x2 = max_x;
+  	  FermiF2 = 1.0/(1.0 + exp(x2));
+          diffF = fabs(FermiF-FermiF2);
+	}
 
-#pragma omp parallel shared(ko,EDM,CDM,k,spin,C,FermiF,natn,FNAN,MP,Spe_Total_CNO,WhatSpecies,M2G,Matomnum) private(OMPID,Nthrds,Nprocs,MA_AN,GA_AN,wanA,tnoA,Anum,LB_AN,GB_AN,wanB,tnoB,Bnum,i,j,dum)
+	if (FermiF>FermiEps || (cal_partial_charge && (FermiEps<FermiF || FermiEps<FermiF2) )) {
+
+#pragma omp parallel shared(ko,EDM,CDM,Partial_DM,k,spin,C,FermiF,diffF,natn,FNAN,MP,Spe_Total_CNO,WhatSpecies,M2G,Matomnum) private(OMPID,Nthrds,Nprocs,MA_AN,GA_AN,wanA,tnoA,Anum,LB_AN,GB_AN,wanB,tnoB,Bnum,i,j,dum)
 	  { 
 
 	    /* get info. on OpenMP */ 
@@ -969,9 +1098,15 @@ double Cluster_collinear(
 		Bnum = MP[GB_AN];
 		for (i=0; i<tnoA; i++){
 		  for (j=0; j<tnoB; j++){
+
 		    dum = FermiF*C[spin][k][Anum+i]*C[spin][k][Bnum+j];
 		    CDM[spin][MA_AN][LB_AN][i][j] += dum;
 		    EDM[spin][MA_AN][LB_AN][i][j] += dum*ko[spin][k];
+
+                    if (cal_partial_charge){
+                       dum = diffF*C[spin][k][Anum+i]*C[spin][k][Bnum+j];
+		       Partial_DM[spin][MA_AN][LB_AN][i][j] += dum;
+                    }
 		  }
 		}
 	      }
@@ -983,6 +1118,58 @@ double Cluster_collinear(
       }
     }
 
+    /*---------- added by TOYODA 08/JAN/2010 from here */
+    if (g_exx_switch) 
+    {
+      exx_nep  = EXX_Number_of_EP(exx);
+      exx_atm1 = EXX_Array_EP_Atom1(exx);
+      exx_atm2 = EXX_Array_EP_Atom2(exx);
+
+      /* clear EXX-DM */
+      for (spin=0; spin<=SpinP_switch; spin++){
+        for (iep=0; iep<exx_nep; iep++) {
+          GA_AN = exx_atm1[iep]+1;
+          GB_AN = exx_atm2[iep]+1;
+          tnoA  = Spe_Total_CNO[WhatSpecies[GA_AN]];
+          tnoB  = Spe_Total_CNO[WhatSpecies[GB_AN]];
+          for (i=0; i<tnoA; i++){
+            for (j=0; j<tnoB; j++){
+              exx_CDM[spin][iep][i][j].r = 0.0;
+              exx_CDM[spin][iep][i][j].i = 0.0;
+            }
+          }
+        }
+      }
+
+      for (spin=0; spin<=SpinP_switch; spin++){
+        for (k=1; k<=MaxN; k++){
+  
+          x = (ko[spin][k] - ChemP)*Beta;
+          if (x<=-max_x) x = -max_x;
+          if (max_x<=x)  x = max_x;
+          FermiF = 1.0/(1.0 + exp(x));
+
+          if (FermiF>FermiEps) {
+            for (iep=0; iep<exx_nep; iep++) {
+              GA_AN = exx_atm1[iep]+1;
+              GB_AN = exx_atm2[iep]+1;
+              tnoA  = Spe_Total_CNO[WhatSpecies[GA_AN]];
+              tnoB  = Spe_Total_CNO[WhatSpecies[GB_AN]];
+              Anum  = MP[GA_AN];
+              Bnum  = MP[GB_AN];
+              for (i=0; i<tnoA; i++){
+                for (j=0; j<tnoB; j++){
+                  dum = FermiF*C[spin][k][Anum+i]*C[spin][k][Bnum+j];
+                  exx_CDM[spin][iep][i][j].r += dum;
+                }
+              } 
+            } /* loop of iep */
+          } /* end if */
+        } /* loop of k */
+      } /* loop of spin */
+    }
+    /*---------- added by TOYODA 08/JAN/2010 until here */
+
     /****************************************************
                       Bond Energies
     ****************************************************/
@@ -1019,11 +1206,6 @@ double Cluster_collinear(
       printf("Eele10=%15.12f Eele11=%15.12f\n",Eele1[0],Eele1[1]);
     */
 
-    /*
-      MPI_Finalize();
-      exit(0);
-    */
-
     if (measure_time){
       dtime(&etime);
       time6 += etime - stime;
@@ -1267,16 +1449,16 @@ double Cluster_collinear(
 
 
 
-double Cluster_non_collinear(
-                   char *mode,
-                   int SCF_iter,
-                   int SpinP_switch,
-                   double *****nh,
-                   double *****ImNL,
-                   double ****CntOLP,
-                   double *****CDM,
-                   double *****EDM,
-                   double Eele0[2], double Eele1[2])
+static double Cluster_non_collinear(
+				    char *mode,
+				    int SCF_iter,
+				    int SpinP_switch,
+				    double *****nh,
+				    double *****ImNL,
+				    double ****CntOLP,
+				    double *****CDM,
+				    double *****EDM,
+				    double Eele0[2], double Eele1[2])
 {
   static int firsttime=1;
   int i,j,l,n,n2,n1,i1,i1s,j1,k1,l1;
@@ -1300,9 +1482,10 @@ double Cluster_non_collinear(
   double sum_r01,sum_i01;
   double sum_r10,sum_i10;
   double sum_r11,sum_i11;
-  double TZ,my_sum,sum,sumE,max_x=50.0;
+  double TZ,my_sum,sum,sumE,max_x=60.0;
   double My_Eele1[2];
   double Num_State,x,FermiF,Dnum,Dnum2;
+  double FermiF2,x2,diffF;
   double dum,ChemP_MAX,ChemP_MIN;
   double TStime,TEtime;
   double FermiEps = 1.0e-13;
@@ -1463,7 +1646,7 @@ double Cluster_non_collinear(
 
     bcast_flag = 1;
 
-    if (numprocs<n)
+    if (numprocs<n && 1<numprocs)
       Eigen_PReHH(mpi_comm_level1,S,ko,n,n,bcast_flag);
     else 
       Eigen_lapack(S,ko,n,n);
@@ -1702,13 +1885,15 @@ double Cluster_non_collinear(
   else {
 
     if ( strcasecmp(mode,"scf")==0 ) 
-      lumos = (double)n1*0.10;      
+      lumos = (double)n1*0.20;      
     else if ( strcasecmp(mode,"dos")==0 )
-      lumos = (double)n1*0.20;
+      lumos = (double)n1*0.40;
 
-    if (lumos<60.0) lumos = 60.0;
+    if (lumos<400.0) lumos = 400.0;
     MaxN = Cluster_HOMO[0] + (int)lumos;
     if (n1<MaxN) MaxN = n1;
+
+    if (cal_partial_charge) MaxN = n1; 
   }
 
   if ( numprocs<=MaxN ){
@@ -1747,10 +1932,10 @@ double Cluster_non_collinear(
 
   bcast_flag = 0;
 
-  if (numprocs<n1)
+  if (numprocs<n1 && 1<numprocs)
     Eigen_PHH(mpi_comm_level1, C, ko, n1, MaxN, bcast_flag);
   else 
-    EigenBand_lapack(C, ko, n1, 1);
+    EigenBand_lapack(C, ko, n1, MaxN, 1);
 
   if (2<=level_stdout){
     for (i1=1; i1<=MaxN; i1++){
@@ -1861,8 +2046,8 @@ double Cluster_non_collinear(
     po = 0;
     loopN = 0;
 
-    ChemP_MAX = 25.0;  
-    ChemP_MIN =-25.0;
+    ChemP_MAX = 5.0;  
+    ChemP_MIN =-5.0;
 
     do {
       ChemP = 0.50*(ChemP_MAX + ChemP_MIN);
@@ -1880,7 +2065,7 @@ double Cluster_non_collinear(
       Dnum = (TZ - Num_State) - system_charge;
       if (0.0<=Dnum) ChemP_MIN = ChemP;
       else           ChemP_MAX = ChemP;
-      if (fabs(Dnum)<1.0e-13) po = 1;
+      if (fabs(Dnum)<1.0e-14) po = 1;
       loopN++;
     } 
     while (po==0 && loopN<1000); 
@@ -1927,7 +2112,14 @@ double Cluster_non_collinear(
       /* HOMOs */
 
       for (j=0; j<num_HOMOs; j++){
+
 	j1 = Cluster_HOMO[0] - j;
+
+        /* store eigenvalue */
+        HOMOs_Coef[0][0][j][0][0].r = ko[j1];
+        HOMOs_Coef[0][1][j][0][0].r = ko[j1];
+
+        /* store eigenvector */
 	for (GA_AN=1; GA_AN<=atomnum; GA_AN++){
 	  wanA = WhatSpecies[GA_AN];
 	  tnoA = Spe_Total_CNO[wanA];
@@ -1944,7 +2136,14 @@ double Cluster_non_collinear(
       /* LUMOs */
 
       for (j=0; j<num_LUMOs; j++){
+
 	j1 = Cluster_HOMO[0] + 1 + j;
+
+        /* store eigenvalue */
+        LUMOs_Coef[0][0][j][0][0].r = ko[j1];
+        LUMOs_Coef[0][1][j][0][0].r = ko[j1];
+
+        /* store eigenvector */
 	for (GA_AN=1; GA_AN<=atomnum; GA_AN++){
 	  wanA = WhatSpecies[GA_AN];
 	  tnoA = Spe_Total_CNO[wanA];
@@ -1988,10 +2187,12 @@ double Cluster_non_collinear(
 	  wanB = WhatSpecies[GB_AN];
 	  tnoB = Spe_Total_CNO[wanB];
 	  Bnum = MP[GB_AN];
+
 	  for (i=0; i<tnoA; i++){
 	    for (j=0; j<tnoB; j++){
 	      CDM[spin][MA_AN][LB_AN][i][j] = 0.0;
 	      EDM[spin][MA_AN][LB_AN][i][j] = 0.0;
+
 	    }
 	  }
 
@@ -2002,6 +2203,13 @@ double Cluster_non_collinear(
 	    }
 	  }
 
+          if (cal_partial_charge && spin<=1){
+	    for (i=0; i<tnoA; i++){
+	      for (j=0; j<tnoB; j++){
+		Partial_DM[spin][MA_AN][LB_AN][i][j] = 0.0;
+	      }
+	    }
+	  }
 	}
       }
     }
@@ -2013,9 +2221,17 @@ double Cluster_non_collinear(
       if (max_x<=x)  x = max_x;
       FermiF = 1.0/(1.0 + exp(x));
 
-      if (FermiF>FermiEps) {
+      if (cal_partial_charge){
+	x2 = (ko[k] - (ChemP+ene_win_partial_charge))*Beta;
+	if (x2<=-max_x) x2 = -max_x;
+	if (max_x<=x2)  x2 = max_x;
+	FermiF2 = 1.0/(1.0 + exp(x2));
+	diffF = fabs(FermiF-FermiF2);
+      }
 
-#pragma omp parallel shared(ko,iDM,EDM,CDM,k,spin,C,FermiF,natn,FNAN,MP,Spe_Total_CNO,WhatSpecies,M2G,Matomnum) private(OMPID,Nthrds,Nprocs,MA_AN,GA_AN,wanA,tnoA,Anum,LB_AN,GB_AN,wanB,tnoB,Bnum,i,j,dum)
+      if ( FermiF>FermiEps || (cal_partial_charge && (FermiEps<FermiF || FermiEps<FermiF2) ) ) {
+
+#pragma omp parallel shared(ko,iDM,EDM,CDM,Partial_DM,k,spin,C,FermiF,diffF,natn,FNAN,MP,Spe_Total_CNO,WhatSpecies,M2G,Matomnum) private(OMPID,Nthrds,Nprocs,MA_AN,GA_AN,wanA,tnoA,Anum,LB_AN,GB_AN,wanB,tnoB,Bnum,i,j,dum)
 	{ 
 
 	  /* get info. on OpenMP */ 
@@ -2039,37 +2255,49 @@ double Cluster_non_collinear(
               
 		  /* Re11 */
 		  dum = FermiF*(C[k][Anum+i].r*C[k][Bnum+j].r
-				+ C[k][Anum+i].i*C[k][Bnum+j].i);
+			      + C[k][Anum+i].i*C[k][Bnum+j].i);
 		  CDM[0][MA_AN][LB_AN][i][j] += dum;
 		  EDM[0][MA_AN][LB_AN][i][j] += dum*ko[k];
 
 		  /* Re22 */
 		  dum = FermiF*(C[k][Anum+i+n].r*C[k][Bnum+j+n].r
-				+ C[k][Anum+i+n].i*C[k][Bnum+j+n].i);
+			      + C[k][Anum+i+n].i*C[k][Bnum+j+n].i);
 		  CDM[1][MA_AN][LB_AN][i][j] += dum;
 		  EDM[1][MA_AN][LB_AN][i][j] += dum*ko[k];
                
 		  /* Re12 */
 		  dum = FermiF*(C[k][Anum+i].r*C[k][Bnum+j+n].r
-				+ C[k][Anum+i].i*C[k][Bnum+j+n].i);
+			      + C[k][Anum+i].i*C[k][Bnum+j+n].i);
 		  CDM[2][MA_AN][LB_AN][i][j] += dum;
 		  EDM[2][MA_AN][LB_AN][i][j] += dum*ko[k];
               
 		  /* Im12 */
 		  dum = FermiF*(C[k][Anum+i].r*C[k][Bnum+j+n].i
-				-C[k][Anum+i].i*C[k][Bnum+j+n].r);
+			       -C[k][Anum+i].i*C[k][Bnum+j+n].r);
 		  CDM[3][MA_AN][LB_AN][i][j] += dum;
 		  EDM[3][MA_AN][LB_AN][i][j] += dum*ko[k];
 
 		  /* Im11 */
 		  dum = FermiF*(C[k][Anum+i].r*C[k][Bnum+j].i
-				-C[k][Anum+i].i*C[k][Bnum+j].r);
+			       -C[k][Anum+i].i*C[k][Bnum+j].r);
 		  iDM[0][0][MA_AN][LB_AN][i][j] += dum;
 
 		  /* Im22 */
 		  dum = FermiF*(C[k][Anum+i+n].r*C[k][Bnum+j+n].i
-				-C[k][Anum+i+n].i*C[k][Bnum+j+n].r);
+			       -C[k][Anum+i+n].i*C[k][Bnum+j+n].r);
 		  iDM[0][1][MA_AN][LB_AN][i][j] += dum;
+
+                  /* partial density matrix for STM simulation */
+                  if (cal_partial_charge){
+
+		    dum = diffF*(C[k][Anum+i].r*C[k][Bnum+j].r
+			       + C[k][Anum+i].i*C[k][Bnum+j].i);
+		    Partial_DM[0][MA_AN][LB_AN][i][j] += dum;
+
+		    dum = diffF*(C[k][Anum+i+n].r*C[k][Bnum+j+n].r
+			       + C[k][Anum+i+n].i*C[k][Bnum+j+n].i);
+		    Partial_DM[1][MA_AN][LB_AN][i][j] += dum;
+		  }
 		}
 	      }
 	    }
