@@ -16,12 +16,8 @@
 #include <math.h>
 #include <time.h>
 #include "openmx_common.h"
-
-#ifdef nompi
-#include "mimic_mpi.h"
-#else
 #include "mpi.h"
-#endif
+
 
 
 static double Cluster_Col_Dosout( int SpinP_switch, 
@@ -120,7 +116,7 @@ double Cluster_Col_Dosout( int SpinP_switch,  double *****nh, double ****CntOLP)
   MPI_Comm_size(mpi_comm_level1,&numprocs);
   MPI_Comm_rank(mpi_comm_level1,&myid);
 
-  if (myid==Host_ID) {
+  if (myid==Host_ID  && 0<level_stdout) {
     printf("Cluster_DFT_Dosout: start\n"); fflush(stdout);
   }
 
@@ -642,8 +638,8 @@ double Cluster_Col_Dosout( int SpinP_switch,  double *****nh, double ****CntOLP)
 #endif
 
   /****************************************************
-          Density and energy density matrices
-	       for up and down spins
+            calculation of density matrices 
+            for up and down spins
   ****************************************************/
 
   for (spin=0; spin<=SpinP_switch; spin++){
@@ -667,9 +663,7 @@ double Cluster_Col_Dosout( int SpinP_switch,  double *****nh, double ****CntOLP)
 	  }
 	}
       }
-#if  0    
-      printf("%d: step1 %d %d\n",myid,spin,k);
-#endif
+
       /*******************************************
                      M_i = S_ij D_ji
                      D_ji = C_nj C_ni
@@ -701,26 +695,6 @@ double Cluster_Col_Dosout( int SpinP_switch,  double *****nh, double ****CntOLP)
 	  }
 	}
       }
-#if 0
-      printf("%d: step2 %d %d\n",myid,spin,k);
-#endif
-
-
-#if 0
-      /* norm check */
-/*
-      sum=0.0;
-      for (GA_AN=1; GA_AN<=atomnum; GA_AN++){
-        wanA = WhatSpecies[GA_AN];
-        tnoA = Spe_Total_CNO[wanA];
-        for (i1=0; i1<tnoA; i1++){
-	  sum+=SD[GA_AN][i1];
-        }
-      }
-      if (fabs(sum-1.0)>1.0e-3) { printf("norm = %lf\n",sum); }
-*/
-#endif
-
 
       i_vec[0]=i_vec[1]=i_vec[2]=0;
       if (myid==Host_ID) fwrite(i_vec,sizeof(int),3,fp_ev);
@@ -751,11 +725,11 @@ double Cluster_Col_Dosout( int SpinP_switch,  double *****nh, double ****CntOLP)
 
       }
 
+
     } /* k */
   } /* spin */
 
-
-Finishing: ;
+ Finishing: ;
 
   if (myid==Host_ID){
     if (fp_eig) fclose(fp_eig);
@@ -1147,7 +1121,7 @@ double Cluster_NonCol_Dosout( int SpinP_switch,
     /* solve eigenvalue problem */
 
     n1 = 2*n;
-    EigenBand_lapack(C,ko,n1,1);
+    EigenBand_lapack(C,ko,n1,n1,1);
 
     for (i1=1; i1<=n1; i1++){
       for (j1=1; j1<=n1; j1++){
