@@ -15,12 +15,8 @@
 #include <math.h>
 #include <string.h>
 #include "openmx_common.h"
-
-#ifdef nompi
-#include "mimic_mpi.h"
-#else
 #include "mpi.h"
-#endif
+
 
 static void Inverse(int n, double **a, double **ia);
 
@@ -46,12 +42,30 @@ void DIIS_Mixing_DM(int SCF_iter, double ******ResidualDM, double ******iResidua
   int numprocs,myid;
   char nanchar[300];
 
+  /*---------- added by TOYODA 22/JAN/2010 */
+#if EXX_MIX_DM
+  int iep, nep;
+  const int *ep_atom1, *ep_atom2;
+  double sum_r, sum_i;
+#endif
+  /*---------- until here */
+
   /* MPI */
   MPI_Comm_size(mpi_comm_level1,&numprocs);
   MPI_Comm_rank(mpi_comm_level1,&myid);
 
   SpinP_num = SpinP_switch;
 
+  /*---------- added by TOYODA 22/JAN/2010 */
+#if EXX_MIX_DM
+  if (g_exx_switch) {
+    nep = EXX_Number_of_EP(g_exx);
+    ep_atom1 = EXX_Array_EP_Atom1(g_exx);
+    ep_atom2 = EXX_Array_EP_Atom2(g_exx);
+  }
+#endif
+  /*---------- until here */
+
   /****************************************************
     allocation of arrays:
 
@@ -184,7 +198,22 @@ void DIIS_Mixing_DM(int SCF_iter, double ******ResidualDM, double ******iResidua
       firsttime=0;
     }
 
+    /*---------- modified by TOYODA 18/JAN/2010 */
+#if EXX_MIX_DM
+    if (g_exx_switch) {
+      Simple_Mixing_DM(0,1.00,DM[0],DM[1],DM[2],
+        iDM[0],iDM[1],iDM[2],ResidualDM[2],iResidualDM[2],
+         g_exx, g_exx_DM[0], g_exx_DM[1], g_exx_DM[2]);
+    } else {
+      Simple_Mixing_DM(0,1.00,DM[0],DM[1],DM[2],
+        iDM[0],iDM[1],iDM[2],ResidualDM[2],iResidualDM[2],
+        NULL, NULL, NULL, NULL);
+    }
+#else
     Simple_Mixing_DM(0,1.00,DM[0],DM[1],DM[2],iDM[0],iDM[1],iDM[2],ResidualDM[2],iResidualDM[2]);
+#endif
+    /*---------- until here */
+
   } 
 
   else if (SCF_iter==2){
@@ -255,7 +284,45 @@ void DIIS_Mixing_DM(int SCF_iter, double ******ResidualDM, double ******iResidua
       }
     }
 
+    /*---------- added by TOYODA */
+#if EXX_MIX_DM
+    if (g_exx_switch) {
+      for (spin=0; spin<=SpinP_num; spin++){
+        for (iep=0; iep<nep; iep++) {
+          Gc_AN = ep_atom1[iep]+1;
+          wan1 = WhatSpecies[Gc_AN];
+          TNO1 = Spe_Total_CNO[wan1];
+	   
+          Gh_AN = ep_atom2[iep]+1;
+          wan2 = WhatSpecies[Gh_AN];
+          TNO2 = Spe_Total_CNO[wan2];
+
+          for (i=0; i<TNO1; i++){
+	    for (j=0; j<TNO2; j++){
+	      g_exx_DM[2][spin][iep][i][j] = g_exx_DM[1][spin][iep][i][j];
+            }
+          }
+        }
+      }
+    }
+#endif
+    /*---------- until here */
+
+    /*---------- modified by TOYODA 18/JAN/2010 */
+#if EXX_MIX_DM
+    if (g_exx_switch) {
+      Simple_Mixing_DM(0,Mixing_weight,DM[0],DM[1],DM[2],
+        iDM[0],iDM[1],iDM[2],ResidualDM[2],iResidualDM[2],
+        g_exx, g_exx_DM[0], g_exx_DM[1], g_exx_DM[2]);
+    } else {
+      Simple_Mixing_DM(0,Mixing_weight,DM[0],DM[1],DM[2],
+        iDM[0],iDM[1],iDM[2],ResidualDM[2],iResidualDM[2],
+        NULL, NULL, NULL, NULL);
+    }
+#else
     Simple_Mixing_DM(0,Mixing_weight,DM[0],DM[1],DM[2],iDM[0],iDM[1],iDM[2],ResidualDM[2],iResidualDM[2]);
+#endif
+    /*---------- until here */
 
   } 
   else {
@@ -536,6 +603,36 @@ void DIIS_Mixing_DM(int SCF_iter, double ******ResidualDM, double ******iResidua
       }
     }
 
+    /*---------- added by TOYODA 22/JAN/2010 */
+#if EXX_MIX_DM
+    if (g_exx_switch) {
+      for (spin=0; spin<=SpinP_num; spin++){
+        for (iep=0; iep<nep; iep++) {
+          Gc_AN = ep_atom1[iep]+1;
+          Gh_AN = ep_atom2[iep]+1;
+          TNO1 = Spe_Total_CNO[WhatSpecies[Gc_AN]];
+	  TNO2 = Spe_Total_CNO[WhatSpecies[Gh_AN]];
+          for (i=0; i<TNO1; i++){
+	    for (j=0; j<TNO2; j++){
+              /* Pulay mixing */
+              sum_r = 0.0;
+              sum_i = 0.0;
+              for (pSCF_iter=1; pSCF_iter<=NumMix; pSCF_iter++){
+                sum_r += alden[pSCF_iter]
+                  *g_exx_DM[pSCF_iter][spin][iep][i][j].r;
+                sum_i += alden[pSCF_iter]
+                  *g_exx_DM[pSCF_iter][spin][iep][i][j].i;
+	      }
+              g_exx_DM[0][spin][iep][i][j].r = sum_r;
+              g_exx_DM[0][spin][iep][i][j].i = sum_i;
+            }
+          }
+        }
+      }
+    }
+#endif 
+    /*---------- until here */
+
     /****************************************************
                          Shift of DM
     ****************************************************/
@@ -578,6 +675,29 @@ void DIIS_Mixing_DM(int SCF_iter, double ******ResidualDM, double ******iResidua
       }
     }
 
+    /*---------- added by TOYODA 22/JAN/2010 */
+#if EXX_MIX_DM
+    if (g_exx_switch) {
+      for (pSCF_iter=NumSlide; 0<pSCF_iter; pSCF_iter--){
+        for (spin=0; spin<=SpinP_num; spin++){
+          for (iep=0; iep<nep; iep++) {
+            Gc_AN = ep_atom1[iep]+1;
+            TNO1 = Spe_Total_CNO[WhatSpecies[Gc_AN]];
+            Gh_AN = ep_atom2[iep]+1;
+            TNO2 = Spe_Total_CNO[WhatSpecies[Gh_AN]];
+            for (i=0; i<TNO1; i++){
+	      for (j=0; j<TNO2; j++){
+	        g_exx_DM[pSCF_iter][spin][iep][i][j]
+                  = g_exx_DM[pSCF_iter-1][spin][iep][i][j];
+              }
+            }
+          }
+        }
+      }
+    }
+#endif
+    /*---------- until here */
+
     /****************************************************
                          Shift of RDM
     ****************************************************/
