@@ -196,6 +196,12 @@ void Dos_Gaussian( char *basename, int Dos_N, double Dos_Gaussian,
   FILE *fp_Dos;
   char fp_buf[fp_bsize];          /* setvbuf */
 
+  /* sawada */
+
+  int q, sw;
+  double h,s1,s2;
+  double ssum[2][Dos_N];
+
   printf("<Dos_Gaussian> start\n");
 
   if (strlen(basename)==0) {
@@ -280,17 +286,51 @@ void Dos_Gaussian( char *basename, int Dos_N, double Dos_Gaussian,
     for (ie=0;ie<Dos_N;ie++) {
       Dos[spin][ie] = Dos[spin][ie] * factor;
     }
+
+  /* sawada */
+  
+  h = (DosE[Dos_N-1] - DosE[0])/(Dos_N-1)*eV2Hartree;
+  sw = 2;
+  
+  if (sw == 1) {
+
+    /* Trapezoidal rule */
+
+    for (q=0;q<Dos_N;q++) {
+      s1 = 0.0;
+      for (ie=1;ie<q;ie++) {
+        s1 += Dos[spin][ie];
+      }   
+      ssum[spin][q] = (0.5*(Dos[spin][0]+Dos[spin][q])+s1)*h;
+    }
+  } else {
+
+    /* Simpson's rule */
+
+    for (q=0;q<Dos_N;q++) {
+      s1 = 0.0;
+      s2 = 0.0;
+      for (ie=1;ie<q;ie+=2) {
+        s1 += Dos[spin][ie];
+      }
+      for (ie=2;ie<q;ie+=2) {
+        s2 += Dos[spin][ie];
+      }
+      ssum[spin][q] = (Dos[spin][0]+4.0*s1+2.0*s2+Dos[spin][q])*h/3.0;
+    }
+  }
+  
   }
 
   if (SpinP_switch==1) {
     for (ie=0;ie<Dos_N;ie++) {
-      fprintf(fp_Dos,"%lf %lf %lf\n", (DosE[ie])*eV2Hartree, Dos[0][ie], -Dos[1][ie]);
+      fprintf(fp_Dos,"%lf %lf %lf %lf %lf\n", (DosE[ie])*eV2Hartree, Dos[0][ie], -Dos[1][ie], ssum[0][ie], ssum[1][ie]);
     }
   }
 
   else {
     for (ie=0;ie<Dos_N;ie++) {
-      fprintf(fp_Dos,"%lf %lf\n", (DosE[ie])*eV2Hartree, Dos[0][ie]*2.0);
+      fprintf(fp_Dos,"%lf %lf %lf\n", (DosE[ie])*eV2Hartree, Dos[0][ie]*2.0, ssum[0][ie]*2.0);
     }
   }
 
@@ -336,6 +376,12 @@ void DosDC_Gaussian( char *basename, int atomnum,
   FILE *fp_Dos;
   char fp_buf[fp_bsize];          /* setvbuf */
 
+  /* sawada */
+   
+  int q, sw;
+  double h,s1,s2;
+  double ssum[2][Dos_N];
+
   printf("<Dos_Gaussian> start\n");
 
   if (strlen(basename)==0) {
@@ -413,16 +459,50 @@ void DosDC_Gaussian( char *basename, int atomnum,
     for (ie=0;ie<Dos_N;ie++) {
       Dos[spin][ie] = Dos[spin][ie] * factor;
     }
+
+  /* sawada */
+
+  h = (DosE[Dos_N-1] - DosE[0])/(Dos_N-1)*eV2Hartree;
+  sw = 2;
+
+  if (sw == 1) {
+
+    /* Trapezoidal rule */
+
+    for (q=0;q<Dos_N;q++) {
+      s1 = 0.0;
+      for (ie=1;ie<q;ie++) {
+        s1 += Dos[spin][ie];
+      }
+      ssum[spin][q] = (0.5*(Dos[spin][0]+Dos[spin][q])+s1)*h;
+    }
+  } else {
+
+    /* Simpson's rule */
+
+    for (q=0;q<Dos_N;q++) {
+      s1 = 0.0;
+      s2 = 0.0;
+      for (ie=1;ie<q;ie+=2) {
+        s1 += Dos[spin][ie];
+      }
+      for (ie=2;ie<q;ie+=2) {
+        s2 += Dos[spin][ie];
+      }
+      ssum[spin][q] = (Dos[spin][0]+4.0*s1+2.0*s2+Dos[spin][q])*h/3.0;
+    }
+  }
+  
   }
 
   if (SpinP_switch==1) {
     for (ie=0;ie<Dos_N;ie++) {
-      fprintf(fp_Dos,"%lf %lf %lf\n", (DosE[ie])*eV2Hartree, Dos[0][ie], -Dos[1][ie]);
+      fprintf(fp_Dos,"%lf %lf %lf %lf %lf\n", (DosE[ie])*eV2Hartree, Dos[0][ie], -Dos[1][ie], ssum[0][ie], ssum[1][ie]);
     }
   }
   else {
     for (ie=0;ie<Dos_N;ie++) {
-      fprintf(fp_Dos,"%lf %lf\n", (DosE[ie])*eV2Hartree, Dos[0][ie]*2.0);
+      fprintf(fp_Dos,"%lf %lf %lf\n", (DosE[ie])*eV2Hartree, Dos[0][ie]*2.0, ssum[0][ie]*2.0);
     }
   }
 
@@ -468,6 +548,12 @@ void Dos_NEGF( char *basename,
   FILE *fp_Dos;
   char fp_buf[fp_bsize];          /* setvbuf */
 
+  /* sawada */
+
+  int q,sw;
+  double h,s1,s2;
+  double ssum[2][Dos_N];
+
   printf("<Dos_%s> start\n",extension);
 
   if (strlen(basename)==0) {
@@ -522,18 +608,52 @@ void Dos_NEGF( char *basename,
       Dos[spin][ieg] = wval/eV2Hartree;
 
     } /* ieg */
+
+  /* sawada */
+   
+  h = (DosE[Dos_N-1] - DosE[0])/(Dos_N-1)*eV2Hartree;
+  sw = 2; 
+   
+  if (sw == 1) {
+ 
+    /* Trapezoidal rule */
+ 
+    for (q=0;q<Dos_N;q++) {
+      s1 = 0.0;
+      for (ie=1;ie<q;ie++) {
+        s1 += Dos[spin][ie];
+      }   
+      ssum[spin][q] = (0.5*(Dos[spin][0]+Dos[spin][q])+s1)*h;
+    } 
+  } else {
+ 
+    /* Simpson's rule */
+ 
+    for (q=0;q<Dos_N;q++) {
+      s1 = 0.0;
+      s2 = 0.0;
+      for (ie=1;ie<q;ie+=2) {
+        s1 += Dos[spin][ie];
+      } 
+      for (ie=2;ie<q;ie+=2) {
+        s2 += Dos[spin][ie];
+      } 
+      ssum[spin][q] = (Dos[spin][0]+4.0*s1+2.0*s2+Dos[spin][q])*h/3.0;
+    } 
+  } 
+    
   } /* spin */
  
   /* save */
 
   if (SpinP_switch==1) {
     for (ie=0; ie<Dos_N; ie++) {
-      fprintf(fp_Dos,"%lf %lf %lf\n", (DosE[ie])*eV2Hartree, Dos[0][ie], -Dos[1][ie]);
+      fprintf(fp_Dos,"%lf %lf %lf %lf %lf\n", (DosE[ie])*eV2Hartree, Dos[0][ie], -Dos[1][ie], ssum[0][ie], ssum[1][ie]);
     }
   }
   else {
     for (ie=0; ie<Dos_N; ie++) {
-      fprintf(fp_Dos,"%lf %lf\n", (DosE[ie])*eV2Hartree, Dos[0][ie]*2.0);
+      fprintf(fp_Dos,"%lf %lf %lf\n", (DosE[ie])*eV2Hartree, Dos[0][ie]*2.0, ssum[0][ie]*2.0);
     }
   }
 
@@ -575,6 +695,12 @@ void Dos_Tetrahedron( char *basename, int Dos_N,
   FILE *fp_Dos;
   char fp_buf[fp_bsize];          /* setvbuf */
 
+  /* sawada */
+
+  int q, sw;
+  double h,s1,s2;
+  double ssum[2][Dos_N];
+
   printf("<Dos_Tetrahedron> start\n");
 
   if (strlen(basename)==0) {
@@ -669,16 +795,50 @@ void Dos_Tetrahedron( char *basename, int Dos_N,
     for (ie=0;ie<Dos_N;ie++) {
       Dos[spin][ie] = Dos[spin][ie] * factor;
     }
+
+  /* sawada */
+   
+  h = (DosE[Dos_N-1] - DosE[0])/(Dos_N-1)*eV2Hartree;
+  sw = 2; 
+   
+  if (sw == 1) {
+ 
+    /* Trapezoidal rule */
+ 
+    for (q=0;q<Dos_N;q++) {
+      s1 = 0.0;
+      for (ie=1;ie<q;ie++) {
+        s1 += Dos[spin][ie];
+      }   
+      ssum[spin][q] = (0.5*(Dos[spin][0]+Dos[spin][q])+s1)*h;
+    } 
+  } else {
+ 
+    /* Simpson's rule */
+ 
+    for (q=0;q<Dos_N;q++) {
+      s1 = 0.0;
+      s2 = 0.0;
+      for (ie=1;ie<q;ie+=2) {
+        s1 += Dos[spin][ie];
+      } 
+      for (ie=2;ie<q;ie+=2) {
+        s2 += Dos[spin][ie];
+      } 
+      ssum[spin][q] = (Dos[spin][0]+4.0*s1+2.0*s2+Dos[spin][q])*h/3.0;
+    } 
+  } 
+
   }
 
   if (SpinP_switch==1) {
     for (ie=0;ie<Dos_N;ie++) {
-      fprintf(fp_Dos,"%lf %lf %lf\n", (DosE[ie])*eV2Hartree, Dos[0][ie], -Dos[1][ie]);
+      fprintf(fp_Dos,"%lf %lf %lf %lf %lf\n", (DosE[ie])*eV2Hartree, Dos[0][ie], -Dos[1][ie], ssum[0][ie], ssum[1][ie]);
     }
   } 
   else {
     for (ie=0;ie<Dos_N;ie++) {
-      fprintf(fp_Dos,"%lf %lf\n", (DosE[ie])*eV2Hartree, Dos[0][ie]*2.0);
+      fprintf(fp_Dos,"%lf %lf %lf\n", (DosE[ie])*eV2Hartree, Dos[0][ie]*2.0, ssum[0][ie]*2.0);
     }
   }
 
@@ -729,6 +889,12 @@ void Spectra_Gaussian( int pdos_n, int *pdos_atoms, char *basename, int Dos_N,
   char fp_buf[fp_bsize];          /* setvbuf */
   static char *extension="PDOS.Gaussian";
 
+  /* sawada */
+
+  int q, sw;
+  double h,s1,s2;
+  double ssum[2][Dos_N], Doss[2][Dos_N];
+
   printf("<Spectra_Gaussian> start\n");
 
   pi2=sqrt(PI);
@@ -829,20 +995,78 @@ void Spectra_Gaussian( int pdos_n, int *pdos_atoms, char *basename, int Dos_N,
 	  }
 	  else {
 	    printf("make %s\n",file_Dos);
+
+            /* sawada */ 
+ 
+            h = (DosE[Dos_N-1] - DosE[0])/(Dos_N-1)*eV2Hartree;
+            sw = 2; 
+     
+            for (spin=0;spin<=SpinP_switch;spin++) {
+              for (ie=1;ie<Dos_N;ie++) {
+                Doss[spin][ie] = 0.0;
+              }
+            }
+
+            for (spin=0;spin<=SpinP_switch;spin++) {
+              if (sw == 1) { 
+              /* Trapezoidal rule */ 
+                for (q=0;q<Dos_N;q++) { 
+                  s1 = 0.0; 
+                  for (ie=1;ie<q;ie++) {
+                    Doss[spin][ie] = 0.0;
+                    for (i1=0;i1< tnoA; i1++) {
+                      if (LM== Spe_Num_Relation[wanA][i1]) {
+                        Doss[spin][ie]+=  Dos[spin][GA_AN][i1][ie];
+                      } 
+                    }
+                    s1 += Doss[spin][ie];
+                  } 
+                  ssum[spin][q] = (0.5*(Doss[spin][0]+Doss[spin][q])+s1)*h;
+                } 
+              } else { 
+              /* Simpson's rule */
+                for (q=0;q<Dos_N;q++) {
+                  s1 = 0.0;
+                  s2 = 0.0;
+                  for (ie=1;ie<q;ie+=2) {
+                    Doss[spin][ie] = 0.0;
+                    for (i1=0;i1< tnoA; i1++) {
+                      if (LM== Spe_Num_Relation[wanA][i1]) {
+                        Doss[spin][ie]+=  Dos[spin][GA_AN][i1][ie];
+                      }  
+                    }
+                    s1 += Doss[spin][ie];
+                  } 
+                  for (ie=2;ie<q;ie+=2) {
+                    Doss[spin][ie] = 0.0;
+                    for (i1=0;i1< tnoA; i1++) {
+                      if (LM== Spe_Num_Relation[wanA][i1]) {
+                        Doss[spin][ie]+=  Dos[spin][GA_AN][i1][ie];
+                      }  
+                    }
+                    s2 += Doss[spin][ie];
+                  } 
+                  ssum[spin][q] = (Doss[spin][0]+4.0*s1+2.0*s2+Doss[spin][q])*h/3.0;
+                } 
+              }             
+            }
+
 	    for (ie=0;ie<Dos_N;ie++) {
 	      for (spin=0;spin<=SpinP_switch;spin++) dossum[spin]=0.0;
 	      for (i1=0;i1< tnoA; i1++) {
 		if (LM== Spe_Num_Relation[wanA][i1]) {
-		  for (spin=0;spin<=SpinP_switch;spin++) 
+		  for (spin=0;spin<=SpinP_switch;spin++) {
 		    dossum[spin]+=  Dos[spin][GA_AN][i1][ie];
+                  }
 		}
 	      }
+             
 	      if (SpinP_switch==1) {
-		fprintf(fp_Dos,"%lf %lf %lf\n", (DosE[ie])*eV2Hartree,
-                        dossum[0],-dossum[1]);
+		fprintf(fp_Dos,"%lf %lf %lf %lf %lf\n", (DosE[ie])*eV2Hartree,
+                        dossum[0],-dossum[1], ssum[0][ie], ssum[1][ie]);
 	      }
 	      else {
-		fprintf(fp_Dos,"%lf %lf\n", (DosE[ie])*eV2Hartree, dossum[0]*2.0);
+		fprintf(fp_Dos,"%lf %lf %lf\n", (DosE[ie])*eV2Hartree, dossum[0]*2.0, ssum[0][ie]*2.0);
 	      }
 
 	    }
@@ -879,6 +1103,56 @@ void Spectra_Gaussian( int pdos_n, int *pdos_atoms, char *basename, int Dos_N,
       }
       */
 
+      /* sawada */
+  
+      h = (DosE[Dos_N-1] - DosE[0])/(Dos_N-1)*eV2Hartree;
+      sw = 2;
+     
+      for (spin=0;spin<=SpinP_switch;spin++) {
+        for (ie=1;ie<Dos_N;ie++) {
+          Doss[spin][ie] = 0.0;
+        } 
+      } 
+ 
+      for (spin=0;spin<=SpinP_switch;spin++) {
+        if (sw == 1) {
+        /* Trapezoidal rule */
+          for (q=0;q<Dos_N;q++) {
+            s1 = 0.0;
+            for (ie=1;ie<q;ie++) {
+              Doss[spin][ie] = 0.0;
+              for (i1=0;i1< tnoA; i1++) {
+                Doss[spin][ie]+=  Dos[spin][GA_AN][i1][ie];
+              }
+              s1 += Doss[spin][ie];
+            }
+            ssum[spin][q] = (0.5*(Doss[spin][0]+Doss[spin][q])+s1)*h;
+          } 
+        } else {
+        /* Simpson's rule */
+          for (q=0;q<Dos_N;q++) {
+            s1 = 0.0;
+            s2 = 0.0;
+            for (ie=1;ie<q;ie+=2) {
+              Doss[spin][ie] = 0.0;
+              for (i1=0;i1< tnoA; i1++) {
+                Doss[spin][ie]+=  Dos[spin][GA_AN][i1][ie];
+              }
+              s1 += Doss[spin][ie];
+            }
+            for (ie=2;ie<q;ie+=2) {
+              Doss[spin][ie] = 0.0;
+              for (i1=0;i1< tnoA; i1++) {
+                Doss[spin][ie]+=  Dos[spin][GA_AN][i1][ie];
+              }
+              s2 += Doss[spin][ie];
+            }
+            ssum[spin][q] = (Doss[spin][0]+4.0*s1+2.0*s2+Doss[spin][q])*h/3.0;
+          } 
+        }   
+      }
+
+
       for (ie=0;ie<Dos_N;ie++) {
 	for (spin=0;spin<=SpinP_switch;spin++) dossum[spin]=0.0;
 	for (i1=0;i1< tnoA; i1++) {
@@ -897,11 +1171,11 @@ void Spectra_Gaussian( int pdos_n, int *pdos_atoms, char *basename, int Dos_N,
 	*/
 
 	if (SpinP_switch==1) {
-	  fprintf(fp_Dos,"%lf %lf %lf\n", (DosE[ie])*eV2Hartree,
-		  dossum[0],-dossum[1]);
+	  fprintf(fp_Dos,"%lf %lf %lf %lf %lf\n", (DosE[ie])*eV2Hartree,
+		  dossum[0],-dossum[1], ssum[0][ie], ssum[1][ie]);
 	}
 	else {
-	  fprintf(fp_Dos,"%lf %lf\n", (DosE[ie])*eV2Hartree, dossum[0]*2.0);
+	  fprintf(fp_Dos,"%lf %lf %lf\n", (DosE[ie])*eV2Hartree, dossum[0]*2.0, ssum[0][ie]*2.0);
 	}
 
       } /* ie */
@@ -973,6 +1247,12 @@ void SpectraDC_Gaussian( int pdos_n, int *pdos_atoms, char *basename, int Dos_N,
   char fp_buf[fp_bsize];          /* setvbuf */
   static char *extension="PDOS.Gaussian";
 
+  /* sawada */
+
+  int q, sw;
+  double h,s1,s2;
+  double ssum[2][Dos_N], Doss[2][Dos_N];
+
   printf("<Spectra_Gaussian> start\n");
 
   pi2=sqrt(PI);
@@ -1066,6 +1346,62 @@ void SpectraDC_Gaussian( int pdos_n, int *pdos_atoms, char *basename, int Dos_N,
 	  }
 	  else {
 	    printf("make %s\n",file_Dos);
+
+            /* sawada */
+
+            h = (DosE[Dos_N-1] - DosE[0])/(Dos_N-1)*eV2Hartree;
+            sw = 2;
+
+            for (spin=0;spin<=SpinP_switch;spin++) {
+              for (ie=1;ie<Dos_N;ie++) {
+                Doss[spin][ie] = 0.0;
+              }
+            }
+
+            for (spin=0;spin<=SpinP_switch;spin++) {
+              if (sw == 1) {
+              /* Trapezoidal rule */
+                for (q=0;q<Dos_N;q++) {
+                  s1 = 0.0;
+                  for (ie=1;ie<q;ie++) {
+                    Doss[spin][ie] = 0.0;
+                    for (i1=0;i1< tnoA; i1++) {
+                      if (LM== Spe_Num_Relation[wanA][i1]) {
+                        Doss[spin][ie]+=  Dos[spin][GA_AN][i1][ie];
+                      }
+                    }
+                    s1 += Doss[spin][ie];
+                  } 
+                  ssum[spin][q] = (0.5*(Doss[spin][0]+Doss[spin][q])+s1)*h;
+                } 
+              } else { 
+              /* Simpson's rule */
+                for (q=0;q<Dos_N;q++) {
+                  s1 = 0.0;
+                  s2 = 0.0;
+                  for (ie=1;ie<q;ie+=2) {
+                    Doss[spin][ie] = 0.0;
+                    for (i1=0;i1< tnoA; i1++) {
+                      if (LM== Spe_Num_Relation[wanA][i1]) {
+                        Doss[spin][ie]+=  Dos[spin][GA_AN][i1][ie];
+                      }
+                    }
+                    s1 += Doss[spin][ie];
+                  } 
+                  for (ie=2;ie<q;ie+=2) {
+                    Doss[spin][ie] = 0.0;
+                    for (i1=0;i1< tnoA; i1++) {
+                      if (LM== Spe_Num_Relation[wanA][i1]) {
+                        Doss[spin][ie]+=  Dos[spin][GA_AN][i1][ie];
+                      }
+                    }
+                    s2 += Doss[spin][ie];
+                  } 
+                  ssum[spin][q] = (Doss[spin][0]+4.0*s1+2.0*s2+Doss[spin][q])*h/3.0;
+                }
+              }
+            }         
+
 	    for (ie=0;ie<Dos_N;ie++) {
 	      for (spin=0;spin<=SpinP_switch;spin++) dossum[spin]=0.0;
 	      for (i1=0;i1< tnoA; i1++) {
@@ -1075,11 +1411,11 @@ void SpectraDC_Gaussian( int pdos_n, int *pdos_atoms, char *basename, int Dos_N,
 		}
 	      }
 	      if (SpinP_switch==1) {
-		fprintf(fp_Dos,"%lf %lf %lf\n", (DosE[ie])*eV2Hartree,
-                        dossum[0],-dossum[1]);
+		fprintf(fp_Dos,"%lf %lf %lf %lf %lf\n", (DosE[ie])*eV2Hartree,
+                        dossum[0],-dossum[1], ssum[0][ie], ssum[1][ie]);
 	      }
 	      else {
-		fprintf(fp_Dos,"%lf %lf\n", (DosE[ie])*eV2Hartree, dossum[0]*2.0);
+		fprintf(fp_Dos,"%lf %lf %lf\n", (DosE[ie])*eV2Hartree, dossum[0]*2.0, ssum[0][ie]*2.0);
 	      }
 
 	    }
@@ -1107,6 +1443,56 @@ void SpectraDC_Gaussian( int pdos_n, int *pdos_atoms, char *basename, int Dos_N,
     }
     else {
       printf("make %s\n",file_Dos);
+
+      /* sawada */
+
+            h = (DosE[Dos_N-1] - DosE[0])/(Dos_N-1)*eV2Hartree;
+            sw = 2;
+
+            for (spin=0;spin<=SpinP_switch;spin++) {
+              for (ie=1;ie<Dos_N;ie++) {
+                Doss[spin][ie] = 0.0;
+              }
+            }
+
+            for (spin=0;spin<=SpinP_switch;spin++) {
+              if (sw == 1) {
+              /* Trapezoidal rule */
+                for (q=0;q<Dos_N;q++) {
+                  s1 = 0.0;
+                  for (ie=1;ie<q;ie++) {
+                    Doss[spin][ie] = 0.0;
+                    for (i1=0;i1< tnoA; i1++) {
+                      Doss[spin][ie]+=  Dos[spin][GA_AN][i1][ie];
+                    }
+                    s1 += Doss[spin][ie];
+                  } 
+                  ssum[spin][q] = (0.5*(Doss[spin][0]+Doss[spin][q])+s1)*h;
+                } 
+              } else { 
+              /* Simpson's rule */
+                for (q=0;q<Dos_N;q++) {
+                  s1 = 0.0;
+                  s2 = 0.0;
+                  for (ie=1;ie<q;ie+=2) {
+                    Doss[spin][ie] = 0.0;
+                    for (i1=0;i1< tnoA; i1++) {
+                      Doss[spin][ie]+=  Dos[spin][GA_AN][i1][ie];
+                    }
+                    s1 += Doss[spin][ie];
+                  } 
+                  for (ie=2;ie<q;ie+=2) {
+                    Doss[spin][ie] = 0.0;
+                    for (i1=0;i1< tnoA; i1++) {
+                      Doss[spin][ie]+=  Dos[spin][GA_AN][i1][ie];
+                    }
+                    s2 += Doss[spin][ie];
+                  } 
+                  ssum[spin][q] = (Doss[spin][0]+4.0*s1+2.0*s2+Doss[spin][q])*h/3.0;
+                }
+              }
+            } 
+ 
       for (ie=0;ie<Dos_N;ie++) {
 	for (spin=0;spin<=SpinP_switch;spin++) dossum[spin]=0.0;
 	for (i1=0;i1< tnoA; i1++) {
@@ -1114,11 +1500,11 @@ void SpectraDC_Gaussian( int pdos_n, int *pdos_atoms, char *basename, int Dos_N,
 	    dossum[spin]+=  Dos[spin][iatom][i1][ie];
 	}
 	if (SpinP_switch==1) {
-	  fprintf(fp_Dos,"%lf %lf %lf\n", (DosE[ie])*eV2Hartree,
-		  dossum[0],-dossum[1]);
+	  fprintf(fp_Dos,"%lf %lf %lf %lf %lf\n", (DosE[ie])*eV2Hartree,
+		  dossum[0],-dossum[1], ssum[0][ie], ssum[1][ie]);
 	}
 	else {
-	  fprintf(fp_Dos,"%lf %lf\n", (DosE[ie])*eV2Hartree, dossum[0]*2.0);
+	  fprintf(fp_Dos,"%lf %lf %lf\n", (DosE[ie])*eV2Hartree, dossum[0]*2.0, ssum[0][ie]*2.0);
 	}
 
       } /* ie */
@@ -1177,6 +1563,12 @@ void Spectra_Tetrahedron( int pdos_n, int *pdos_atoms, char *basename, int Dos_N
   FILE *fp_Dos;
   char fp_buf[fp_bsize];          /* setvbuf */
 
+  /* sawada */
+
+  int q, sw;
+  double h,s1,s2;
+  double ssum[2][Dos_N], Doss[2][Dos_N];
+
   static char *extension="PDOS.Tetrahedron";
 
   printf("<Spectra_Tetrahedron> start\n");
@@ -1361,6 +1753,7 @@ void Spectra_Tetrahedron( int pdos_n, int *pdos_atoms, char *basename, int Dos_N
   }
 #else
 
+  /*
   for (iatom=0;iatom<pdos_n;iatom++) {
     GA_AN = pdos_atoms[iatom]-1;
     wanA = WhatSpecies[GA_AN];
@@ -1369,6 +1762,7 @@ void Spectra_Tetrahedron( int pdos_n, int *pdos_atoms, char *basename, int Dos_N
       printf("Spe_Num_Relation %d %d %d\n",wanA,i1,Spe_Num_Relation[wanA][i1]);
     }
   }
+  */
 
   /* orbital projection */
   for (iatom=0;iatom<pdos_n;iatom++) {
@@ -1391,6 +1785,62 @@ void Spectra_Tetrahedron( int pdos_n, int *pdos_atoms, char *basename, int Dos_N
 	  }
 	  else {
 	    printf("make %s\n",file_Dos);
+
+            /* sawada */
+ 
+            h = (DosE[Dos_N-1] - DosE[0])/(Dos_N-1)*eV2Hartree;
+            sw = 2;
+ 
+            for (spin=0;spin<=SpinP_switch;spin++) {
+              for (ie=1;ie<Dos_N;ie++) {
+                Doss[spin][ie] = 0.0;
+              }
+            }
+ 
+            for (spin=0;spin<=SpinP_switch;spin++) {
+              if (sw == 1) {
+              /* Trapezoidal rule */
+                for (q=0;q<Dos_N;q++) {
+                  s1 = 0.0;
+                  for (ie=1;ie<q;ie++) {
+                    Doss[spin][ie] = 0.0;
+                    for (i1=0;i1< tnoA; i1++) {
+                      if (LM== Spe_Num_Relation[wanA][i1]) {
+                        Doss[spin][ie]+=  Dos[spin][GA_AN][i1][ie];
+                      }
+                    }
+                    s1 += Doss[spin][ie];
+                  }
+                  ssum[spin][q] = (0.5*(Doss[spin][0]+Doss[spin][q])+s1)*h;
+                }
+              } else {
+              /* Simpson's rule */
+                for (q=0;q<Dos_N;q++) {
+                  s1 = 0.0;
+                  s2 = 0.0;
+                  for (ie=1;ie<q;ie+=2) {
+                    Doss[spin][ie] = 0.0;
+                    for (i1=0;i1< tnoA; i1++) {
+                      if (LM== Spe_Num_Relation[wanA][i1]) {
+                        Doss[spin][ie]+=  Dos[spin][GA_AN][i1][ie];
+                      }
+                    }
+                    s1 += Doss[spin][ie];
+                  }
+                  for (ie=2;ie<q;ie+=2) {
+                    Doss[spin][ie] = 0.0;
+                    for (i1=0;i1< tnoA; i1++) {
+                      if (LM== Spe_Num_Relation[wanA][i1]) {
+                        Doss[spin][ie]+=  Dos[spin][GA_AN][i1][ie];
+                      }
+                    }
+                    s2 += Doss[spin][ie];
+                  }
+                  ssum[spin][q] = (Doss[spin][0]+4.0*s1+2.0*s2+Doss[spin][q])*h/3.0;
+                }
+              }
+            }
+           
 	    for (ie=0;ie<Dos_N;ie++) {
 	      for (spin=0;spin<=SpinP_switch;spin++) dossum[spin]=0.0;
 	      for (i1=0;i1< tnoA; i1++) {
@@ -1404,14 +1854,14 @@ void Spectra_Tetrahedron( int pdos_n, int *pdos_atoms, char *basename, int Dos_N
 		}
 	      }
 	      if (SpinP_switch==1) {
-		fprintf(fp_Dos,"%lf %lf %lf\n", 
+		fprintf(fp_Dos,"%lf %lf %lf %lf %lf\n", 
                         (DosE[ie])*eV2Hartree, 
-			dossum[0],-dossum[1]);
+			dossum[0],-dossum[1],ssum[0][ie],ssum[1][ie]);
 	      }
 	      else {
-		fprintf(fp_Dos,"%lf %lf\n", 
+		fprintf(fp_Dos,"%lf %lf %lf\n", 
                         (DosE[ie])*eV2Hartree,
-			dossum[0]*2.0);
+			dossum[0]*2.0,ssum[0][ie]*2.0);
 	      }
 
 	    }
@@ -1439,6 +1889,56 @@ void Spectra_Tetrahedron( int pdos_n, int *pdos_atoms, char *basename, int Dos_N
     }
     else {
       printf("make %s\n",file_Dos);
+
+      /* sawada */
+ 
+            h = (DosE[Dos_N-1] - DosE[0])/(Dos_N-1)*eV2Hartree;
+            sw = 2;
+ 
+            for (spin=0;spin<=SpinP_switch;spin++) {
+              for (ie=1;ie<Dos_N;ie++) {
+                Doss[spin][ie] = 0.0;
+              }
+            }
+ 
+            for (spin=0;spin<=SpinP_switch;spin++) {
+              if (sw == 1) {
+              /* Trapezoidal rule */
+                for (q=0;q<Dos_N;q++) {
+                  s1 = 0.0;
+                  for (ie=1;ie<q;ie++) {
+                    Doss[spin][ie] = 0.0;
+                    for (i1=0;i1< tnoA; i1++) {
+                      Doss[spin][ie]+=  Dos[spin][GA_AN][i1][ie];
+                    }
+                    s1 += Doss[spin][ie];
+                  }
+                  ssum[spin][q] = (0.5*(Doss[spin][0]+Doss[spin][q])+s1)*h;
+                }
+              } else {
+              /* Simpson's rule */
+                for (q=0;q<Dos_N;q++) {
+                  s1 = 0.0;
+                  s2 = 0.0;
+                  for (ie=1;ie<q;ie+=2) {
+                    Doss[spin][ie] = 0.0;
+                    for (i1=0;i1< tnoA; i1++) {
+                      Doss[spin][ie]+=  Dos[spin][GA_AN][i1][ie];
+                    }
+                    s1 += Doss[spin][ie];
+                  }
+                  for (ie=2;ie<q;ie+=2) {
+                    Doss[spin][ie] = 0.0;
+                    for (i1=0;i1< tnoA; i1++) {
+                      Doss[spin][ie]+=  Dos[spin][GA_AN][i1][ie];
+                    }
+                    s2 += Doss[spin][ie];
+                  }
+                  ssum[spin][q] = (Doss[spin][0]+4.0*s1+2.0*s2+Doss[spin][q])*h/3.0;
+                }
+              }
+            }
+  
       for (ie=0;ie<Dos_N;ie++) {
 	for (spin=0;spin<=SpinP_switch;spin++) dossum[spin]=0.0;
 	for (i1=0;i1< tnoA; i1++) {
@@ -1446,12 +1946,12 @@ void Spectra_Tetrahedron( int pdos_n, int *pdos_atoms, char *basename, int Dos_N
 	    dossum[spin]+=  Dos[spin][GA_AN][i1][ie];
 	}
 	if (SpinP_switch==1) {
-	  fprintf(fp_Dos,"%lf %lf %lf\n", (DosE[ie])*eV2Hartree,
-		  dossum[0],dossum[1]);
+	  fprintf(fp_Dos,"%lf %lf %lf %lf %lf\n", (DosE[ie])*eV2Hartree,
+		  dossum[0],dossum[1],ssum[0][ie],ssum[1][ie]);
 	}
 	else {
-	  fprintf(fp_Dos,"%lf %lf\n", (DosE[ie])*eV2Hartree,
-		  dossum[0]*2.0);
+	  fprintf(fp_Dos,"%lf %lf %lf\n", (DosE[ie])*eV2Hartree,
+		  dossum[0]*2.0, ssum[0][ie]*2.0);
 	}
 
       } /* ie */
@@ -1525,6 +2025,12 @@ void Spectra_NEGF( int pdos_n, int *pdos_atoms, char *basename, char *extension,
   FILE *fp_Dos;
   char fp_buf[fp_bsize];          /* setvbuf */
 
+  /* sawada */
+
+  int q, sw;
+  double h,s1,s2;
+  double ssum[2][Dos_N], Doss[2][Dos_N];
+
   printf("<Spectra_%s> start\n",extension);
 
   Max_tnoA=0;
@@ -1592,20 +2098,77 @@ void Spectra_NEGF( int pdos_n, int *pdos_atoms, char *basename, char *extension,
 
 	  else {
 	    printf("make %s\n",file_Dos);
+
+            /* sawada */
+ 
+            h = (DosE[Dos_N-1] - DosE[0])/(Dos_N-1)*eV2Hartree;
+            sw = 2;
+ 
+            for (spin=0;spin<=SpinP_switch;spin++) {
+              for (ie=1;ie<Dos_N;ie++) {
+                Doss[spin][ie] = 0.0;
+              }
+            }
+ 
+            for (spin=0;spin<=SpinP_switch;spin++) {
+              if (sw == 1) {
+              /* Trapezoidal rule */
+                for (q=0;q<Dos_N;q++) {
+                  s1 = 0.0;
+                  for (ie=1;ie<q;ie++) {
+                    Doss[spin][ie] = 0.0;
+                    for (i1=0;i1< tnoA; i1++) {
+                      if (LM== Spe_Num_Relation[wanA][i1]) {
+                        Doss[spin][ie]+=  Dos[spin][GA_AN][i1][ie];
+                      }
+                    }
+                    s1 += Doss[spin][ie];
+                  }
+                  ssum[spin][q] = (0.5*(Doss[spin][0]+Doss[spin][q])+s1)*h;
+                }
+              } else {
+              /* Simpson's rule */
+                for (q=0;q<Dos_N;q++) {
+                  s1 = 0.0;
+                  s2 = 0.0;
+                  for (ie=1;ie<q;ie+=2) {
+                    Doss[spin][ie] = 0.0;
+                    for (i1=0;i1< tnoA; i1++) {
+                      if (LM== Spe_Num_Relation[wanA][i1]) {
+                        Doss[spin][ie]+=  Dos[spin][GA_AN][i1][ie];
+                      }
+                    }
+                    s1 += Doss[spin][ie];
+                  }
+                  for (ie=2;ie<q;ie+=2) {
+                    Doss[spin][ie] = 0.0;
+                    for (i1=0;i1< tnoA; i1++) {
+                      if (LM== Spe_Num_Relation[wanA][i1]) {
+                        Doss[spin][ie]+=  Dos[spin][GA_AN][i1][ie];
+                      }
+                    }
+                    s2 += Doss[spin][ie];
+                  }
+                  ssum[spin][q] = (Doss[spin][0]+4.0*s1+2.0*s2+Doss[spin][q])*h/3.0;
+                }
+              }
+            }
+            
 	    for (ie=0;ie<Dos_N;ie++) {
 	      for (spin=0;spin<=SpinP_switch;spin++) dossum[spin]=0.0;
 	      for (i1=0;i1< tnoA; i1++) {
 		if (LM== Spe_Num_Relation[wanA][i1]) {
-		  for (spin=0;spin<=SpinP_switch;spin++) 
+		  for (spin=0;spin<=SpinP_switch;spin++) {
 		    dossum[spin]+=  Dos[spin][GA_AN][i1][ie];
+                  }
 		}
 	      }
 	      if (SpinP_switch==1) {
-		fprintf(fp_Dos,"%lf %lf %lf\n", (DosE[ie])*eV2Hartree,
-                        dossum[0],-dossum[1]);
+		fprintf(fp_Dos,"%lf %lf %lf %lf %lf\n", (DosE[ie])*eV2Hartree,
+                        dossum[0],-dossum[1],ssum[0][ie],ssum[1][ie]);
 	      }
 	      else {
-		fprintf(fp_Dos,"%lf %lf\n", (DosE[ie])*eV2Hartree, dossum[0]*2.0);
+		fprintf(fp_Dos,"%lf %lf %lf\n", (DosE[ie])*eV2Hartree, dossum[0]*2.0, ssum[0][ie]*2.0);
 	      }
 
 	    }
@@ -1642,6 +2205,55 @@ void Spectra_NEGF( int pdos_n, int *pdos_atoms, char *basename, char *extension,
       }
       */
 
+      /* sawada */
+ 
+            h = (DosE[Dos_N-1] - DosE[0])/(Dos_N-1)*eV2Hartree;
+            sw = 2;
+ 
+            for (spin=0;spin<=SpinP_switch;spin++) {
+              for (ie=1;ie<Dos_N;ie++) {
+                Doss[spin][ie] = 0.0;
+              }
+            }
+ 
+            for (spin=0;spin<=SpinP_switch;spin++) {
+              if (sw == 1) {
+              /* Trapezoidal rule */
+                for (q=0;q<Dos_N;q++) {
+                  s1 = 0.0;
+                  for (ie=1;ie<q;ie++) {
+                    Doss[spin][ie] = 0.0;
+                    for (i1=0;i1< tnoA; i1++) {
+                      Doss[spin][ie]+=  Dos[spin][GA_AN][i1][ie];
+                    }
+                    s1 += Doss[spin][ie];
+                  }
+                  ssum[spin][q] = (0.5*(Doss[spin][0]+Doss[spin][q])+s1)*h;
+                }
+              } else {
+              /* Simpson's rule */
+                for (q=0;q<Dos_N;q++) {
+                  s1 = 0.0;
+                  s2 = 0.0;
+                  for (ie=1;ie<q;ie+=2) {
+                    Doss[spin][ie] = 0.0;
+                    for (i1=0;i1< tnoA; i1++) {
+                      Doss[spin][ie]+=  Dos[spin][GA_AN][i1][ie];
+                    }
+                    s1 += Doss[spin][ie];
+                  }
+                  for (ie=2;ie<q;ie+=2) {
+                    Doss[spin][ie] = 0.0;
+                    for (i1=0;i1< tnoA; i1++) {
+                      Doss[spin][ie]+=  Dos[spin][GA_AN][i1][ie];
+                    }
+                    s2 += Doss[spin][ie];
+                  }
+                  ssum[spin][q] = (Doss[spin][0]+4.0*s1+2.0*s2+Doss[spin][q])*h/3.0;
+                }
+              }
+            }
+
       for (ie=0;ie<Dos_N;ie++) {
 	for (spin=0;spin<=SpinP_switch;spin++) dossum[spin]=0.0;
 	for (i1=0;i1< tnoA; i1++) {
@@ -1660,11 +2272,11 @@ void Spectra_NEGF( int pdos_n, int *pdos_atoms, char *basename, char *extension,
 	*/
 
 	if (SpinP_switch==1) {
-	  fprintf(fp_Dos,"%lf %lf %lf\n", (DosE[ie])*eV2Hartree,
-		  dossum[0],-dossum[1]);
+	  fprintf(fp_Dos,"%lf %lf %lf %lf %lf\n", (DosE[ie])*eV2Hartree,
+		  dossum[0],-dossum[1],ssum[0][ie],ssum[1][ie]);
 	}
 	else {
-	  fprintf(fp_Dos,"%lf %lf\n", (DosE[ie])*eV2Hartree, dossum[0]*2.0);
+	  fprintf(fp_Dos,"%lf %lf %lf\n", (DosE[ie])*eV2Hartree, dossum[0]*2.0, ssum[0][ie]*2.0);
 	}
 
       } /* ie */
