@@ -16,56 +16,268 @@
 #include <stdlib.h>
 #include "openmx_common.h"
 #include "lapack_prototypes.h"
-
-
-#ifdef nompi
-#include "mimic_mpi.h"
-#else
 #include "mpi.h"
-#endif
+#include <omp.h>
+
+
  
 #define  measure_time   0
 
 
-static void Eigen_Original_PReHH(MPI_Comm MPI_Curret_Comm_WD, 
+static void Eigen_Original_PReHH(MPI_Comm MPI_Current_Comm_WD, 
 			 double **ac, double *ko, int n, int EVmax, int bcast_flag);
 
-static void Eigen_Improved_PReHH(MPI_Comm MPI_Curret_Comm_WD, 
+static void Eigen_Improved_PReHH(MPI_Comm MPI_Current_Comm_WD, 
 			double **ac, double *ko, int n, int EVmax, int bcast_flag);
 
-static void myHH( MPI_Comm MPI_Curret_Comm_WD, int numprocs, int myid, int n, double **ac, double **ad, 
+static void Eigen_ELPA1_Re(MPI_Comm MPI_Current_Comm_WD, 
+			double **ac, double *ko, int n, int EVmax, int bcast_flag);
+
+static void myHH( MPI_Comm MPI_Current_Comm_WD, int numprocs, int myid, int n, double **ac, double **ad, 
 		  double *uu, double *b1, 
 		  MPI_Request *request_send, 
 		  MPI_Request *request_recv, 
 		  MPI_Status *stat_send);
 
-static void call_dsytrd( MPI_Comm MPI_Curret_Comm_WD, int numprocs, int myid, int n, double **ac, double **ad, 
+static void call_dsytrd( MPI_Comm MPI_Current_Comm_WD, int numprocs, int myid, int n, double **ac, double **ad, 
 			 double *uu, double *p, double *q, double *b1, 
 			 MPI_Request *request_send, 
 			 MPI_Request *request_recv, 
 			 MPI_Status *stat_send);
 
+static int numrocC(int N, int NB, int IPROC, int ISRCPROC, int NPROCS);
 
-void Eigen_PReHH(MPI_Comm MPI_Curret_Comm_WD, 
+void Eigen_PReHH(MPI_Comm MPI_Current_Comm_WD, 
                  double **ac, double *ko, int n, int EVmax, int bcast_flag)
 {
-  Eigen_Improved_PReHH(MPI_Curret_Comm_WD, ac, ko, n, EVmax, bcast_flag); 
+
+  if      (scf_eigen_lib_flag==0 || n<100)
+    Eigen_Improved_PReHH(MPI_Current_Comm_WD, ac, ko, n, EVmax, bcast_flag); 
+
+  else if (scf_eigen_lib_flag==1)
+    Eigen_ELPA1_Re(MPI_Current_Comm_WD, ac, ko, n, EVmax, bcast_flag); 
+
 }
 
 
 
+void Eigen_ELPA1_Re(MPI_Comm MPI_Current_Comm_WD, 
+                    double **ac, double *ko, int n, int EVmax, int bcast_flag)
+{
+
+ /*
+   !-------------------------------------------------------------------------------
+   ! na:   System size (of the global matrix)
+   ! nev:  Number of eigenvectors to be calculated
+   ! nblk: Blocking factor in block cyclic distribution
+   !-------------------------------------------------------------------------------
+ */
+
+  int na = n;
+  int nev = EVmax;
+  int nblk = 16;
+
+  int np_rows, np_cols, na_rows, na_cols;
+  int myid, numprocs, my_prow, my_pcol;
+  MPI_Comm mpi_comm_rows, mpi_comm_cols;
+  int mpi_comm_rows_int,mpi_comm_cols_int;
+  int i, j, my_blacs_ctxt, info, nprow, npcol, ig, jg;
+  int zero=0, one=1, LOCr, LOCc, node, irow, icol, mpiworld;
+  double *a, *z, *lz;
+  MPI_Status *stat_send;
+
+  MPI_Comm_size(MPI_Current_Comm_WD,&numprocs);
+  MPI_Comm_rank(MPI_Current_Comm_WD,&myid);
+
+  stat_send = (MPI_Status*)malloc(sizeof(MPI_Status)*numprocs);
+  mpiworld = MPI_Comm_c2f(MPI_Current_Comm_WD);
+
+  /*
+  printf("MPI_Current_Comm_WD=%2d mpiworld=%2d\n",MPI_Current_Comm_WD,mpiworld);
+  */
+
+  /*
+   !-------------------------------------------------------------------------------
+   ! Selection of number of processor rows/columns
+   ! We try to set up the grid square-like, i.e. start the search for possible
+   ! divisors of nprocs with a number next to the square root of nprocs
+   ! and decrement it until a divisor is found.
+  */
+
+  np_cols=(int)(sqrt((float)numprocs));
+  do{
+    if((numprocs%np_cols)==0) break;
+    np_cols--;
+  } while(np_cols>=2);
+
+  np_rows = numprocs/np_cols;
+
+  my_prow = myid/np_cols;
+  my_pcol = myid%np_cols;
+
+   /*
+   !-------------------------------------------------------------------------------
+   ! Set up BLACS context and MPI communicators
+   ! For ELPA, the MPI communicators along rows/cols are sufficient,
+   ! and the grid setup may be done in an arbitrary way as long as it is
+   ! consistent (i.e. 0<=my_prow<np_rows, 0<=my_pcol<np_cols and every
+   ! process has a unique (my_prow,my_pcol) pair).
+   ! All ELPA routines need MPI communicators for communicating within
+   ! rows or columns of processes, these are set in get_elpa_row_col_comms.
+   */
+
+  MPI_Comm_split(MPI_Current_Comm_WD,my_pcol,my_prow,&mpi_comm_rows);
+  MPI_Comm_split(MPI_Current_Comm_WD,my_prow,my_pcol,&mpi_comm_cols);
+
+  /*
+  printf("QQQ1 myid=%2d Number of processor rows=%d, cols=%d, total=%d\n",myid,np_rows,np_cols,numprocs);
+  printf("QQQ2 myid=%d, my_prow=%d, my_pcol=%d mpi_comm_rows=%d mpi_comm_cols=%d\n",
+          myid,my_prow,my_pcol,mpi_comm_rows,mpi_comm_cols);
+  */
+
+  /*
+    Determine the necessary size of the distributed matrices,
+    we use the Scalapack tools routine NUMROC for that.
+  */
+
+  na_rows = numrocC(na, nblk, my_prow, 0, np_rows);
+  na_cols = numrocC(na, nblk, my_pcol, 0, np_cols);
+
+  /*
+    Set up a scalapack descriptor for the checks below.
+    For ELPA the following restrictions hold:
+    - block sizes in both directions must be identical (args 4+5)
+    - first row and column of the distributed matrix must be on row/col 0/0 (args 6+7)
+    Allocate the local matrices and distribute the matrix elements from the global matrix to 
+    the local matrices using block-cyclic Scalapack distribution 
+  */
+
+  a = (double*)malloc(sizeof(double)*na_rows*na_cols);
+  z = (double*)malloc(sizeof(double)*na_rows*na_cols);
+
+  for(i=0;i<na_rows;i++){
+    for(j=0;j<na_cols;j++){
+      ig = np_rows*nblk*((i)/nblk) + (i)%nblk + ((np_rows+my_prow)%np_rows)*nblk + 1;
+      jg = np_cols*nblk*((j)/nblk) + (j)%nblk + ((np_cols+my_pcol)%np_cols)*nblk + 1;
+      a[j*na_rows+i]=ac[ig][jg];
+    }
+  }
+
+  /* Calculate eigenvalues/eigenvectors with ELPA */
+
+  mpi_comm_rows_int = MPI_Comm_c2f(mpi_comm_rows);
+  mpi_comm_cols_int = MPI_Comm_c2f(mpi_comm_cols);
+
+  solve_evp_real_(&na, &nev, a, &na_rows, &ko[1], z, &na_rows, &nblk, &mpi_comm_rows_int, &mpi_comm_cols_int);
+
+  MPI_Comm_free(&mpi_comm_rows);
+  MPI_Comm_free(&mpi_comm_cols);
+
+  /*
+   The eigenvectors are distributed to the processes using block-cyclic Scalapack distribution 
+   Collect the eigenvectors to the host process
+  */
+
+  if(myid==0){
+
+   for(node=0;node<numprocs;node++){
+
+     if(node==0){
+
+       for(i=0;i<na_rows;i++){
+	 for(j=0;j<na_cols;j++){
+	   ig = np_rows*nblk*((i)/nblk) + (i)%nblk + ((np_rows+my_prow)%np_rows)*nblk + 1;
+	   jg = np_cols*nblk*((j)/nblk) + (j)%nblk + ((np_cols+my_pcol)%np_cols)*nblk + 1;
+	   ac[ig][jg] = z[j*na_rows+i];
+	 }
+       }
+     }
+
+     else{
+         
+       MPI_Recv(&irow, 1, MPI_INT, node, 10, MPI_Current_Comm_WD, stat_send);
+       MPI_Recv(&icol, 1, MPI_INT, node, 20, MPI_Current_Comm_WD, stat_send);
+       MPI_Recv(&LOCr, 1, MPI_INT, node, 40, MPI_Current_Comm_WD, stat_send);
+       MPI_Recv(&LOCc, 1, MPI_INT, node, 50, MPI_Current_Comm_WD, stat_send);
+         
+       lz = (double*)malloc(sizeof(double)*LOCr*LOCc);
+
+       MPI_Recv(lz, LOCr*LOCc, MPI_DOUBLE, node, 30, MPI_Current_Comm_WD, stat_send);
+
+       for(i=0;i<LOCr;i++){
+	 for(j=0;j<LOCc;j++){
+	   ig = np_rows*nblk*((i)/nblk) + (i)%nblk + ((np_rows+irow)%np_rows)*nblk + 1;
+	   jg = np_cols*nblk*((j)/nblk) + (j)%nblk + ((np_cols+icol)%np_cols)*nblk + 1;
+	   ac[ig][jg] = lz[j*LOCr+i];
+	 }
+       }
+       free(lz);
+     }
+   }
+  }
+  else{
+    MPI_Send(&my_prow, 1, MPI_INT, 0, 10, MPI_Current_Comm_WD);
+    MPI_Send(&my_pcol, 1, MPI_INT, 0, 20, MPI_Current_Comm_WD);
+    MPI_Send(&na_rows, 1, MPI_INT, 0, 40, MPI_Current_Comm_WD);
+    MPI_Send(&na_cols, 1, MPI_INT, 0, 50, MPI_Current_Comm_WD);
+    MPI_Send(z, na_rows*na_cols, MPI_DOUBLE, 0, 30, MPI_Current_Comm_WD);
+  } 
+
+  /*
+   ! Broadcast the eigenvectors to all proceses 
+  */
+
+  for(i=1;i<=n;i++){
+    MPI_Bcast(ac[i],n+1,MPI_DOUBLE,0,MPI_Current_Comm_WD);
+  }
+
+  free(a);
+  free(z);
+  free(stat_send);
+}
 
 
+int numrocC(int N, int NB, int IPROC, int ISRCPROC, int NPROCS)
+{
+  int EXTRABLKS, MYDIST, NBLOCKS, NUMROC;
+
+  /* Figure PROC's distance from source process */
+
+  MYDIST = (NPROCS+IPROC-ISRCPROC) % NPROCS;
+
+  /* Figure the total number of whole NB blocks N is split up into */
+
+  NBLOCKS = N / NB;
+
+  /* Figure the minimum number of rows/cols a process can have */
+
+  NUMROC = (NBLOCKS/NPROCS) * NB;
+
+  /* See if there are any extra blocks */
+
+  EXTRABLKS = NBLOCKS % NPROCS;
+
+  /* If I have an extra block */
+
+  if(MYDIST < EXTRABLKS)
+    NUMROC = NUMROC + NB;
+
+  /* If I have last block, it may be a partial block */
+
+  else if(MYDIST==EXTRABLKS)
+    NUMROC = NUMROC +  (N % NB);
+
+  return NUMROC;
+}
 
 
-
-
-
-void Eigen_Improved_PReHH(MPI_Comm MPI_Curret_Comm_WD, 
+#pragma optimization_level 1
+void Eigen_Improved_PReHH(MPI_Comm MPI_Current_Comm_WD, 
                           double **ac, double *ko, int n, int EVmax, int bcast_flag)
 {
   double ABSTOL=1.0e-13;
-  double **ad,*b1,*uu,*q,*p,
+  double *q,*p;
+  double **ad,*b1,*uu,
     s1,s2,s3,ss,u1,u2,r,p1,my_r,my_s2, 
     xsum,bunbo,si,co,sum,
     a1,a2,a3,a4,a5,a6,b7,r1,r2,
@@ -94,8 +306,8 @@ void Eigen_Improved_PReHH(MPI_Comm MPI_Curret_Comm_WD,
   MPI_Request *request_recv;
 
   /* MPI */
-  MPI_Comm_size(MPI_Curret_Comm_WD,&numprocs);
-  MPI_Comm_rank(MPI_Curret_Comm_WD,&myid);
+  MPI_Comm_size(MPI_Current_Comm_WD,&numprocs);
+  MPI_Comm_rank(MPI_Current_Comm_WD,&myid);
 
   stat_send = malloc(sizeof(MPI_Status)*numprocs);
   request_send = malloc(sizeof(MPI_Request)*numprocs);
@@ -162,40 +374,12 @@ void Eigen_Improved_PReHH(MPI_Comm MPI_Curret_Comm_WD,
                    Householder method
   ****************************************************/
 
-  myHH(MPI_Curret_Comm_WD, numprocs, myid, n, ac, ad, uu, b1, request_send, request_recv, stat_send);
+  myHH(MPI_Current_Comm_WD, numprocs, myid, n, ac, ad, uu, b1, request_send, request_recv, stat_send);
 
   /*
-  call_dsytrd(MPI_Curret_Comm_WD, numprocs, myid, n, ac, ad, uu, p, q, b1, request_send, request_recv, stat_send);
+  call_dsytrd(MPI_Current_Comm_WD, numprocs, myid, n, ac, ad, uu, p, q, b1, request_send, request_recv, stat_send);
   */
 
-  /*
-  printf("ad\n");
-  for (i=1; i<=n; i++){
-    for (j=1; j<=n; j++){
-      printf("%10.5f ",ad[i][j]);
-    }
-    printf("\n");
-  }
-
-  printf("uu\n");
-  for (i=1; i<=n; i++){
-     printf("%10.5f ",uu[i]);
-  }
-  printf("\n");
-
-  printf("b1\n");
-  for (i=1; i<=n; i++){
-     printf("%10.5f ",b1[i]);
-  }
-  printf("\n");
-  */
-
-  /*
-  MPI_Finalize();
-  exit(0);
-  */
-
-
   /****************************************************
                    call a lapack routine
   ****************************************************/
@@ -204,21 +388,40 @@ void Eigen_Improved_PReHH(MPI_Comm MPI_Curret_Comm_WD,
 
   for (i=1; i<=n; i++){
     p[i-1] = ad[i][i];
-  }
-  for (i=1; i<n; i++){
     q[i-1] = ad[i][i+1];
   }
 
   if      (dste_flag==0) lapack_dstegr1(n,EVmax,p,q,ko,ac);
   else if (dste_flag==1) lapack_dstedc1(n,p,q,ko,ac);
-  else if (dste_flag==2) lapack_dstevx1(n,EVmax,p,q,ko,ac);
+  else if (dste_flag==2) {
+
+    /*
+    lapack_dstevx1(n,EVmax,p,q,ko,ac);
+    */
+
+    if (is1[myid]<=ie1[myid]){
+      lapack_dstevx4(n,is1[myid],ie1[myid],p,q,ko,ac);
+    }
+
+    /* MPI_Bcast */
+    for (ID=0; ID<numprocs; ID++){
+
+      num1 = ie1[ID] - is1[ID] + 1;
+      i = is1[ID];
+
+      if (0<num1){
+        MPI_Bcast(&ko[i], num1, MPI_DOUBLE, ID, MPI_Current_Comm_WD);
+      }
+    }
+  }
+
   else if (dste_flag==3) lapack_dsteqr1(n,p,q,ko,ac);
 
   if (measure_time==1){
     dtime(&Etime);
     printf("T2 myid=%2d   %15.12f\n",myid,Etime-Stime); 
   }
- 
+
   /****************************************************
     transformation of eigenvectors to original space
   ****************************************************/
@@ -320,7 +523,7 @@ void Eigen_Improved_PReHH(MPI_Comm MPI_Curret_Comm_WD,
 
     if (measure_time==1) dtime(&Stime);
 
-    BroadCast_ReMatrix(MPI_Curret_Comm_WD,ac,n,is1,ie1,myid,numprocs,
+    BroadCast_ReMatrix(MPI_Current_Comm_WD,ac,n,is1,ie1,myid,numprocs,
                        stat_send,request_send,request_recv);
 
     if (measure_time==1){
@@ -346,6 +549,25 @@ void Eigen_Improved_PReHH(MPI_Comm MPI_Curret_Comm_WD,
     }
   }
 
+
+  if (myid==0 && 0){
+
+  printf("ko myid=%2d\n",myid);
+  for (i=1; i<=n; i++){
+    printf("i=%2d %10.5f\n",i,ko[i]);
+  }
+
+  printf("ac myid=%2d\n",myid);
+  for (i=1; i<=n; i++){
+    for (j=1; j<=n; j++){
+      printf("%10.5f ",ac[i][j]);
+    }
+    printf("\n");
+  }
+
+  }
+
+
   /****************************************************
                   freeing of arrays:
   ****************************************************/
@@ -370,7 +592,7 @@ void Eigen_Improved_PReHH(MPI_Comm MPI_Curret_Comm_WD,
 
 
 
-void call_dsytrd( MPI_Comm MPI_Curret_Comm_WD, int numprocs, int myid, int n, double **ac, double **ad, 
+void call_dsytrd( MPI_Comm MPI_Current_Comm_WD, int numprocs, int myid, int n, double **ac, double **ad, 
 		  double *uu, double *p, double *q, double *b1, 
 		  MPI_Request *request_send, 
 		  MPI_Request *request_recv, 
@@ -400,14 +622,14 @@ void call_dsytrd( MPI_Comm MPI_Curret_Comm_WD, int numprocs, int myid, int n, do
     }
   }
 
-
-
+  free(A);
+  free(work);
 }
 
 
 
 
-void myHH( MPI_Comm MPI_Curret_Comm_WD, int numprocs, int myid, int n, double **ac, double **ad, 
+void myHH( MPI_Comm MPI_Current_Comm_WD, int numprocs, int myid, int n, double **ac, double **ad, 
            double *uu, double *b1, 
            MPI_Request *request_send, 
            MPI_Request *request_recv, 
@@ -418,12 +640,14 @@ void myHH( MPI_Comm MPI_Curret_Comm_WD, int numprocs, int myid, int n, double **
   int nump,num0,k0,num,count,ID;
   int IDS,IDR,num1,tag=999;
   int *is2,*ie2,*row_flag;
+  double p0,p1,p2,p3;
   double *p,*q,*u;
   double u1,u2,ss,my_r,r;
   double s1,s2,s3,s20,s21,s22,s23,tmp1,tmp2;
   double time1a,time1b,time1c,time1d,time1e;
   double Stime1, Etime1,av_num;
   double Stime, Etime;
+  int OMPID,Nthrds; 
 
   MPI_Status stat;
   MPI_Request request;
@@ -538,7 +762,7 @@ void myHH( MPI_Comm MPI_Curret_Comm_WD, int numprocs, int myid, int n, double **
 
     if (myid==ID){
       for (IDS=ID+1; IDS<numprocs; IDS++){
-        MPI_Isend(&u[i], count, MPI_DOUBLE, IDS, tag, MPI_Curret_Comm_WD, &request_send[IDS-(ID+1)]);
+        MPI_Isend(&u[i], count, MPI_DOUBLE, IDS, tag, MPI_Current_Comm_WD, &request_send[IDS-(ID+1)]);
       }
 
       num = (numprocs-1) - (ID+1) + 1;  
@@ -547,7 +771,7 @@ void myHH( MPI_Comm MPI_Curret_Comm_WD, int numprocs, int myid, int n, double **
       }
     }
     else if ( (ID+1)<=myid ) {
-      MPI_Recv(&u[i], count, MPI_DOUBLE, ID, tag, MPI_Curret_Comm_WD, &stat);
+      MPI_Recv(&u[i], count, MPI_DOUBLE, ID, tag, MPI_Current_Comm_WD, &stat);
     }
 
     s2 = u[i];
@@ -593,13 +817,30 @@ void myHH( MPI_Comm MPI_Curret_Comm_WD, int numprocs, int myid, int n, double **
 	for (i1=is0; i1<=ie2[myid]; i1++){
  
           i2 = i1 + (myid - ID0);
-	  p[i2] = 0.0;
 
+          p0 = 0.0;
+          p1 = 0.0;
+          p2 = 0.0;
+          p3 = 0.0;
+
+	  for (j=i+1; j<=(n-3); j+=4){
+	    p0 += ac[i1][j  ] * u[j  ];
+	    p1 += ac[i1][j+1] * u[j+1];
+	    p2 += ac[i1][j+2] * u[j+2];
+	    p3 += ac[i1][j+3] * u[j+3];
+	  }
+
+	  for ( ; j<=n; j++) p0 += ac[i1][j] * u[j];
+	  p[i2] = (p0+p1+p2+p3) / u1;
+
+	  /*
+          p[i2] = 0.0;
 	  for (j=i+1; j<=n; j++){
 	    p[i2] += ac[i1][j] * u[j];
 	  }
-
 	  p[i2] = p[i2] / u1;
+	  */
+
 	  my_r += u[i1] * p[i2];
 	}
 
@@ -628,7 +869,7 @@ void myHH( MPI_Comm MPI_Curret_Comm_WD, int numprocs, int myid, int n, double **
 
 	for (ID=0; ID<nump; ID++){
 	  IDS = (myid + ID) % nump + ID0; 
-	  MPI_Isend(&p[k0], num0, MPI_DOUBLE, IDS, tag, MPI_Curret_Comm_WD, &request_send[IDS-ID0]);
+	  MPI_Isend(&p[k0], num0, MPI_DOUBLE, IDS, tag, MPI_Current_Comm_WD, &request_send[IDS-ID0]);
 	}
 
         /* receiving */
@@ -638,7 +879,7 @@ void myHH( MPI_Comm MPI_Curret_Comm_WD, int numprocs, int myid, int n, double **
 	  IDR = (myid - ID + nump) % nump + ID0;
 	  num1 = ie2[IDR] - is2[IDR] + 2;
 	  k1 = is2[IDR] + (IDR - ID0);
-	  MPI_Irecv(&p[k1], num1, MPI_DOUBLE, IDR, tag, MPI_Curret_Comm_WD, &request_recv[IDR-ID0]);
+	  MPI_Irecv(&p[k1], num1, MPI_DOUBLE, IDR, tag, MPI_Current_Comm_WD, &request_recv[IDR-ID0]);
 	}
 
         /* waitall */
@@ -684,6 +925,7 @@ void myHH( MPI_Comm MPI_Curret_Comm_WD, int numprocs, int myid, int n, double **
 	if (is2[myid]<(i+1)) is0 = i + 1;
 	else                 is0 = is2[myid];
         
+	/*
 	for (i1=is0; i1<=ie2[myid]; i1++){
           
 	  tmp1 = u[i1];
@@ -692,8 +934,27 @@ void myHH( MPI_Comm MPI_Curret_Comm_WD, int numprocs, int myid, int n, double **
 	  for (j1=i+1; j1<=n; j1++){
 	    ac[i1][j1] -= tmp1 * q[j1] + tmp2 * u[j1];
 	  }
-
 	}
+	*/
+
+	for (i1=is0; i1<=ie2[myid]; i1++){
+          
+	  tmp1 = u[i1];
+	  tmp2 = q[i1];
+
+	  for (j1=i+1; j1<=(n-3); j1+=4){
+	    ac[i1][j1  ] -= tmp1 * q[j1  ] + tmp2 * u[j1  ];
+	    ac[i1][j1+1] -= tmp1 * q[j1+1] + tmp2 * u[j1+1];
+	    ac[i1][j1+2] -= tmp1 * q[j1+2] + tmp2 * u[j1+2];
+	    ac[i1][j1+3] -= tmp1 * q[j1+3] + tmp2 * u[j1+3];
+	  }
+
+	  for ( ; j1<=n; j1++){
+	    ac[i1][j1] -= tmp1 * q[j1] + tmp2 * u[j1];
+	  }
+	}
+
+
       }
 
       if (measure_time==1){
@@ -706,16 +967,16 @@ void myHH( MPI_Comm MPI_Curret_Comm_WD, int numprocs, int myid, int n, double **
     if (myid<row_flag[i+1]) break;
   }
 
-  MPI_Barrier(MPI_Curret_Comm_WD);
+  MPI_Barrier(MPI_Current_Comm_WD);
 
   if (measure_time==1) dtime(&Stime1);
 
   /* broadcast uu, b1, and ac */
 
-  MPI_Bcast(&uu[0],n+1,MPI_DOUBLE,numprocs-1,MPI_Curret_Comm_WD);
-  MPI_Bcast(&b1[0],n+1,MPI_DOUBLE,numprocs-1,MPI_Curret_Comm_WD);
+  MPI_Bcast(&uu[0],n+1,MPI_DOUBLE,numprocs-1,MPI_Current_Comm_WD);
+  MPI_Bcast(&b1[0],n+1,MPI_DOUBLE,numprocs-1,MPI_Current_Comm_WD);
 
-  BroadCast_ReMatrix(MPI_Curret_Comm_WD,ac,n,is2,ie2,myid,numprocs,
+  BroadCast_ReMatrix(MPI_Current_Comm_WD,ac,n,is2,ie2,myid,numprocs,
                      stat_send,request_send,request_recv);
 
   if (measure_time==1){
@@ -760,7 +1021,7 @@ void myHH( MPI_Comm MPI_Curret_Comm_WD, int numprocs, int myid, int n, double **
 
 
 
-void Eigen_Original_PReHH(MPI_Comm MPI_Curret_Comm_WD, 
+void Eigen_Original_PReHH(MPI_Comm MPI_Current_Comm_WD, 
                   double **ac, double *ko, int n, int EVmax, int bcast_flag)
 {
   double ABSTOL=1.0e-13;
@@ -791,8 +1052,8 @@ void Eigen_Original_PReHH(MPI_Comm MPI_Curret_Comm_WD,
   MPI_Request *request_recv;
 
   /* MPI */
-  MPI_Comm_size(MPI_Curret_Comm_WD,&numprocs);
-  MPI_Comm_rank(MPI_Curret_Comm_WD,&myid);
+  MPI_Comm_size(MPI_Current_Comm_WD,&numprocs);
+  MPI_Comm_rank(MPI_Current_Comm_WD,&myid);
 
   stat_send = malloc(sizeof(MPI_Status)*numprocs);
   request_send = malloc(sizeof(MPI_Request)*numprocs);
@@ -1062,7 +1323,7 @@ void Eigen_Original_PReHH(MPI_Comm MPI_Curret_Comm_WD,
 
     if (measure_time==1) dtime(&Stime);
 
-    BroadCast_ReMatrix(MPI_Curret_Comm_WD,ac,n,is1,ie1,myid,numprocs,
+    BroadCast_ReMatrix(MPI_Current_Comm_WD,ac,n,is1,ie1,myid,numprocs,
                        stat_send,request_send,request_recv);
 
     if (measure_time==1){
