@@ -22,12 +22,8 @@
 /*  end stat section */
 #include "openmx_common.h"
 #include "Inputtools.h"
- 
-#ifdef nompi
-#include "mimic_mpi.h"
-#else
 #include "mpi.h"
-#endif
+ 
 
 
 void Check_Force(char *argv[]);
@@ -50,6 +46,7 @@ static int
 };
 
 
+
 void Force_test(int argc, char *argv[]) 
 {
   FILE *fp,*fp0,*fp1,*fp2,*fp3;
@@ -74,7 +71,7 @@ void Force_test(int argc, char *argv[])
   printf("\n*******************************************************\n");   fflush(stdout);
   printf("*******************************************************\n");     fflush(stdout);
   printf(" Welcome to OpenMX                                     \n");     fflush(stdout);
-  printf(" Copyright (C), 2002-2009, T.Ozaki                     \n");     fflush(stdout);
+  printf(" Copyright (C), 2002-2013, T.Ozaki                     \n");     fflush(stdout);
   printf(" OpenMX comes with ABSOLUTELY NO WARRANTY.             \n");     fflush(stdout);
   printf(" This is free software, and you are welcome to         \n");     fflush(stdout);
   printf(" redistribute it under the constitution of the GNU-GPL.\n");     fflush(stdout);
@@ -211,7 +208,6 @@ void Force_test(int argc, char *argv[])
 
 
 
-
 void Check_Force(char *argv[])
 {
   int ixyz;
@@ -227,8 +223,8 @@ void Check_Force(char *argv[])
 
   /* MPI */ 
 
-  MPI_Comm_size(mpi_comm_level1,&numprocs);
-  MPI_Comm_rank(mpi_comm_level1,&myid);
+  MPI_Comm_size(MPI_COMM_WORLD1,&numprocs);
+  MPI_Comm_rank(MPI_COMM_WORLD1,&myid);
 
   /* initialize */
 
@@ -239,7 +235,7 @@ void Check_Force(char *argv[])
 
   /* step for calculation of the numerical derivatives */
 
-  step = 0.0001;
+  step = 0.0003;
 
   /* flags */
 
@@ -266,7 +262,7 @@ void Check_Force(char *argv[])
     Gxyz[1][3] = original_z;
 
     MD_iter = 1;
-    time1 = truncation(MD_iter,Solver==6,1);
+    time1 = truncation(MD_iter,1);
     time2 = DFT(MD_iter,(MD_iter-1)%orbitalOpt_per_MDIter+1);
     Utot1 = Utot;
 
@@ -276,17 +272,23 @@ void Check_Force(char *argv[])
       Analytic_Force[ixyz] = -Gxyz[1][16+ixyz];
     }
 
-    MPI_Bcast(&Analytic_Force[ixyz], 1, MPI_DOUBLE, G2ID[1], mpi_comm_level1);
+    MPI_Bcast(&Analytic_Force[ixyz], 1, MPI_DOUBLE, G2ID[1], MPI_COMM_WORLD1);
+
+    /*  do not use the restart files for later calculations */
+    Scf_RestartFromFile = 0;
 
     MD_iter = 2;
     Gxyz[1][ixyz] -= step; 
-    time1 = truncation(MD_iter,Solver==6,1);
+    time1 = truncation(MD_iter,1);
     time2 = DFT(MD_iter,(MD_iter-1)%orbitalOpt_per_MDIter+1);
     Utot2 = Utot;
 
+    /*  do not use the restart files for later calculations */
+    Scf_RestartFromFile = 0;
+
     MD_iter = 3;
     Gxyz[1][ixyz] += 2.0*step; 
-    time1 = truncation(MD_iter,Solver==6,1);
+    time1 = truncation(MD_iter,1);
     time2 = DFT(MD_iter,(MD_iter-1)%orbitalOpt_per_MDIter+1);
     Utot3 = Utot;
 
@@ -296,7 +298,7 @@ void Check_Force(char *argv[])
       Numerical_Force[ixyz] = -0.5*(Utot3 - Utot2)/step;     
     }
 
-    MPI_Bcast(&Numerical_Force[ixyz], 1, MPI_DOUBLE, G2ID[1], mpi_comm_level1);
+    MPI_Bcast(&Numerical_Force[ixyz], 1, MPI_DOUBLE, G2ID[1], MPI_COMM_WORLD1);
   }
 
   /* save forces to a file */
