@@ -1,7 +1,7 @@
 /**********************************************************************
   Get_Orbitals.c:
 
-     Get_Orbitals.c is a subrutine to calculate basis orbitals
+     Get_Orbitals.c is a subroutine to calculate basis orbitals
 
   Log of Get_Orbitals.c:
 
@@ -21,7 +21,7 @@ void Get_Orbitals(int wan, double x, double y, double z, double *Chi)
   double S_coordinate[3];
   double dum,dum1,dum2,dum3,dum4,a,b,c,d;
   double siQ,coQ,siP,coP,Q,P,R,Rmin;
-  double rm,df;
+  double rm,df,sum0,sum1;
   double **RF;
   double **AF;
   double SH[Supported_MaxL*2+1][2];
@@ -34,7 +34,7 @@ void Get_Orbitals(int wan, double x, double y, double z, double *Chi)
   double g1,g2,x1,x2,y1,y2,y12,y22,f;
 
   /****************************************************
-     allocation of arrays:
+   allocation of arrays:
 
    double RF[List_YOUSO[25]+1][List_YOUSO[24]];
    double AF[List_YOUSO[25]+1][2*(List_YOUSO[25]+1)+1];
@@ -243,6 +243,7 @@ void Get_Orbitals(int wan, double x, double y, double z, double *Chi)
         AF[2][3] = dum2*coP;
         AF[2][4] = dum2*siP;
       }
+
       else if (L0==3){
         AF[3][0] = 0.373176332590116*(5.0*coQ*coQ*coQ - 3.0*coQ);
         AF[3][1] = 0.457045799464466*coP*siQ*(5.0*coQ*coQ - 1.0);
@@ -255,37 +256,24 @@ void Get_Orbitals(int wan, double x, double y, double z, double *Chi)
 
       else if (4<=L0){
 
+        /* calculation of complex spherical harmonics functions */
         for(m=-L0; m<=L0; m++){ 
           ComplexSH(L0,m,Q,P,SH[L0+m],dSHt[L0+m],dSHp[L0+m]);
 	}
 
-        AF[L0][0] = SH[L0][0];
- 
-        j = -1;
-        for (i=1; i<(L0*2+1); i=i+4){
-          j++;
-          AF[L0][i] = 2.0/sqrt(2.0)*SH[L0-(2*j+1)][0];
-	}
+        /* transformation of complex to real */
+        for (i=0; i<(L0*2+1); i++){
 
-	j = 0;
-	for (i=3; i<(L0*2+1); i=i+4){
-	  j++;
-          AF[L0][i] = 2.0/sqrt(2.0)*SH[L0-2*j][0];
-	}
+          sum0 = 0.0;
+          sum1 = 0.0; 
+	  for (j=0; j<(L0*2+1); j++){
+            sum0 += Comp2Real[L0][i][j].r*SH[j][0] - Comp2Real[L0][i][j].i*SH[j][1]; 
+            sum1 += Comp2Real[L0][i][j].r*SH[j][1] + Comp2Real[L0][i][j].i*SH[j][0]; 
+	  }
+          AF[L0][i] = sum0 + sum1; 
+	}              
 
-	j = -1;
-	for (i=2; i<(L0*2+1); i=i+4){
-	  j++;
-          AF[L0][i] = 2.0/sqrt(2.0)*SH[L0-(2*j+1)][1];
-	}
-
-	j = 0;
-	for (i=4; i<(L0*2+1); i=i+4){
-	  j++;
-          AF[L0][i] = 2.0/sqrt(2.0)*SH[L0-2*j][1];
-	}
       }
-
     }
   }
 
@@ -301,7 +289,7 @@ void Get_Orbitals(int wan, double x, double y, double z, double *Chi)
   }
 
   /****************************************************
-     freeing of arrays:
+   freeing of arrays:
 
    double RF[List_YOUSO[25]+1][List_YOUSO[24]];
    double AF[List_YOUSO[25]+1][2*(List_YOUSO[25]+1)+1];
