@@ -15,12 +15,8 @@
 #include <math.h>
 #include <time.h>
 #include "openmx_common.h"
-
-#ifdef nompi
-#include "mimic_mpi.h"
-#else
 #include "mpi.h"
-#endif
+
 
 void Initial_CntCoes(double *****nh, double *****OLP)
 {
@@ -631,7 +627,7 @@ void Initial_CntCoes(double *****nh, double *****OLP)
     for (l=P_min; l<=NUM; l++){
       M1[l] = 1.0/sqrt(ko[l]);
     }
-
+    
     for (i1=1; i1<=NUM; i1++){
       for (j1=P_min; j1<=NUM; j1++){
         sum = 0.0;
@@ -722,7 +718,7 @@ void Initial_CntCoes(double *****nh, double *****OLP)
 
     if (3<=level_stdout){
       printf("\n");
-      printf("first C\n");
+      printf("first C Gc_AN=%2d\n",Gc_AN);
       for (i1=1; i1<=NUM; i1++){
         for (j1=1; j1<=10; j1++){
           printf("%6.3f ",C[i1][j1]); 
@@ -1321,6 +1317,15 @@ void Initial_CntCoes(double *****nh, double *****OLP)
       }        
 
     }
+
+    if (3<=level_stdout){
+      for (al=0; al<Spe_Total_CNO[wan]; al++){
+	for (p=0; p<Spe_Specified_Num[wan][al]; p++){
+	  printf("D Init_CntCoes Mc_AN=%2d Gc_AN=%2d al=%2d p=%2d  %15.12f\n",
+		 Mc_AN,Gc_AN,al,p,CntCoes[Mc_AN][al][p]);
+	}
+      }
+    }
   }
 
   /****************************************************
