@@ -3,38 +3,28 @@
 #include <math.h>
 #include <time.h>
 #include "openmx_common.h"
-
-#ifdef nompi
-#include "mimic_mpi.h"
-#else
 #include "mpi.h"
-#endif
 
 
-void Shift_Rhok(double *****ReRhok, double *****ImRhok, double *****Residual_ReRhok, double *****Residual_ImRhok);
-
 
 double Mixing_DM(int MD_iter,
                  int SCF_iter,
                  int SCF_iter0,
                  int SucceedReadingDMfile,
-                 double *****ReRhok,
-                 double *****ImRhok,
-                 double ****ReBestRhok,
-                 double ****ImBestRhok,
-                 double *****Residual_ReRhok,
-                 double *****Residual_ImRhok,
-                 double ***ReV1,
-                 double ***ImV1,
-                 double ***ReV2,
-                 double ***ImV2,
-                 double ***ReRhoAtomk,
-                 double ***ImRhoAtomk)
+                 double ***ReRhok,
+                 double ***ImRhok,
+                 double **Residual_ReRhok,
+                 double **Residual_ImRhok,
+                 double *ReVk,
+                 double *ImVk,
+                 double *ReRhoAtomk,
+                 double *ImRhoAtomk)
 {
   int pSCF_iter,NumMix,NumSlide;
   int spin,ct_AN,wan1,TNO1,h_AN,Gh_AN,wan2;
   int TNO2,i,j,ian,jan,m,n;
   int spinmax,k1,k2,k3;
+  int Mc_AN,Gc_AN;
   double time0;
   double TStime,TEtime;
   int numprocs,myid,ID;
@@ -51,11 +41,47 @@ double Mixing_DM(int MD_iter,
   *******************************************************/
 
   if (Mixing_switch==0){
+
     if (MD_iter==1 && SCF_iter==1){
-      Simple_Mixing_DM(0,Mixing_weight,DM[0],DM[1],DM[2],iDM[0],iDM[1],iDM[2],ResidualDM[2],iResidualDM[2]);
+
+     /*---------- modified by TOYODA 18/JAN/2010 */
+
+#if EXX_MIX_DM
+
+      if (g_exx_switch) {
+        Simple_Mixing_DM(0,1.0,DM[0],DM[1],DM[2], 
+          iDM[0],iDM[1],iDM[2],ResidualDM[2],iResidualDM[2],
+          g_exx, g_exx_DM[0], g_exx_DM[1], g_exx_DM[2]);
+      } else {
+        Simple_Mixing_DM(0,1.0,DM[0],DM[1],DM[2], 
+          iDM[0],iDM[1],iDM[2],ResidualDM[2],iResidualDM[2],
+          NULL, NULL, NULL, NULL);
+      }
+#else
+      Simple_Mixing_DM( 0,1.0,DM[0],DM[1],DM[2],
+                        iDM[0],iDM[1],iDM[2],ResidualDM[2],iResidualDM[2]);
+#endif
+      /*---------- until here */
+
     }
     else{
+
+      /*---------- modified by TOYODA 18/JAN/2010 */
+#if EXX_MIX_DM
+      if (g_exx_switch) {
+        Simple_Mixing_DM(1,Mixing_weight,DM[0],DM[1],DM[2],
+          iDM[0],iDM[1],iDM[2],ResidualDM[2],iResidualDM[2],
+          g_exx, g_exx_DM[0], g_exx_DM[1], g_exx_DM[2]);
+      } else {
+        Simple_Mixing_DM(1,Mixing_weight,DM[0],DM[1],DM[2],
+          iDM[0],iDM[1],iDM[2],ResidualDM[2],iResidualDM[2],
+          NULL, NULL, NULL, NULL);
+      }
+#else
       Simple_Mixing_DM(1,Mixing_weight,DM[0],DM[1],DM[2],iDM[0],iDM[1],iDM[2],ResidualDM[2],iResidualDM[2]);
+#endif
+      /*---------- until here */
+
     }
   }
 
@@ -72,7 +98,23 @@ double Mixing_DM(int MD_iter,
       DIIS_Mixing_DM(1,ResidualDM,iResidualDM);
     }
     else if (SCF_iter<=(Pulay_SCF-1)){
+
+      /*---------- added by TOYODA 18/JAN/2010 */
+#if EXX_MIX_DM
+      if (g_exx_switch) {
+        Simple_Mixing_DM(1,Mixing_weight,DM[0],DM[1],DM[2],
+          iDM[0],iDM[1],iDM[2],ResidualDM[2],iResidualDM[2],
+          g_exx, g_exx_DM[0], g_exx_DM[1], g_exx_DM[2]);
+      } else {
+        Simple_Mixing_DM(1,Mixing_weight,DM[0],DM[1],DM[2],
+          iDM[0],iDM[1],iDM[2],ResidualDM[2],iResidualDM[2],
+          NULL, NULL, NULL, NULL);
+      }
+#else
       Simple_Mixing_DM(1,Mixing_weight,DM[0],DM[1],DM[2],iDM[0],iDM[1],iDM[2],ResidualDM[2],iResidualDM[2]);
+#endif
+      /*---------- until here */
+
     }
     else if (SCF_iter==Pulay_SCF)
       DIIS_Mixing_DM(2,ResidualDM,iResidualDM);
@@ -89,7 +131,21 @@ double Mixing_DM(int MD_iter,
       GR_Pulay_DM(1,ResidualDM);
     }
     else if (SCF_iter<=(Pulay_SCF-1))
+      /*---------- added by TOYODA 18/JAN/2010 */
+#if EXX_MIX_DM
+      if (g_exx_switch) {
+        Simple_Mixing_DM(1,Mixing_weight,DM[0],DM[1],DM[2],
+          iDM[0],iDM[1],iDM[2],ResidualDM[2],iResidualDM[2],
+           g_exx, g_exx_DM[0], g_exx_DM[1], g_exx_DM[2]);
+      } else {
+        Simple_Mixing_DM(1,Mixing_weight,DM[0],DM[1],DM[2],
+          iDM[0],iDM[1],iDM[2],ResidualDM[2],iResidualDM[2],
+           NULL, NULL, NULL, NULL);
+      }
+#else
       Simple_Mixing_DM(1,Mixing_weight,DM[0],DM[1],DM[2],iDM[0],iDM[1],iDM[2],ResidualDM[2],iResidualDM[2]);
+#endif
+      /*---------- until here */
     else if (SCF_iter==Pulay_SCF)
       GR_Pulay_DM(2,ResidualDM);
     else 
@@ -104,30 +160,30 @@ double Mixing_DM(int MD_iter,
 
     if (MD_iter==1 && SCF_iter0==1 && SucceedReadingDMfile==0){
       Kerker_Mixing_Rhok(0, Mixing_weight,
-                         ReRhok,ImRhok,ReBestRhok,ImBestRhok,
+                         ReRhok,ImRhok,
                          Residual_ReRhok,Residual_ImRhok,
-                         ReV1,ImV1,ReV2,ImV2,ReRhoAtomk,ImRhoAtomk);
+                         ReVk,ImVk,ReRhoAtomk,ImRhoAtomk);
     }
 
     else if (MD_iter==1 && SCF_iter==1){
       Kerker_Mixing_Rhok(0, Mixing_weight,
-                         ReRhok,ImRhok,ReBestRhok,ImBestRhok,
+                         ReRhok,ImRhok,
                          Residual_ReRhok,Residual_ImRhok,
-                         ReV1,ImV1,ReV2,ImV2,ReRhoAtomk,ImRhoAtomk);
+                         ReVk,ImVk,ReRhoAtomk,ImRhoAtomk);
     }
 
     else if ( (SCF_iter%30)==1) {
       Kerker_Mixing_Rhok(1,Max_Mixing_weight,
-                         ReRhok,ImRhok,ReBestRhok,ImBestRhok,
+                         ReRhok,ImRhok,
                          Residual_ReRhok,Residual_ImRhok,
-                         ReV1,ImV1,ReV2,ImV2,ReRhoAtomk,ImRhoAtomk);
+                         ReVk,ImVk,ReRhoAtomk,ImRhoAtomk);
     }
 
     else{
       Kerker_Mixing_Rhok(1,Mixing_weight,
-                         ReRhok,ImRhok,ReBestRhok,ImBestRhok,
+                         ReRhok,ImRhok,
                          Residual_ReRhok,Residual_ImRhok,
-                         ReV1,ImV1,ReV2,ImV2,ReRhoAtomk,ImRhoAtomk);
+                         ReVk,ImVk,ReRhoAtomk,ImRhoAtomk);
     }
   }
 
@@ -139,24 +195,25 @@ double Mixing_DM(int MD_iter,
  
     if (MD_iter==1 && SCF_iter==1){
       Kerker_Mixing_Rhok(0,Mixing_weight,
-                         ReRhok,ImRhok,ReBestRhok,ImBestRhok,
+                         ReRhok,ImRhok,
                          Residual_ReRhok,Residual_ImRhok,
-                         ReV1,ImV1,ReV2,ImV2,ReRhoAtomk,ImRhoAtomk);
+                         ReVk,ImVk,ReRhoAtomk,ImRhoAtomk);
     }
 
     else if (SCF_iter<Pulay_SCF){
       Kerker_Mixing_Rhok(1,Mixing_weight,
-                         ReRhok,ImRhok,ReBestRhok,ImBestRhok,
+                         ReRhok,ImRhok,
                          Residual_ReRhok,Residual_ImRhok,
-                         ReV1,ImV1,ReV2,ImV2,ReRhoAtomk,ImRhoAtomk);
+                         ReVk,ImVk,ReRhoAtomk,ImRhoAtomk);
     }
 
     else{
+
       DIIS_Mixing_Rhok(SCF_iter-(Pulay_SCF-3),
                        Mixing_weight,
-                       ReRhok,ImRhok,ReBestRhok,ImBestRhok,
+                       ReRhok,ImRhok,
                        Residual_ReRhok,Residual_ImRhok,
-                       ReV1,ImV1,ReV2,ImV2,ReRhoAtomk,ImRhoAtomk);
+                       ReVk,ImVk,ReRhoAtomk,ImRhoAtomk);
     }
 
   }
