@@ -21,29 +21,27 @@
 #include <unistd.h>
 /*  end stat section */
 #include "openmx_common.h"
-
-#ifdef nompi
-#include "mimic_mpi.h"
-#else
 #include "mpi.h"
-#endif
-
-
 
 
 static int  Input_HKS( int MD_iter, double *Uele, double *****CH );
 static void Output_HKS(int MD_iter, double *Uele, double *****CH );
 static void Output_Charge_Density(int MD_iter);
-static void Input_Charge_Density(int MD_iter, double *extpln_coes);
+static int Input_Charge_Density(int MD_iter, double *extpln_coes);
 static void Inverse(int n, double **a, double **ia);
 static void Extp_Charge(int MD_iter, double *extpln_coes);
 
 
-int RestartFileDFT(char *mode, int MD_iter, double *Uele, double *****H, double *****CntH)
+
+int RestartFileDFT(char *mode, int MD_iter, double *Uele, double *****H, double *****CntH, double *etime)
 {
-  int ret;
+  int ret,ret1;
   int numprocs,myid;
   double *extpln_coes;
+  double Stime,Etime;
+
+  *etime = 0.0; 
+  dtime(&Stime);
 
   /* MPI */
   MPI_Comm_size(mpi_comm_level1,&numprocs);
@@ -84,14 +82,17 @@ int RestartFileDFT(char *mode, int MD_iter, double *Uele, double *****H, double
 
     /* read charge density */
 
-    Input_Charge_Density(MD_iter,extpln_coes);
+    ret1 = Input_Charge_Density(MD_iter,extpln_coes);
+    ret *= ret1; 
 
     /* free */
 
     free(extpln_coes);
-
   }
 
+  dtime(&Etime);
+  *etime = Etime - Stime;
+
   return ret;
 }
 
@@ -112,7 +113,7 @@ void Extp_Charge(int MD_iter, double *extpln_coes)
 
   if (Cnt_switch==1) return;
 
-  if (2<MD_iter){
+  if (Extrapolated_Charge_History<MD_iter && 1<Extrapolated_Charge_History){
 
     /* allocation */
 
@@ -167,35 +168,10 @@ void Extp_Charge(int MD_iter, double *extpln_coes)
       B[i] = sum; 
     }
   
-    /*
-    printf("A\n");fflush(stdout);
-    for (i=0; i<NumHis; i++){
-      for (j=0; j<NumHis; j++){
-        printf("%10.5f ",A[i][j]);fflush(stdout);
-      }
-      printf("\n");fflush(stdout);
-    }
-
-    printf("B\n");fflush(stdout);
-    for (i=0; i<NumHis; i++){
-      printf("%10.5f\n",B[i]);fflush(stdout);
-    }
-    */
-
     /* calculate the inverse of A */
 
     Inverse(NumHis-1,A,IA);
 
-    /*
-    printf("IA\n");fflush(stdout);
-    for (i=0; i<NumHis; i++){
-      for (j=0; j<NumHis; j++){
-        printf("%10.5f ",IA[i][j]);fflush(stdout);
-      }
-      printf("\n");fflush(stdout);
-    }
-    */
-
     /* calculate the coefficients */
 
     for (i=0; i<NumHis; i++){
@@ -206,10 +182,6 @@ void Extp_Charge(int MD_iter, double *extpln_coes)
       }
 
       extpln_coes[i] = sum;
-
-      /*
-      printf("i=%2d extpln_coes=%15.12f\n",i,sum);fflush(stdout);
-      */
     }
 
     /* free */
@@ -225,7 +197,8 @@ void Extp_Charge(int MD_iter, double *extpln_coes)
     free(IA);
 
     free(B);
-  }  
+
+  } /* if (2<MD_iter)  */   
 
   /* shift His_Gxyz */
 
@@ -319,6 +292,7 @@ int Input_HKS( int MD_iter, double *Uele, double *****CH )
   int i_vec[20],*p_vec,po;
   int my_check,exit_flag;
   int pFNAN;
+  int l1,l2,l3,l10,l20,l30;
   int numprocs,myid;
   char fileHKS[YOUSO10];
   FILE *fp;
@@ -350,7 +324,7 @@ int Input_HKS( int MD_iter, double *Uele, double *****CH )
 
     sprintf(fileHKS,"%s%s_rst/%s.rst%i",filepath,filename,filename,Gc_AN);
 
-    if ((fp = fopen(fileHKS,"r")) != NULL){
+    if ((fp = fopen(fileHKS,"rb")) != NULL){
 
 #ifdef xt3
       setvbuf(fp,buf,_IOFBF,fp_bsize);  /* setvbuf */
@@ -385,24 +359,26 @@ int Input_HKS( int MD_iter, double *Uele, double *****CH )
       if (my_check!=0){
 
         /****************************************************
-                   read Gh_AN0, Rn0, wan20, TNO20
+              read Gh_AN0, l10, l20, l30, wan20, TNO20
         ****************************************************/
 
-        p_vec = (int*)malloc(sizeof(int)*(pFNAN+1)*4);
-        fread(p_vec, sizeof(int), (pFNAN+1)*4, fp);
+        p_vec = (int*)malloc(sizeof(int)*(pFNAN+1)*6);
+        fread(p_vec, sizeof(int), (pFNAN+1)*6, fp);
         fread(Uele,sizeof(double),1,fp);
 
         /****************************************************
           store Hamiltonian to appropriate position while 
-          comparing Gh_AN, Rn, wan2, TNO2
+          comparing Gh_AN, l1, l2, l3, wan2, TNO2
         ****************************************************/
 
         for (spin=0; spin<=SpinP_switch; spin++){ 
 	  for (h_AN0=0; h_AN0<=pFNAN; h_AN0++){
 	    Gh_AN0 = p_vec[              h_AN0];
-	    Rn0    = p_vec[(pFNAN+1)*1 + h_AN0];
-	    wan20  = p_vec[(pFNAN+1)*2 + h_AN0];
-	    TNO20  = p_vec[(pFNAN+1)*3 + h_AN0]; 
+	    l10    = p_vec[(pFNAN+1)*1 + h_AN0];
+	    l20    = p_vec[(pFNAN+1)*2 + h_AN0];
+	    l30    = p_vec[(pFNAN+1)*3 + h_AN0];
+	    wan20  = p_vec[(pFNAN+1)*4 + h_AN0];
+	    TNO20  = p_vec[(pFNAN+1)*5 + h_AN0]; 
  
 	    exit_flag = 0;
 	    h_AN = 0;
@@ -411,11 +387,16 @@ int Input_HKS( int MD_iter, double *Uele, double *****CH )
 
 	      Gh_AN = natn[Gc_AN][h_AN];
 	      Rn    = ncn[Gc_AN][h_AN];
+              l1 = atv_ijk[Rn][1];
+              l2 = atv_ijk[Rn][2];
+              l3 = atv_ijk[Rn][3];
 	      wan2  = WhatSpecies[Gh_AN];
 	      TNO2  = Spe_Total_CNO[wan2];
 
 	      if ( Gh_AN==Gh_AN0 &&
-		   Rn==Rn0       && 
+		   l1==l10 &&
+		   l2==l20 &&
+		   l3==l30 &&
 		   wan2==wan20   &&
 		   TNO2==TNO20 )
 		{
@@ -463,9 +444,11 @@ int Input_HKS( int MD_iter, double *Uele, double *****CH )
 	    for (h_AN0=0; h_AN0<=pFNAN; h_AN0++){
 
 	      Gh_AN0 = p_vec[              h_AN0];
-	      Rn0    = p_vec[(pFNAN+1)*1 + h_AN0];
-	      wan20  = p_vec[(pFNAN+1)*2 + h_AN0];
-	      TNO20  = p_vec[(pFNAN+1)*3 + h_AN0]; 
+	      l10    = p_vec[(pFNAN+1)*1 + h_AN0];
+	      l20    = p_vec[(pFNAN+1)*2 + h_AN0];
+	      l30    = p_vec[(pFNAN+1)*3 + h_AN0];
+	      wan20  = p_vec[(pFNAN+1)*4 + h_AN0];
+	      TNO20  = p_vec[(pFNAN+1)*5 + h_AN0]; 
  
 	      exit_flag = 0;
 	      h_AN = 0;
@@ -474,11 +457,16 @@ int Input_HKS( int MD_iter, double *Uele, double *****CH )
 
 		Gh_AN = natn[Gc_AN][h_AN];
 		Rn    = ncn[Gc_AN][h_AN];
+                l1 = atv_ijk[Rn][1];
+                l2 = atv_ijk[Rn][2];
+                l3 = atv_ijk[Rn][3];
 		wan2  = WhatSpecies[Gh_AN];
 		TNO2  = Spe_Total_CNO[wan2];
 
 		if ( Gh_AN==Gh_AN0 &&
-		     Rn==Rn0       && 
+		     l1==l10 &&
+		     l2==l20 &&
+		     l3==l30 &&
 		     wan2==wan20   &&
 		     TNO2==TNO20 )
 		  {
@@ -534,85 +522,6 @@ int Input_HKS( int MD_iter, double *Uele, double *****CH )
   /* freeing of array */ 
   free(tmpvec);
 
-
-
-
-
-  /*
-
-  for (Mc_AN=1; Mc_AN<=Matomnum; Mc_AN++){
-
-    Gc_AN = M2G[Mc_AN];
-    wan1 = WhatSpecies[Gc_AN];
-    TNO1 = Spe_Total_CNO[wan1];
-
-    for (h_AN=0; h_AN<=0; h_AN++){
-
-
-
-
-      printf("CH0\n");fflush(stdout);
-      for (i=0; i<TNO1; i++){
-        for (j=0; j<TNO1; j++){
-          printf("%7.4f ",CH[0][Mc_AN][h_AN][i][j]);fflush(stdout);
-	}
-        printf("\n");fflush(stdout);
-      }
-
-
-      printf("CH1\n");fflush(stdout);
-      for (i=0; i<TNO1; i++){
-        for (j=0; j<TNO1; j++){
-          printf("%7.4f ",CH[1][Mc_AN][h_AN][i][j]);fflush(stdout);           
-	}
-        printf("\n");fflush(stdout);
-      }
-
-      printf("CH2\n");fflush(stdout);
-      for (i=0; i<TNO1; i++){
-        for (j=0; j<TNO1; j++){
-          printf("%7.4f ",CH[2][Mc_AN][h_AN][i][j]);fflush(stdout);           
-	}
-        printf("\n");fflush(stdout);
-      }
-
-      printf("CH3\n");fflush(stdout);
-      for (i=0; i<TNO1; i++){
-        for (j=0; j<TNO1; j++){
-          printf("%7.4f ",CH[3][Mc_AN][h_AN][i][j]);fflush(stdout);           
-	}
-        printf("\n");fflush(stdout);
-      }
-
-      printf("iHNL0\n");fflush(stdout);
-      for (i=0; i<TNO1; i++){
-        for (j=0; j<TNO1; j++){
-          printf("%7.4f ",iHNL[0][Mc_AN][h_AN][i][j]);fflush(stdout);           
-	}
-        printf("\n");fflush(stdout);
-      }
-
-
-      printf("iHNL1\n");fflush(stdout);
-      for (i=0; i<TNO1; i++){
-        for (j=0; j<TNO1; j++){
-          printf("%7.4f ",iHNL[1][Mc_AN][h_AN][i][j]);fflush(stdout);           
-	}
-        printf("\n");
-      }
-
-      printf("iHNL2\n");fflush(stdout);
-      for (i=0; i<TNO1; i++){
-        for (j=0; j<TNO1; j++){
-          printf("%7.4f ",iHNL[2][Mc_AN][h_AN][i][j]);fflush(stdout);           
-	}
-        printf("\n");fflush(stdout);
-      }
-    }
-  }
-
-  */
-
   return my_check;
 }
 
@@ -622,12 +531,11 @@ int Input_HKS( int MD_iter, double *Uele, double *****CH )
 
 void Output_HKS(int MD_iter, double *Uele, double *****CH )
 {
-
   int Mc_AN,Gc_AN,h_AN,i,j,can,Gh_AN;
   int wan1,wan2,TNO1,TNO2,spin,Rn,po;
   int i_vec[20],*p_vec;
   int numprocs,myid;
-  char operate[300];
+  char operate[1000];
   char fileHKS[YOUSO10];
   FILE *fp;
   char buf[fp_bsize];          /* setvbuf */
@@ -667,7 +575,7 @@ void Output_HKS(int MD_iter, double *Uele, double *****CH )
 
     do {
 
-      if ((fp = fopen(fileHKS,"w")) != NULL){
+      if ((fp = fopen(fileHKS,"wb")) != NULL){
 
 #ifdef xt3
          setvbuf(fp,buf,_IOFBF,fp_bsize);  /* setvbuf */
@@ -699,18 +607,20 @@ void Output_HKS(int MD_iter, double *Uele, double *****CH )
                   # of orbitals in each FNAN atom
 	****************************************************/
 
-	p_vec = (int*)malloc(sizeof(int)*(FNAN[Gc_AN]+1)*4);
+	p_vec = (int*)malloc(sizeof(int)*(FNAN[Gc_AN]+1)*6);
 	for (h_AN=0; h_AN<=FNAN[Gc_AN]; h_AN++){
 	  Gh_AN = natn[Gc_AN][h_AN];
 	  Rn = ncn[Gc_AN][h_AN];
 	  wan2 = WhatSpecies[Gh_AN];
 	  TNO2 = Spe_Total_CNO[wan2];
 	  p_vec[                    h_AN] = Gh_AN;
-	  p_vec[(FNAN[Gc_AN]+1)*1 + h_AN] = Rn;
-	  p_vec[(FNAN[Gc_AN]+1)*2 + h_AN] = wan2;
-	  p_vec[(FNAN[Gc_AN]+1)*3 + h_AN] = TNO2;
+	  p_vec[(FNAN[Gc_AN]+1)*1 + h_AN] = atv_ijk[Rn][1];
+	  p_vec[(FNAN[Gc_AN]+1)*2 + h_AN] = atv_ijk[Rn][2];
+	  p_vec[(FNAN[Gc_AN]+1)*3 + h_AN] = atv_ijk[Rn][3];
+	  p_vec[(FNAN[Gc_AN]+1)*4 + h_AN] = wan2;
+	  p_vec[(FNAN[Gc_AN]+1)*5 + h_AN] = TNO2;
 	}
-	fwrite(p_vec,sizeof(int), (FNAN[Gc_AN]+1)*4, fp);
+	fwrite(p_vec,sizeof(int), (FNAN[Gc_AN]+1)*6, fp);
 	free(p_vec);
 
 	/****************************************************
@@ -769,214 +679,86 @@ void Output_HKS(int MD_iter, double *Uele, double *****CH )
 
 void Output_Charge_Density(int MD_iter)
 {
-  int i,j,k,i1,i2,i3,c;
-  int GN,mul,n1,n2,n3,nn1,nn0;
-  int cmd,MN,MN0,MN1,MN2,MN3;
-  double ***V;
-  double *tmp_array0;
-  double *tmp_array1;
-  int numprocs,myid,ID,IDS,IDR,tag=999;
-  char operate[300];
-  char fname1[300];
-  char fname2[300];
+  int i,spin,BN;
+  int numprocs,myid;
+  double *TmpRho;
   FILE *fp;
+  char file_check[YOUSO10];
   char fileCD0[YOUSO10];
   char fileCD1[YOUSO10];
   char fileCD2[YOUSO10];
-  char buf[fp_bsize];          /* setvbuf */
-
-  MPI_Status stat;
-  MPI_Request request;
 
   /* MPI */
   MPI_Comm_size(mpi_comm_level1,&numprocs);
   MPI_Comm_rank(mpi_comm_level1,&myid);
 
-  if      (SpinP_switch==0) mul = 1;
-  else if (SpinP_switch==1) mul = 2;
-  else if (SpinP_switch==3) mul = 4;
-  
-  /****************************************************
-   allocation of arrays:
+  /* allocation of array */
+  TmpRho = (double*)malloc(sizeof(double)*My_NumGridB_AB); 
 
-   double V[mul][My_NGrid1_Poisson][Ngrid2*Ngrid3];
-  ****************************************************/
+  /* save numprocs, Ngrid1, Ngrid2, and Ngrid3 */
 
-  V = (double***)malloc(sizeof(double**)*mul); 
-  for (k=0; k<mul; k++){
-    V[k] = (double**)malloc(sizeof(double*)*My_NGrid1_Poisson); 
-    for (i=0; i<My_NGrid1_Poisson; i++){
-      V[k][i] = (double*)malloc(sizeof(double)*Ngrid2*Ngrid3); 
+  if (myid==Host_ID){
+
+    sprintf(file_check,"%s%s_rst/%s.crst_check",filepath,filename,filename);
+
+    if ((fp = fopen(file_check,"w")) != NULL){
+      fprintf(fp,"%d %d %d %d",numprocs,Ngrid1,Ngrid2,Ngrid3);
+      fclose(fp);
+    }
+    else{
+      printf("Failure of saving %s\n",file_check);
     }
   }
 
-  /****************************************************
-                    set V0 and V1
-  ****************************************************/
-
-  /* initialize */
-
-  for (k=0; k<mul; k++){
-    for (n1=0; n1<My_NGrid1_Poisson; n1++){
-      for (n2=0; n2<Ngrid2*Ngrid3; n2++){
-        V[k][n1][n2] = 0.0;
-      }
-    }
-  }
-
-  /* use their densities using MPI */
-
-  for (k=0; k<mul; k++){
-
-    for (ID=0; ID<numprocs; ID++){
-
-      IDS = (myid + ID) % numprocs;
-      IDR = (myid - ID + numprocs) % numprocs;
-
-      /* Isend */
-      if (Num_Snd_Grid1[IDS]!=0){
-
-        tmp_array0 = (double*)malloc(sizeof(double)*Num_Snd_Grid1[IDS]*Ngrid2*Ngrid3); 
-  
-        for (i=0; i<Num_Snd_Grid1[IDS]; i++){ 
-
-	  n1 = Snd_Grid1[IDS][i];
-          nn1 = My_Cell0[n1];
-          MN1 = nn1*Ngrid2*Ngrid3;
-          MN0 = i*Ngrid2*Ngrid3;
-          for (n2=0; n2<Ngrid2; n2++){
-            MN2 = n2*Ngrid3;
-
-            if (k<=1){
-	      for (n3=0; n3<Ngrid3; n3++){
-		MN = MN1 + MN2 + n3;
-		tmp_array0[MN0+MN2+n3] = Density_Grid[k][MN] - 0.5*ADensity_Grid[MN];
-	      }
-	    }
-
-            else {
-	      for (n3=0; n3<Ngrid3; n3++){
-		MN = MN1 + MN2 + n3;
-		tmp_array0[MN0+MN2+n3] = Density_Grid[k][MN];
-	      }
-            }
-
-	  }
-        }
-
-        MPI_Isend(&tmp_array0[0], Num_Snd_Grid1[IDS]*Ngrid2*Ngrid3,
-                  MPI_DOUBLE, IDS, tag, mpi_comm_level1, &request);
-      }
-
-      /* Recv */
-      if (Num_Rcv_Grid1[IDR]!=0){
-
-        tmp_array1 = (double*)malloc(sizeof(double)*Num_Rcv_Grid1[IDR]*Ngrid2*Ngrid3); 
-
-        MPI_Recv(&tmp_array1[0], Num_Rcv_Grid1[IDR]*Ngrid2*Ngrid3,
-                MPI_DOUBLE, IDR, tag, mpi_comm_level1, &stat);
-
-        for (i=0; i<Num_Rcv_Grid1[IDR]; i++){ 
-	  n1 = Rcv_Grid1[IDR][i];
-          nn1 = My_Cell0[n1];
-          nn0 = n1 - Start_Grid1[myid];
-          MN0 = i*Ngrid2*Ngrid3;
-          for (n2=0; n2<Ngrid2; n2++){
-            MN2 = n2*Ngrid3;
-	    for (n3=0; n3<Ngrid3; n3++){
-	      MN = MN0 + MN2 + n3;
-	      V[k][nn0][MN2+n3] = tmp_array1[MN];
-	    }
-	  }
-        }
-
-        free(tmp_array1);
-      }
-
-      if (Num_Snd_Grid1[IDS]!=0){
-        MPI_Wait(&request,&stat);
-        free(tmp_array0);
-      }
-
-    }
-
-    /* use own densities */
-    for (n1=Start_Grid1[myid]; n1<=End_Grid1[myid]; n1++){
-      nn1 = My_Cell0[n1];
-      nn0 = n1 - Start_Grid1[myid]; 
-      if (nn1!=-1){
-        MN1 = nn1*Ngrid2*Ngrid3;
-        for (n2=0; n2<Ngrid2; n2++){
-          MN2 = n2*Ngrid3;
-
-          if (k<=1){
-	    for (n3=0; n3<Ngrid3; n3++){
-	      MN = MN1 + MN2 + n3;
-	      V[k][nn0][MN2+n3] = Density_Grid[k][MN] - 0.5*ADensity_Grid[MN];
-	    }    
-	  }
-          else {
-	    for (n3=0; n3<Ngrid3; n3++){
-	      MN = MN1 + MN2 + n3;
-	      V[k][nn0][MN2+n3] = Density_Grid[k][MN];
-	    }    
-	  }
-        }    
-      }
-    }
-
-  } /* mul */
-
   /****************************************************
                     output data 
   ****************************************************/
 
+  for (spin=0; spin<=SpinP_switch; spin++){
 
-  for (k=0; k<mul; k++){
-    for (n1=0; n1<My_NGrid1_Poisson; n1++){
+    /* store data to TmpRho */
 
-      nn0 = n1 + Start_Grid1[myid]; 
+    if (spin<=1){
+      for (BN=0; BN<My_NumGridB_AB; BN++){
+        TmpRho[BN] = Density_Grid_B[spin][BN] - ADensity_Grid_B[BN];
+      }
+    }
+    else{
+      for (BN=0; BN<My_NumGridB_AB; BN++){
+        TmpRho[BN] = Density_Grid_B[spin][BN];
+      }
+    } 
 
-      if (Cnt_switch!=1){ 
+    /* shift the index of stored data */
 
-	for (i=(Extrapolated_Charge_History-2); 0<=i; i--){
+    for (i=(Extrapolated_Charge_History-2); 0<=i; i--){
  
-	  sprintf(fileCD1,"%s%s_rst/%s.crst%i_%i_%i",filepath,filename,filename,k,nn0,i);
-	  sprintf(fileCD2,"%s%s_rst/%s.crst%i_%i_%i",filepath,filename,filename,k,nn0,i+1);
+      sprintf(fileCD1,"%s%s_rst/%s.crst%i_%i_%i",filepath,filename,filename,spin,myid,i);
+      sprintf(fileCD2,"%s%s_rst/%s.crst%i_%i_%i",filepath,filename,filename,spin,myid,i+1);
 
-	  if ((fp = fopen(fileCD1,"r")) != NULL){
-	    fclose(fp);
-	    rename(fileCD1,fileCD2); 
-	  }
-	} 
-      }
-
-      sprintf(fileCD0,"%s%s_rst/%s.crst%i_%i_0",filepath,filename,filename,k,nn0);
-
-      if ((fp = fopen(fileCD0,"w")) != NULL){
-        fwrite(V[k][n1],sizeof(double),Ngrid2*Ngrid3,fp);
+      if ((fp = fopen(fileCD1,"rb")) != NULL){
 	fclose(fp);
+	rename(fileCD1,fileCD2); 
       }
-      else{
-        printf("Could not open a file %s\n",fileCD0);
-      }
+    } 
+
+    /* save current data */
+
+    sprintf(fileCD0,"%s%s_rst/%s.crst%i_%i_0",filepath,filename,filename,spin,myid);
+
+    if ((fp = fopen(fileCD0,"wb")) != NULL){
+
+      fwrite(TmpRho,sizeof(double),My_NumGridB_AB,fp);
+      fclose(fp);
     }
-  }
-
-  /****************************************************
-   freeing of arrays:
-
-   double V[mul][My_NGrid1_Poisson][Ngrid2*Ngrid3];
-  ****************************************************/
-
-  for (k=0; k<mul; k++){
-    for (i=0; i<My_NGrid1_Poisson; i++){
-      free(V[k][i]);
+    else{
+      printf("Could not open a file %s\n",fileCD0);
     }
-    free(V[k]);
-  }
-  free(V);
 
+  } /* spin */
+
+  /* freeing of array */
+  free(TmpRho);
 }
 
 
@@ -986,84 +768,93 @@ void Output_Charge_Density(int MD_iter)
 
 
 
-void Input_Charge_Density(int MD_iter, double *extpln_coes)
+int Input_Charge_Density(int MD_iter, double *extpln_coes)
 {
-  int i,j,k,i1,i2,i3,c,spin,ri;
-  int GN,mul,n1,n2,n3,nn1,nn0;
-  int cmd,MN,MN0,MN1,MN2,MN3;
-  int numprocs,myid,ID,IDS,IDR,tag=999;
-  char operate[300];
-  char fname1[300];
-  char fname2[300];
+  int spin,i,BN,ret;
+  int numprocs,myid;
   FILE *fp;
   char fileCD[YOUSO10];
-  char buf[fp_bsize];          /* setvbuf */
+  char file_check[YOUSO10];
   double *tmp_array;
-
-  MPI_Status stat;
-  MPI_Request request;
+  int i_vec[10];
 
   /* MPI */
   MPI_Comm_size(mpi_comm_level1,&numprocs);
   MPI_Comm_rank(mpi_comm_level1,&myid);
 
-  /* allocation */
-  tmp_array = (double*)malloc(sizeof(double)*Ngrid2*Ngrid3);
+  /* check consistency in the number of grids */
 
-  for (spin=0; spin<=SpinP_switch; spin++){
+  sprintf(file_check,"%s%s_rst/%s.crst_check",filepath,filename,filename);
 
-    for (i=0; i<Num_Cells0; i++){
-      ri = My_Cell1[i];
-      MN = i*Ngrid2*Ngrid3;
+  i_vec[1] = 0; i_vec[2] = 0; i_vec[3] = 0;       
+  if ((fp = fopen(file_check,"r")) != NULL){
+    fscanf(fp,"%d %d %d %d",&i_vec[0],&i_vec[1],&i_vec[2],&i_vec[3]);
+    fclose(fp);
+  }
 
-      for (k=0; k<Extrapolated_Charge_History; k++){
+  /* read files */
 
-        sprintf(fileCD,"%s%s_rst/%s.crst%i_%i_%i",filepath,filename,filename,spin,ri,k);
-       
-	if ((fp = fopen(fileCD,"r")) != NULL){
+  if ( i_vec[1]==Ngrid1 && i_vec[2]==Ngrid2 && i_vec[3]==Ngrid3 ){
 
-	  fread(tmp_array,sizeof(double),Ngrid2*Ngrid3,fp);
+    /* allocation of array */
+
+    tmp_array = (double*)malloc(sizeof(double)*My_NumGridB_AB);
+
+    /* read data and extrapolate data */
+
+    for (spin=0; spin<=SpinP_switch; spin++){
+      for (i=0; i<Extrapolated_Charge_History; i++){
+
+	sprintf(fileCD,"%s%s_rst/%s.crst%i_%i_%i",filepath,filename,filename,spin,myid,i);
+
+	if ((fp = fopen(fileCD,"rb")) != NULL){
+
+	  fread(tmp_array,sizeof(double),My_NumGridB_AB,fp);
 	  fclose(fp);
 
-          if (k==0){
+	  if (i==0){
 
-            if (spin<=1){
-	      for (MN1=0; MN1<Ngrid2*Ngrid3; MN1++){
-		Density_Grid[spin][MN+MN1] = 0.5*ADensity_Grid[MN+MN1] + extpln_coes[k]*tmp_array[MN1];
+	    if (spin<=1){
+	      for (BN=0; BN<My_NumGridB_AB; BN++){
+		Density_Grid_B[spin][BN] = ADensity_Grid_B[BN] + extpln_coes[i]*tmp_array[BN];
 	      }
 	    }
-            else{
-	      for (MN1=0; MN1<Ngrid2*Ngrid3; MN1++){
-		Density_Grid[spin][MN+MN1] = extpln_coes[k]*tmp_array[MN1];
+	    else{
+	      for (BN=0; BN<My_NumGridB_AB; BN++){
+		Density_Grid_B[spin][BN] = extpln_coes[i]*tmp_array[BN];
 	      }
 	    }
 
-          }
+	  }
 
-          else{
-            for (MN1=0; MN1<Ngrid2*Ngrid3; MN1++){
-              Density_Grid[spin][MN+MN1] += extpln_coes[k]*tmp_array[MN1];
+	  else{
+	    for (BN=0; BN<My_NumGridB_AB; BN++){
+	      Density_Grid_B[spin][BN] += extpln_coes[i]*tmp_array[BN];
 	    }
-          }
+	  }
 
 	}
 	else{
-	  /*
-	  printf("Could not open a file %s\n",fileCD);
-	  */
+	  /* printf("Could not open a file %s\n",fileCD); */
 	}
       }
-
     }
+
+    /* MPI: from the partitions B to D */
+
+    Density_Grid_Copy_B2D();  
+
+    /* free */
+    free(tmp_array);
+
+    /* ret1 */
+    ret = 1;
   }
 
-  if (SpinP_switch==0){
-    for (MN=0; MN<My_NumGrid1; MN++){
-      Density_Grid[1][MN] = Density_Grid[0][MN];
-    }
+  else{
+    if (myid==0) printf("Failed (3) in reading the restart files\n"); fflush(stdout);     
+    ret = 0;
   }
 
-  /* free */
-  free(tmp_array);
-
+  return ret;  
 }
