@@ -24,13 +24,7 @@
 /*  end stat section */
 #include "openmx_common.h"
 #include "Inputtools.h"
- 
-#ifdef nompi
-#include "mimic_mpi.h"
-#else
 #include "mpi.h"
-#endif
-
 #include "tran_prototypes.h"
 #include "tran_variables.h"
 
@@ -82,8 +76,8 @@ void Runtest(char *mode, int argc, char *argv[])
 
   /* set up MPI */
 
-  MPI_Comm_size(mpi_comm_level1,&numprocs);
-  MPI_Comm_rank(mpi_comm_level1,&myid);
+  MPI_Comm_size(MPI_COMM_WORLD1, &numprocs);
+  MPI_Comm_rank(MPI_COMM_WORLD1, &myid);
 
   if (strcasecmp(mode,"S")==0){  
     input_dir = "input_example";
@@ -93,6 +87,10 @@ void Runtest(char *mode, int argc, char *argv[])
     input_dir = "large_example";
     output_file = "runtestL.result";
   }
+  else if (strcasecmp(mode,"L2")==0){  
+    input_dir = "large2_example";
+    output_file = "runtestL2.result";
+  }
   else if (strcasecmp(mode,"G")==0){  
     input_dir = "geoopt_example";
     output_file = "runtestG.result";
@@ -121,7 +119,7 @@ void Runtest(char *mode, int argc, char *argv[])
     printf("\n*******************************************************\n");  fflush(stdout);
     printf("*******************************************************\n");    fflush(stdout);
     printf(" Welcome to OpenMX   Ver. %s                           \n",Version_OpenMX); fflush(stdout);
-    printf(" Copyright (C), 2002-2009, T.Ozaki                     \n");    fflush(stdout);
+    printf(" Copyright (C), 2002-2013, T.Ozaki                     \n");    fflush(stdout);
     printf(" OpenMX comes with ABSOLUTELY NO WARRANTY.             \n");    fflush(stdout);
     printf(" This is free software, and you are welcome to         \n");    fflush(stdout);
     printf(" redistribute it under the constitution of the GNU-GPL.\n");    fflush(stdout);
@@ -189,7 +187,6 @@ void Runtest(char *mode, int argc, char *argv[])
 
   } /* if (myid==Host_ID) */
 
-
   sprintf(fname2,"%s",output_file);
 
   if (myid==Host_ID){
@@ -205,10 +202,10 @@ void Runtest(char *mode, int argc, char *argv[])
     printf(" %2d dat files are found in the directory '%s'.\n\n\n",Num_DatFiles,input_dir);
   }
 
-  MPI_Bcast(&Num_DatFiles, 1, MPI_INT, Host_ID, mpi_comm_level1);
+  MPI_Bcast(&Num_DatFiles, 1, MPI_INT, Host_ID, MPI_COMM_WORLD1);
 
   /***********************************************************
-         start calculations
+                      start calculations
   ***********************************************************/
 
   for (i=0; i<Num_DatFiles; i++){
@@ -217,7 +214,7 @@ void Runtest(char *mode, int argc, char *argv[])
       sprintf(fname_dat,"%s",fndat[i].fn);
     }  
 
-    MPI_Bcast(&fname_dat, YOUSO10, MPI_CHAR, Host_ID, mpi_comm_level1);
+    MPI_Bcast(&fname_dat, YOUSO10, MPI_CHAR, Host_ID, MPI_COMM_WORLD1);
 
     /* run openmx */
 
@@ -336,7 +333,7 @@ void Runtest(char *mode, int argc, char *argv[])
 	      || (NGrid1_2!=NGrid2_2)
 	      || (NGrid1_3!=NGrid2_3) )
 	  {
-	    fprintf(fp2,"  Invalid comparison due to the diffrent number of grids.\n");
+	    fprintf(fp2,"  Invalid comparison due to the different number of grids.\n");
 	    fprintf(fp2,"  You may use a different radix for FFT.\n");
 	  }
 
@@ -372,7 +369,7 @@ void Runtest(char *mode, int argc, char *argv[])
   }
 
 
-  MPI_Barrier(mpi_comm_level1);
+  MPI_Barrier(MPI_COMM_WORLD1);
   MPI_Finalize();
   exit(0);
 }
@@ -409,8 +406,8 @@ int run_main(int argc, char *argv[], int numprocs0, int myid0)
   /* allocation of CompTime */
   CompTime = (double**)malloc(sizeof(double*)*numprocs0); 
   for (i=0; i<numprocs0; i++){
-    CompTime[i] = (double*)malloc(sizeof(double)*20); 
-    for (j=0; j<20; j++) CompTime[i][j] = 0.0;
+    CompTime[i] = (double*)malloc(sizeof(double)*30); 
+    for (j=0; j<30; j++) CompTime[i][j] = 0.0;
   }
 
   if (myid0==Host_ID){  
@@ -433,14 +430,9 @@ int run_main(int argc, char *argv[], int numprocs0, int myid0)
                    Read the input file
   ****************************************************/
 
-  /* setup CPU group */
-  setup_CPU_group(argv[1]); 
-  if (myid0>=atomnum)  goto LAST_PROC;  /*  to cut off CPUs */
-
   init_alloc_first();
   CompTime[myid0][1] = readfile(argv);
-
-  MPI_Barrier(mpi_comm_level1);
+  MPI_Barrier(MPI_COMM_WORLD1);
 
   /* initialize PrintMemory routine */
 
@@ -463,7 +455,7 @@ int run_main(int argc, char *argv[], int numprocs0, int myid0)
 
   do {
 
-    CompTime[myid0][2] += truncation(MD_iter,Solver==6,1);
+    CompTime[myid0][2] += truncation(MD_iter,1);
 
     if (ML_flag==1 && myid0==Host_ID) Get_VSZ(MD_iter);  
 
@@ -483,7 +475,6 @@ int run_main(int argc, char *argv[], int numprocs0, int myid0)
 
   } while(MD_Opt_OK==0 && MD_iter<=MD_IterNumber);
 
-
   if ( TRAN_output_hks ) {
     /* left is dummy */
     TRAN_RestartFile(mpi_comm_level1, "write","left",filepath,TRAN_hksoutfilename);
@@ -516,7 +507,7 @@ int run_main(int argc, char *argv[], int numprocs0, int myid0)
                   Making of output files
   ****************************************************/
 
-  OutData(argv[1]);
+  CompTime[myid0][20] = OutData(argv[1]);
 
   /****************************************************
     write connectivity, Hamiltonian, overlap, density
@@ -538,11 +529,14 @@ int run_main(int argc, char *argv[], int numprocs0, int myid0)
   /* merge log files */
 
   Merge_LogFile(argv[1]);
+
+  /* free arrays */
+
   Free_Arrays(0);
 
-LAST_PROC: 
+  /* print memory */
 
-  if (myid0<atomnum) PrintMemory("total",0,"sum");
+  PrintMemory("total",0,"sum");
 
   /****************************************************
          reconstruct the original MPI group
@@ -557,19 +551,20 @@ LAST_PROC:
       new_ranks[i]=i; /* a new group is made of original rank=0:Pnum[k]-1 */
     }
 
-    MPI_Comm_group(MPI_COMM_WORLD, &old_group);
+    MPI_Comm_group(MPI_COMM_WORLD1, &old_group);
 
     /* define a new group */
     MPI_Group_incl(old_group,numprocs0,new_ranks,&new_group);
-    MPI_Comm_create(MPI_COMM_WORLD,new_group,&mpi_comm_level1);
+    MPI_Comm_create(MPI_COMM_WORLD1,new_group,&mpi_comm_level1);
 
     MPI_Group_free(&new_group);
     free(new_ranks); /* never forget cleaning! */
   }
 
-
-  MPI_Barrier(mpi_comm_level1);
-  printf("\nThe calculation was normally finished. (proc=%3d)\n",myid0);fflush(stdout);
+  MPI_Barrier(MPI_COMM_WORLD1);
+  if (myid0==Host_ID){
+    printf("\nThe calculation was normally finished.\n");
+  }
 
   return 0;
 }
