@@ -15,20 +15,8 @@
 #include <math.h>
 #include <time.h>
 #include "openmx_common.h"
-
-#ifdef nompi
-#include "mimic_mpi.h"
-#else
 #include "mpi.h"
-#endif
-
-#ifdef noomp
-#include "mimic_omp.h"
-#else
 #include <omp.h>
-#endif
-
-
 
 
 double Set_OLP_Kin(double *****OLP, double *****H0)
@@ -925,7 +913,7 @@ double Set_OLP_Kin(double *****OLP, double *****H0)
                   
                   /* add a small value for stabilization of eigenvalue routine */
 
-                  OLP[0][Mc_AN][h_AN][num0+L0+M0][num1+L1+M1] = 8.0*CsumS0.r + 0.0*rnd(1.0e-17);
+                  OLP[0][Mc_AN][h_AN][num0+L0+M0][num1+L1+M1] = 8.0*CsumS0.r + 1.0*rnd(1.0e-13);
 		  H0[0][Mc_AN][h_AN][num0+L0+M0][num1+L1+M1] = 4.0*CsumK0.r;
 
                   if (h_AN!=0){
@@ -1225,6 +1213,7 @@ double Set_OLP_Kin(double *****OLP, double *****H0)
 
   } /* #pragma omp parallel */
 
+
   /*
   printf("OLP_Lx\n");
   for (Mc_AN=1; Mc_AN<=Matomnum; Mc_AN++){
