@@ -14,12 +14,8 @@
 #include <stdlib.h>
 #include <math.h>
 #include "openmx_common.h"
-
-#ifdef nompi
-#include "mimic_mpi.h"
-#else
 #include "mpi.h"
-#endif
+
 
 void Simple_Mixing_DM(int Change_switch,
                       double Mix_wgt,
@@ -30,7 +26,14 @@ void Simple_Mixing_DM(int Change_switch,
                       double *****iPDM,
                       double *****iP2DM,
                       double *****RDM,
-                      double *****iRDM)
+                      double *****iRDM,
+/*---------- modified by TOYODA 18/JAN/2010 */
+                      EXX_t  *exx,
+                      dcomplex ****exx_CDM,
+                      dcomplex ****exx_PDM,
+                      dcomplex ****exx_P2DM
+/*---------- until here */
+)
 {
   int ian,jan,Mc_AN,Gc_AN;
   int h_AN,Gh_AN,m,n,spin,i;
@@ -256,6 +259,13 @@ void Simple_Mixing_DM(int Change_switch,
 
       }
     }
+
+    /*---------- added by TOYODA 18/JAN/2010 */ 
+    if (g_exx_switch) {
+      EXX_Simple_Mixing_DM(exx, Mix_wgt1, 
+        exx_CDM[spin], exx_PDM[spin], exx_P2DM[spin]);
+    }
+    /*---------- until here */
   }
 
 }
