@@ -18,15 +18,8 @@
 #include <math.h>
 #include <time.h>
 #include <string.h>
-
 #include "Inputtools.h"
-
-#ifdef nompi
-#include "mimic_mpi.h"
-#else
 #include "mpi.h"
-#endif
-
 #include "tran_prototypes.h"
 
 #define eV2Hartree    27.2113845                
@@ -1836,7 +1829,11 @@ void MTRAN_Input(MPI_Comm comm1,
   MPI_Comm_rank(comm1,&myid);
 
   /* open the input file */
-  input_open(fname);
+
+  if (input_open(fname)==0){
+    MPI_Finalize(); 
+    exit(0);
+  }
 
   s_vec[0]="Off"; s_vec[1]="On"; s_vec[2]="NC";
   i_vec[0]=0    ; i_vec[1]=1   ; i_vec[2]=3;
@@ -1987,7 +1984,11 @@ void MTRAN_Input_Sys(int argc,char *file, char *filepath, char *filename,
   double r_vec[20];
   double r_vec1[20];
 
-  input_open(file);
+  if (input_open(file)==0){
+    MPI_Finalize(); 
+    exit(0);
+  }
+
   input_string("System.CurrrentDirectory",filepath,"./");
   input_string("System.Name",filename,"default");
 
@@ -2635,7 +2636,7 @@ void MTRAN_Allocate_HS(int NUM_c,
 {
   int i,side,spin;
 
-  for (side=0;side<=1;side++) {
+  for (side=0; side<=1; side++) {
 
     S00_e[side] = (dcomplex*)malloc(sizeof(dcomplex)*NUM_e[side]*NUM_e[side] );
     S01_e[side] = (dcomplex*)malloc(sizeof(dcomplex)*NUM_e[side]*NUM_e[side] );
@@ -2658,51 +2659,52 @@ void MTRAN_Allocate_HS(int NUM_c,
         H01_e[side][spin][i].i = 0.0;
       }
     }
+  }
 
-    SCC = (dcomplex*)malloc(sizeof(dcomplex)*NUM_c*NUM_c ); 
-    SCL = (dcomplex*)malloc(sizeof(dcomplex)*NUM_c*NUM_e[0] ); 
-    SCR = (dcomplex*)malloc(sizeof(dcomplex)*NUM_c*NUM_e[1] ); 
+  SCC = (dcomplex*)malloc(sizeof(dcomplex)*NUM_c*NUM_c ); 
+  SCL = (dcomplex*)malloc(sizeof(dcomplex)*NUM_c*NUM_e[0] ); 
+  SCR = (dcomplex*)malloc(sizeof(dcomplex)*NUM_c*NUM_e[1] ); 
 
+  for (i=0; i<NUM_c*NUM_c; i++) {
+    SCC[i].r = 0.0;
+    SCC[i].i = 0.0;
+  }
+  for (i=0; i<NUM_c*NUM_e[0]; i++) {
+    SCL[i].r = 0.0;
+    SCL[i].i = 0.0;
+  }
+  for (i=0; i<NUM_c*NUM_e[1]; i++) {
+    SCR[i].r = 0.0;
+    SCR[i].i = 0.0;
+  }
+
+  HCC = (dcomplex**)malloc(sizeof(dcomplex*)*(SpinP_switch+1)); 
+  for (spin=0; spin<=SpinP_switch; spin++) {
+    HCC[spin] = (dcomplex*)malloc(sizeof(dcomplex)*NUM_c*NUM_c); 
     for (i=0; i<NUM_c*NUM_c; i++) {
-      SCC[i].r = 0.0;
-      SCC[i].i = 0.0;
+      HCC[spin][i].r = 0.0;
+      HCC[spin][i].i = 0.0;
     }
+  }
+
+  HCL = (dcomplex**)malloc(sizeof(dcomplex*)*(SpinP_switch+1)); 
+  for (spin=0; spin<=SpinP_switch; spin++) {
+    HCL[spin] = (dcomplex*)malloc(sizeof(dcomplex)*NUM_c*NUM_e[0]); 
     for (i=0; i<NUM_c*NUM_e[0]; i++) {
-      SCL[i].r = 0.0;
-      SCL[i].i = 0.0;
+      HCL[spin][i].r = 0.0;
+      HCL[spin][i].i = 0.0;
     }
+  }
+
+  HCR = (dcomplex**)malloc(sizeof(dcomplex*)*(SpinP_switch+1)); 
+  for (spin=0; spin<=SpinP_switch; spin++) {
+    HCR[spin] = (dcomplex*)malloc(sizeof(dcomplex)*NUM_c*NUM_e[1]); 
     for (i=0; i<NUM_c*NUM_e[1]; i++) {
-      SCR[i].r = 0.0;
-      SCR[i].i = 0.0;
+      HCR[spin][i].r = 0.0;
+      HCR[spin][i].i = 0.0;
     }
+  }
 
-    HCC = (dcomplex**)malloc(sizeof(dcomplex*)*(SpinP_switch+1)); 
-    for (spin=0; spin<=SpinP_switch; spin++) {
-      HCC[spin] = (dcomplex*)malloc(sizeof(dcomplex)*NUM_c*NUM_c); 
-      for (i=0; i<NUM_c*NUM_c; i++) {
-        HCC[spin][i].r = 0.0;
-        HCC[spin][i].i = 0.0;
-      }
-    }
-
-    HCL = (dcomplex**)malloc(sizeof(dcomplex*)*(SpinP_switch+1)); 
-    for (spin=0; spin<=SpinP_switch; spin++) {
-      HCL[spin] = (dcomplex*)malloc(sizeof(dcomplex)*NUM_c*NUM_e[0]); 
-      for (i=0; i<NUM_c*NUM_e[0]; i++) {
-        HCL[spin][i].r = 0.0;
-        HCL[spin][i].i = 0.0;
-      }
-    }
-
-    HCR = (dcomplex**)malloc(sizeof(dcomplex*)*(SpinP_switch+1)); 
-    for (spin=0; spin<=SpinP_switch; spin++) {
-      HCR[spin] = (dcomplex*)malloc(sizeof(dcomplex)*NUM_c*NUM_e[1]); 
-      for (i=0; i<NUM_c*NUM_e[1]; i++) {
-        HCR[spin][i].r = 0.0;
-        HCR[spin][i].i = 0.0;
-      }
-    }
-  } 
 }
 
 
@@ -2756,7 +2758,7 @@ int main(int argc, char **argv)
     printf(" Welcome to TranMain                                   \n");
     printf(" This is a post-processing code of OpenMX to calculate \n");
     printf(" electronic transmission and current.                  \n");
-    printf(" Copyright (C), 2002-2008, H.Kino and T.Ozaki          \n"); 
+    printf(" Copyright (C), 2002-2013, H.Kino and T.Ozaki          \n"); 
     printf(" TranMain comes with ABSOLUTELY NO WARRANTY.            \n"); 
     printf(" This is free software, and you are welcome to         \n"); 
     printf(" redistribute it under the constitution of the GNU-GPL.\n");
