@@ -13,17 +13,12 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <math.h>
-
+#include <complex.h>
 #include "openmx_common.h"
-
-#ifdef nompi
-#include "mimic_mpi.h"
-#else
 #include "mpi.h"
-#endif
+
 
 static void InitV();
-
 static void HGF();
 static void Matsubara();
 static void MM();
@@ -35,6 +30,7 @@ void init()
 {
   int i,p,wsp,wan;
   double dp,dM,dum,dum1;
+  double kappa,R,R1,R2;
 
   /****************************************************
                  Correct_Position_flag
@@ -53,6 +49,7 @@ void init()
   F_dVHart_flag = 1;
   F_Vxc_flag    = 1;
   F_U_flag      = 1;
+  F_dftD_flag   = 1; /* okuno */
 
   /****************************************************
                 Setting of atomic weight
@@ -60,12 +57,22 @@ void init()
 
   for (i=1; i<=atomnum; i++){
     wsp = WhatSpecies[i];
-    wan = Spe_WhatAtom[wsp];
-    Gxyz[i][20] = Atom_Weight[wan];
+
+    if (0.0<Spe_AtomicMass[wsp]){
+     Gxyz[i][20] = Spe_AtomicMass[wsp];
+    }
+    else{
+      wan = Spe_WhatAtom[wsp];
+      Gxyz[i][20] = Atom_Weight[wan];
+    }
   }
 
   InitV();
 
+  /* set Beta */
+
+  Beta = 1.0/kB/E_Temp;
+
   /****************************************************
    (1+(1+x/n)^n)^{-1}
    Complex poles for the modified Matsubara summation
@@ -93,8 +100,6 @@ void init()
   of Green's functions with Fermi-Dirac distribution
   ****************************************************/
 
-  Beta = 1.0/kB/E_Temp;
-
   if (Solver==1) {
 
     zero_cfrac( POLES, zp, Rp ); 
@@ -111,6 +116,189 @@ void init()
     }
   }
 
+  /**********************************************************
+                 O(N^2) cluster diagonalization 
+  **********************************************************/
+
+  if (Solver==9) {
+
+    ON2_Npoles_f = ON2_Npoles; 
+
+    /************************************
+      for calculation of density matrix
+    ************************************/
+
+    /* set up poles */
+
+    zero_cfrac( ON2_Npoles, ON2_zp, ON2_Rp ); 
+
+    for (p=0; p<ON2_Npoles; p++){
+      ON2_method[p] = 1; 
+    }
+
+    /* for the zero-th moment */
+
+    R = 1.0e+10;
+
+    ON2_zp[ON2_Npoles].r = 0.0;
+    ON2_zp[ON2_Npoles].i = R;
+
+    ON2_Rp[ON2_Npoles].r = 0.0;
+    ON2_Rp[ON2_Npoles].i = 0.5*R;
+
+    ON2_method[ON2_Npoles] = 2; 
+
+    ON2_Npoles++;
+
+    /*
+    for (p=0; p<ON2_Npoles; p++){
+      printf("p=%5d zp %15.12f %15.12f Rp %15.12f %15.12f\n",
+              p,ON2_zp[p].r,ON2_zp[p].i,ON2_Rp[p].r,ON2_Rp[p].i); 
+    }
+    */
+
+    /*
+    for (p=0; p<Npoles_ON2; p++){
+      Ep[p].r = 0.0;
+      Ep[p].i = zp[p].i/Beta;
+    }
+    */
+
+    /*********************************************
+      for calculation of energy density matrix
+    *********************************************/
+
+    /* calculation of kappa */
+
+    kappa = 0.0; 
+    for (p=0; p<ON2_Npoles_f; p++){
+      kappa += ON2_Rp[p].r; 
+    }
+    kappa = 4.0*kappa/Beta;
+
+    /* set up poles */
+
+    for (p=0; p<ON2_Npoles_f; p++){
+      ON2_zp_f[p] = ON2_zp[p];
+      ON2_Rp_f[p] = ON2_Rp[p];
+      ON2_method_f[p] = 1; 
+    }
+
+    /* for the zero-th order moment */
+
+    if (0){
+
+    R = 1.0e+8;
+
+    ON2_zp_f[ON2_Npoles_f].r = 0.0;
+    ON2_zp_f[ON2_Npoles_f].i = R;
+
+    ON2_Rp_f[ON2_Npoles_f].r = 0.25*R*(kappa - R);
+    ON2_Rp_f[ON2_Npoles_f].i = 0.25*R*(kappa + R);
+
+    ON2_method_f[ON2_Npoles_f] = 2; 
+    ON2_Npoles_f++;
+
+    /* for the 1st order moment */
+
+    ON2_zp_f[ON2_Npoles_f].r = -R;
+    ON2_zp_f[ON2_Npoles_f].i = 0.0;
+
+    ON2_Rp_f[ON2_Npoles_f].r = 0.25*R*(R - kappa);
+    ON2_Rp_f[ON2_Npoles_f].i = 0.25*R*(R - kappa);
+
+    ON2_method_f[ON2_Npoles_f] = 2; 
+    ON2_Npoles_f++;
+
+    }
+
+
+    if (0){
+
+    R1 = 1.0e+21;
+    R2 = 1.0e+7;
+
+    ON2_zp_f[ON2_Npoles_f].r = 0.0;
+    ON2_zp_f[ON2_Npoles_f].i = R1;
+
+    ON2_Rp_f[ON2_Npoles_f].r = 0.5*R1*R2;
+    ON2_Rp_f[ON2_Npoles_f].i = 0.5*R1*kappa;
+
+    ON2_method_f[ON2_Npoles_f] = 2; 
+    ON2_Npoles_f++;
+
+    /* for the 1st order moment */
+
+    ON2_zp_f[ON2_Npoles_f].r = 0.0;
+    ON2_zp_f[ON2_Npoles_f].i = R2;
+
+    ON2_Rp_f[ON2_Npoles_f].r =-0.5*R2*R2;
+    ON2_Rp_f[ON2_Npoles_f].i = 0.0;
+
+    ON2_method_f[ON2_Npoles_f] = 2; 
+    ON2_Npoles_f++;
+    }
+
+
+    R = 1.0e+7;
+
+    ON2_zp_f[ON2_Npoles_f].r = 0.0;
+    ON2_zp_f[ON2_Npoles_f].i = R*R;
+
+    ON2_Rp_f[ON2_Npoles_f].r = 0.5*R*R*R*R/(R-1.0);
+    ON2_Rp_f[ON2_Npoles_f].i = 0.5*R*R*R*kappa/(R-1.0);
+
+    ON2_method_f[ON2_Npoles_f] = 2; 
+    ON2_Npoles_f++;
+
+    /* for the 1st order moment */
+
+    ON2_zp_f[ON2_Npoles_f].r = 0.0;
+    ON2_zp_f[ON2_Npoles_f].i = R;
+
+    ON2_Rp_f[ON2_Npoles_f].r =-0.5*R*R*R/(R-1.0);
+    ON2_Rp_f[ON2_Npoles_f].i =-0.5*R*kappa/(R-1.0);
+
+    ON2_method_f[ON2_Npoles_f] = 2; 
+    ON2_Npoles_f++;
+
+  }
+
+
+  /*
+  {
+
+    double sum;
+    double complex alpha,weight;
+
+    sum = 0.0;
+ 
+    for (p=0; p<ON2_Npoles_f; p++){
+
+      if (ON2_method_f[p]==1){
+        alpha  = 0.0 + I*(ON2_zp_f[p].i/Beta);
+        weight = -2.0*ON2_Rp_f[p].r/Beta*alpha;
+      }
+      else {
+        alpha  = ON2_zp_f[p].r + I*ON2_zp_f[p].i;
+        weight = ON2_Rp_f[p].r + I*ON2_Rp_f[p].i;
+      }
+
+      sum += creal(weight*1.0/(alpha+2.0));
+
+      printf("%2d %18.15f\n",p,sum);
+
+    }
+
+    printf("\nsum = %18.15f\n",sum);
+
+  }
+
+  MPI_Finalize();
+  exit(0);
+  */
+
+
   /****************************************************
       Matsubara
   ****************************************************/
@@ -324,33 +512,39 @@ void InitV()
 {
   /******************************************************* 
    1 a.u.=2.4189*10^-2 fs, 1fs=41.341105 a.u. 
-   Atom weight trasformation: proton=1836.1526 a.u 
   ********************************************************/
 
-  double ax,ay,az,bx,by,bz,tmp,Wscale,sum;
+  double ax,ay,az,bx,by,bz,tmp,Wscale,sum,v;
   int j,Mc_AN,Gc_AN,ID,myid,numprocs;
 
-  /* MPI */
-  MPI_Comm_size(mpi_comm_level1,&numprocs);
-  MPI_Comm_rank(mpi_comm_level1,&myid);
+  Wscale = unified_atomic_mass_unit/electron_mass;
 
-  if ( (MD_switch==2 || MD_switch==9) && MD_Init_Velocity==0 ){ 
+  /* MPI */
+  MPI_Comm_size(MPI_COMM_WORLD1,&numprocs);
+  MPI_Comm_rank(MPI_COMM_WORLD1,&myid);
+
+
+  if ( (MD_switch==2 || MD_switch==9 || MD_switch==11 || MD_switch==14 || MD_switch==15)
+       && MD_Init_Velocity==0 ){
 
     if (myid==Host_ID){
       for (Gc_AN=1; Gc_AN<=atomnum; Gc_AN++){
 
-	/* modified by mari */
-	ax = rnd0to1();
-	ay = rnd0to1();
-	az = rnd0to1();
-	bx = rnd0to1();
-	by = rnd0to1();
-	bz = rnd0to1();
+        v = sqrt(3.0*kB*TempPara[1][2]/(Gxyz[Gc_AN][20]*Wscale*eV2Hartree));
 
-	tmp = -2.0*kB/eV2Hartree*TempPara[1][2]/(Gxyz[Gc_AN][20]*1836.1526);
-	Gxyz[Gc_AN][24] = sqrt(tmp*log(ax))*cos(2.0*PI*bx);
-	Gxyz[Gc_AN][25] = sqrt(tmp*log(ay))*cos(2.0*PI*by);
-	Gxyz[Gc_AN][26] = sqrt(tmp*log(az))*cos(2.0*PI*bz);
+	ax = rnd(1.0);
+	ay = rnd(1.0);
+	az = rnd(1.0);
+
+        tmp = 1.0/sqrt(ax*ax+ay*ay+az*az);
+
+        ax *= tmp; 
+        ay *= tmp; 
+        az *= tmp; 
+        
+	Gxyz[Gc_AN][24] = v*ax;
+	Gxyz[Gc_AN][25] = v*ay;
+	Gxyz[Gc_AN][26] = v*az;
 
 	Gxyz[Gc_AN][27] = Gxyz[Gc_AN][24];
 	Gxyz[Gc_AN][28] = Gxyz[Gc_AN][25];
@@ -364,7 +558,7 @@ void InitV()
 
     for (Gc_AN=1; Gc_AN<=atomnum; Gc_AN++){
       ID = G2ID[Gc_AN];
-      MPI_Bcast(&Gxyz[Gc_AN][24], 3, MPI_DOUBLE, Host_ID, mpi_comm_level1);
+      MPI_Bcast(&Gxyz[Gc_AN][24], 3, MPI_DOUBLE, Host_ID, MPI_COMM_WORLD1);
     }
   }
 
@@ -384,11 +578,9 @@ void InitV()
    calculate the initial Ukc:
 
    1 a.u.=2.4189*10^-2 fs, 1fs=41.341105 a.u. 
-   Atom weight trasformation: proton = 1836.1526 a.u 
   ***********************************************************/
 
   Ukc = 0.0;
-  Wscale = 1836.1526;
 
   for (Gc_AN=1; Gc_AN<=atomnum; Gc_AN++){
     sum = 0.0;
