@@ -6,17 +6,25 @@
   coupling constant J between the selected two atoms
 
   Log of jx.c:
-
      30/Aug/2003  Released by Myung Joon Han (supervised by Prof. J. Yu)
       7/Dec/2003  Modified by Taisuke Ozaki
+     03/Mar/2011  Modified by Fumiyuki Ishii for MPI 
 ***********************************************************************/
 
 #include <stdio.h>
 #include <stdlib.h>
 #include <math.h>
+#include <time.h>
+#include <sys/types.h>
+#include <sys/times.h>
+#include <sys/time.h> 
 #include "read_scfout.h"
 #include "lapack_prototypes.h"
 #include "f77func.h"
+#include "mpi.h"
+
+
+#define Host_ID       0         /* ID of the host CPU in MPI */
 
 #define printout  0    /* 0:off, 1:on */
 #define PI   3.1415926535897932384626
@@ -29,7 +37,6 @@ static void calc_J_cluster1(int First_Atom, int Second_Atom, int *MPF,
                             double ***C, double **ko,
                             int F_TNumOrbs, int F_TNumOrbs3);
 
-static void jx_band1(int argc, char *argv[]);
 static void k_inversion(int i,  int j,  int k, 
                         int mi, int mj, int mk, 
                         int *ii, int *ij, int *ik );
@@ -41,43 +48,14 @@ static void Hamiltonian_Band(double ****RH, dcomplex **H, int *MP,
 static void calc_J_band1(int First_Atom, int Second_Atom, int *MPF,
                          dcomplex ***C, double **ko,
                          int F_TNumOrbs, int F_TNumOrbs3, double Jmat[2]);
+static void dtime(double *t);
+
+
+MPI_Comm comm1;
+
+
 
 int main(int argc, char *argv[]) 
-{
-
-  char *s_vec[20];
-
-  printf("\n********************************************************************\n"); 
-  printf("********************************************************************\n"); 
-  printf(" jx: code for calculating an effective exchange coupling constant J \n"); 
-  printf(" Copyright (C), 2003, Myung Joon Han, Jaejun Yu, and Taisuke Ozaki \n"); 
-  printf(" This is free software, and you are welcome to         \n"); 
-  printf(" redistribute it under the constitution of the GNU-GPL.\n");
-  printf("********************************************************************\n"); 
-  printf("********************************************************************\n"); 
-
-  read_scfout(argv);
-
-  s_vec[0]="Recursion"; s_vec[1]="Cluster"; s_vec[2]="Band";
-  s_vec[3]="NEGF";      s_vec[4]="DC";      s_vec[5]="GDC";
-  s_vec[6]="Cluster2";
-
-  printf(" Previous eigenvalue solver = %s\n",s_vec[Solver-1]);
-  printf(" atomnum                    = %i\n",atomnum);
-  printf(" ChemP                      = %15.12f (Hartree)\n",ChemP);
-  printf(" E_Temp                     = %15.12f (K)\n",E_Temp);
-
-  if      (Solver==2 || Solver==7) jx_cluster1(argc, argv);
-  else if (Solver==3)              jx_band1(argc, argv);
-
-  exit(1);
-}
-
-
-
-
-
-void jx_band1(int argc, char *argv[]) 
 {
   static int ct_AN,h_AN,Gh_AN,i,j,k,TNO1,TNO2;  
   static int spin,Rn;
@@ -99,6 +77,18 @@ void jx_band1(int argc, char *argv[])
   double ***FullHks; /* full Hamiltonian */
   double **FullOLP;  /* full overlap     */
 
+  /* for MPI */
+  int numprocs,myid,ID,ID1, *arpo;
+  int T_knum,S_knum,E_knum,kloop0,kloop,num_kloop0;
+  double TStime,TEtime;
+  int Nk[4],atmij[3];
+  int *T_k_op; 
+  double *T_KGrids1, *T_KGrids2, *T_KGrids3, tmp4;
+  int *Tkmesh2ID;
+  double *JrTk;
+  double *JiTk;
+  /* for MPI end */
+
   static double snum_i,snum_j,snum_k;
   static double *KGrids1,*KGrids2,*KGrids3;
   static double Jmat[2],Jr,Ji;
@@ -116,231 +106,382 @@ void jx_band1(int argc, char *argv[])
   static int l,n,n2,n1,i1,j1,l1;
   static double sum,sumi,sum1;
 
-  /************************************************************************* 
+  /* MPI initialize */
+
+  MPI_Status stat;
+  MPI_Request request;
+  MPI_Init(&argc, &argv);
+  comm1 = MPI_COMM_WORLD;
+  MPI_Comm_size(comm1,&numprocs);
+  MPI_Comm_rank(comm1,&myid);
+
+  /* MPI initialize end */
+
+  char *s_vec[20];
+  dtime(&TStime); 
+
+  if (myid==Host_ID){
+    printf("\n********************************************************************\n"); 
+    printf("********************************************************************\n"); 
+    printf(" jx: code for calculating an effective exchange coupling constant J \n"); 
+    printf(" Copyright (C), 2003, Myung Joon Han, Jaejun Yu, and Taisuke Ozaki \n"); 
+    printf(" This is free software, and you are welcome to         \n"); 
+    printf(" redistribute it under the constitution of the GNU-GPL.\n");
+    printf("********************************************************************\n"); 
+    printf("********************************************************************\n"); 
+  }
+  read_scfout(argv);
+
+  s_vec[0]="Recursion"; s_vec[1]="Cluster"; s_vec[2]="Band";
+  s_vec[3]="NEGF";      s_vec[4]="DC";      s_vec[5]="GDC";
+  s_vec[6]="Cluster2";
+
+  if (myid==Host_ID){
+    printf(" Previous eigenvalue solver = %s\n",s_vec[Solver-1]);
+    printf(" atomnum                    = %i\n",atomnum);
+    printf(" ChemP                      = %15.12f (Hartree)\n",ChemP);
+    printf(" E_Temp                     = %15.12f (K)\n",E_Temp);
+  }
+  if      (Solver==2 || Solver==7) jx_cluster1(argc, argv);
+  else if (Solver==3) { 
+
+    /**** Calculation of J for bulk system  ****/
+
+    /************************************************************************* 
      Calculation flow in the evaluation of J:
 
      PART-1 : set full Hamiltonian and overlap
      PART-2 : the generalized eigenvalue problem HC = ESC
      PART-3 : Calculation of J
-  **************************************************************************/
+    **************************************************************************/
 
-  /************************************************************************* 
+    /************************************************************************* 
      PART-1 :  set full Hamiltonian and overlap
-  **************************************************************************/
+    **************************************************************************/
 
-  printf(" \nEvaluation of J based on band calculation\n");
+    if (myid==Host_ID){
+      printf(" \nEvaluation of J based on band calculation\n");
+    }
+    Full_atom = (int*)malloc(sizeof(int)*atomnum); 
+    MPF = (int*)malloc(sizeof(int)*(atomnum+1)); 
 
-  Full_atom = (int*)malloc(sizeof(int)*atomnum); 
-  MPF = (int*)malloc(sizeof(int)*(atomnum+1)); 
+    /* read Full_atom */
+    for (i=0; i<atomnum; i++){
+      Full_atom[i] = i+1;
+    }
 
-  /* read Full_atom */
-  for (i=0; i<atomnum; i++){
-    Full_atom[i] = i+1;
-  }
-
-  /*********************************************
+    /*********************************************
    make an array MPF which specifies the starting
    position of atom II in the martix such as
    a full but small Hamiltonian
-  *********************************************/
+    *********************************************/
 
-  Anum = 1;
-  for (i=0; i<atomnum; i++){
-    ct_AN = Full_atom[i];
-    Anum = Anum + Total_NumOrbs[ct_AN];    
-  }
-  F_TNumOrbs = Anum - 1;
-  F_TNumOrbs3 = F_TNumOrbs + 3;  
+    Anum = 1;
+    for (i=0; i<atomnum; i++){
+      ct_AN = Full_atom[i];
+      Anum = Anum + Total_NumOrbs[ct_AN];    
+    }
+    F_TNumOrbs = Anum - 1;
+    F_TNumOrbs3 = F_TNumOrbs + 3;  
 
-  /*********************************************
+    /*********************************************
          get knum_i, knum_j, and knum_k
-  *********************************************/
+    *********************************************/
 
-  do {
-    printf(" Specify the number of k-grids (e.g 4 4 4)  ");
-    scanf("%d %d %d",&knum_i, &knum_j, &knum_k);
-    if ( knum_i<=0 || knum_j<=0 || knum_k<=0 ){
-      printf("invalid number\n");
-      knum_switch = 0;
-    }
-    else{
-      knum_switch = 1;
-    }
-  } while(knum_switch==0);
+    do {
+      if (myid==Host_ID){
+	printf(" Specify the number of k-grids (e.g 4 4 4)  \n");
+	scanf("%d %d %d",&Nk[1],&Nk[2],&Nk[3]);
+	printf(" Nk1, Nk2, Nk3 = %d %d %d  \n", Nk[1],Nk[2], Nk[3]);
+      }
+      MPI_Bcast(Nk, 4, MPI_INT, 0, comm1); 
+      knum_i=Nk[1];
+      knum_j=Nk[2];
+      knum_k=Nk[3];
 
-  /****************************************************
+
+      if ( knum_i<=0 || knum_j<=0 || knum_k<=0 ){
+	printf("invalid number\n");
+	knum_switch = 0;
+      }
+      else{
+	knum_switch = 1;
+      }
+    } while(knum_switch==0);
+
+    /****************************************************
    allocation of arrays:
 
    double    KGrids1[knum_i]
    double    KGrids2[knum_j]
    double    KGrids3[knum_k]
    double    k_op[knum_i][knum_j][knum_k]
-  ****************************************************/
+    ****************************************************/
 
-  ko = (double**)malloc(sizeof(double*)*(SpinP_switch+1));
-  for (i=0; i<=SpinP_switch; i++){
-    ko[i] = (double*)malloc(sizeof(double)*F_TNumOrbs3);
-  }
+    ko = (double**)malloc(sizeof(double*)*(SpinP_switch+1));
+    for (i=0; i<=SpinP_switch; i++){
+      ko[i] = (double*)malloc(sizeof(double)*F_TNumOrbs3);
+    }
 
-  H = (dcomplex**)malloc(sizeof(dcomplex*)*F_TNumOrbs3);
-  for (j=0; j<F_TNumOrbs3; j++){
-    H[j] = (dcomplex*)malloc(sizeof(dcomplex)*F_TNumOrbs3);
-  }
-
-  S = (dcomplex**)malloc(sizeof(dcomplex*)*F_TNumOrbs3);
-  for (i=0; i<F_TNumOrbs3; i++){
-    S[i] = (dcomplex*)malloc(sizeof(dcomplex)*F_TNumOrbs3);
-  }
-
-  M1 = (double*)malloc(sizeof(double)*F_TNumOrbs3);
-
-  C = (dcomplex**)malloc(sizeof(dcomplex*)*F_TNumOrbs3);
-  for (j=0; j<F_TNumOrbs3; j++){
-    C[j] = (dcomplex*)malloc(sizeof(dcomplex)*F_TNumOrbs3);
-  }
-
-  Coes = (dcomplex***)malloc(sizeof(dcomplex**)*(SpinP_switch+1));
-  for (spin=0; spin<=SpinP_switch; spin++){
-    Coes[spin] = (dcomplex**)malloc(sizeof(dcomplex*)*F_TNumOrbs3);
+    H = (dcomplex**)malloc(sizeof(dcomplex*)*F_TNumOrbs3);
     for (j=0; j<F_TNumOrbs3; j++){
-      Coes[spin][j] = (dcomplex*)malloc(sizeof(dcomplex)*F_TNumOrbs3);
+      H[j] = (dcomplex*)malloc(sizeof(dcomplex)*F_TNumOrbs3);
     }
-  }
 
-  KGrids1 = (double*)malloc(sizeof(double)*knum_i);
-  KGrids2 = (double*)malloc(sizeof(double)*knum_j);
-  KGrids3 = (double*)malloc(sizeof(double)*knum_k);
-
-  k_op=(int***)malloc(sizeof(int**)*(knum_i));
-  for (i=0;i<knum_i; i++) {
-    k_op[i]=(int**)malloc(sizeof(int*)*(knum_j));
-    for (j=0;j<knum_j; j++) {
-      k_op[i][j]=(int*)malloc(sizeof(int)*(knum_k));
+    S = (dcomplex**)malloc(sizeof(dcomplex*)*F_TNumOrbs3);
+    for (i=0; i<F_TNumOrbs3; i++){
+      S[i] = (dcomplex*)malloc(sizeof(dcomplex)*F_TNumOrbs3);
     }
-  }
 
-  /*********************************************
+    M1 = (double*)malloc(sizeof(double)*F_TNumOrbs3);
+
+    C = (dcomplex**)malloc(sizeof(dcomplex*)*F_TNumOrbs3);
+    for (j=0; j<F_TNumOrbs3; j++){
+      C[j] = (dcomplex*)malloc(sizeof(dcomplex)*F_TNumOrbs3);
+    }
+
+    Coes = (dcomplex***)malloc(sizeof(dcomplex**)*(SpinP_switch+1));
+    for (spin=0; spin<=SpinP_switch; spin++){
+      Coes[spin] = (dcomplex**)malloc(sizeof(dcomplex*)*F_TNumOrbs3);
+      for (j=0; j<F_TNumOrbs3; j++){
+	Coes[spin][j] = (dcomplex*)malloc(sizeof(dcomplex)*F_TNumOrbs3);
+      }
+    }
+
+    KGrids1 = (double*)malloc(sizeof(double)*knum_i);
+    KGrids2 = (double*)malloc(sizeof(double)*knum_j);
+    KGrids3 = (double*)malloc(sizeof(double)*knum_k);
+
+    k_op=(int***)malloc(sizeof(int**)*(knum_i));
+    for (i=0;i<knum_i; i++) {
+      k_op[i]=(int**)malloc(sizeof(int*)*(knum_j));
+      for (j=0;j<knum_j; j++) {
+	k_op[i][j]=(int*)malloc(sizeof(int)*(knum_k));
+      }
+    }
+
+    /*********************************************
    make an array MPF which specifies the starting
    position of atom II in the martix such as
    a full but small Hamiltonian
-  *********************************************/
+    *********************************************/
 
-  snum_i = (double)knum_i;
-  snum_j = (double)knum_j;
-  snum_k = (double)knum_k;
+    snum_i = (double)knum_i;
+    snum_j = (double)knum_j;
+    snum_k = (double)knum_k;
 
-  for (i=0; i<=(knum_i-1); i++){
-    if (knum_i==1) k1 = 0.0;
-    else           k1 = -0.5 + (2.0*(double)i+1.0)/(2.0*snum_i);
-    KGrids1[i]=k1;
-  }
-  for (i=0; i<=(knum_j-1); i++){
-    if (knum_j==1) k1 = 0.0;
-    else           k1 = -0.5 + (2.0*(double)i+1.0)/(2.0*snum_j);
-    KGrids2[i]=k1;
-  }
-  for (i=0; i<=(knum_k-1); i++){
-    if (knum_k==1) k1 = 0.0;
-    else           k1 = -0.5 + (2.0*(double)i+1.0)/(2.0*snum_k);
-    KGrids3[i]=k1;
-  }
+    for (i=0; i<=(knum_i-1); i++){
+      if (knum_i==1) k1 = 0.0;
+      else           k1 = -0.5 + (2.0*(double)i+1.0)/(2.0*snum_i);
+      KGrids1[i]=k1;
+    }
+    for (i=0; i<=(knum_j-1); i++){
+      if (knum_j==1) k1 = 0.0;
+      else           k1 = -0.5 + (2.0*(double)i+1.0)/(2.0*snum_j);
+      KGrids2[i]=k1;
+    }
+    for (i=0; i<=(knum_k-1); i++){
+      if (knum_k==1) k1 = 0.0;
+      else           k1 = -0.5 + (2.0*(double)i+1.0)/(2.0*snum_k);
+      KGrids3[i]=k1;
+    }
 
-  if (printout==1){
-    printf("   KGrids1: ");
-    for (i=0;i<=knum_i-1;i++) printf("%f ",KGrids1[i]);
-    printf("\n");
-    printf("   KGrids2: ");
-    for (i=0;i<=knum_j-1;i++) printf("%f ",KGrids2[i]);
-    printf("\n");
-    printf("   KGrids3: ");
-    for (i=0;i<=knum_k-1;i++) printf("%f ",KGrids3[i]);
-    printf("\n");
-  }
 
-  /**************************************************************
+
+
+    if (printout==1){
+      printf("   KGrids1: ");
+      for (i=0;i<=knum_i-1;i++) printf("%f ",KGrids1[i]);
+      printf("\n");
+      printf("   KGrids2: ");
+      for (i=0;i<=knum_j-1;i++) printf("%f ",KGrids2[i]);
+      printf("\n");
+      printf("   KGrids3: ");
+      for (i=0;i<=knum_k-1;i++) printf("%f ",KGrids3[i]);
+      printf("\n");
+    }
+
+    /**************************************************************
    k_op[i][j][k]: weight of DOS 
                  =0   no calc.
                  =1   G-point
                  =2   which has k<->-k point
    Now , only the relation, E(k)=E(-k), is used. 
-  *************************************************************/
+    *************************************************************/
 
-  for (i=0;i<=knum_i-1;i++) {
-   for (j=0;j<=knum_j-1;j++) {
-    for (k=0;k<=knum_k-1;k++) {
-      k_op[i][j][k]=-999;
-    }
-   }
-  }
-  for (i=0;i<=knum_i-1;i++) {
-   for (j=0;j<=knum_j-1;j++) {
-    for (k=0;k<=knum_k-1;k++) {
-     if ( k_op[i][j][k]==-999 ) {
-      k_inversion(i,j,k,knum_i,knum_j,knum_k,&ii,&ij,&ik);
-      if ( i==ii && j==ij && k==ik ) {
-	/*  printf("* : %d %d %d (%f %f %f)\n",i,j,k,
-                KGrids1[i], KGrids2[j], KGrids3[k]);
-	*/
-
-        k_op[i][j][k]=1;
+    for (i=0;i<=knum_i-1;i++) {
+      for (j=0;j<=knum_j-1;j++) {
+	for (k=0;k<=knum_k-1;k++) {
+	  k_op[i][j][k]=-999;
+	}
       }
-      else {
-        k_op[i][j][k]=2;
-        k_op[ii][ij][ik]=0;
+    }
+    for (i=0;i<=knum_i-1;i++) {
+      for (j=0;j<=knum_j-1;j++) {
+	for (k=0;k<=knum_k-1;k++) {
+	  if ( k_op[i][j][k]==-999 ) {
+	    k_inversion(i,j,k,knum_i,knum_j,knum_k,&ii,&ij,&ik);
+	    if ( i==ii && j==ij && k==ik ) {
+	      /*  printf("* : %d %d %d (%f %f %f)\n",i,j,k,
+		  KGrids1[i], KGrids2[j], KGrids3[k]);
+	      */
+
+	      k_op[i][j][k]=1;
+	    }
+	    else {
+	      k_op[i][j][k]=2;
+	      k_op[ii][ij][ik]=0;
+	    }
+	  }
+	} /* k */
+      } /* j */
+    } /* i */
+
+
+    /***********************************
+       one-dimentionalize for MPI
+    ************************************/
+
+
+    T_knum = 0;
+    for (i=0; i<=(knum_i-1); i++){
+      for (j=0; j<=(knum_j-1); j++){
+	for (k=0; k<=(knum_k-1); k++){
+          T_knum++;
+	}
       }
-     }
-    } /* k */
-   } /* j */
-  } /* i */
-
-  /****************************************************
-   start calc J
-  ****************************************************/
-
-  printf("\n");
-
-  end_switch = 0;
-  do {
-
-    /* specify two atoms */ 
-    printf(" Specify two atoms (e.g 1 2, quit: 0 0)  ");
-    scanf("%d %d",&First_Atom,&Second_Atom);
-    if      (First_Atom==0 && Second_Atom==0) end_switch = 1;       
-    else if (First_Atom==0 && Second_Atom!=0 || 
-             First_Atom!=0 && Second_Atom==0){
-      printf(" Invalid atom\n");
     }
-    else if (atomnum<First_Atom || atomnum<Second_Atom){
-      printf(" Invalid atom\n");
+
+    Tkmesh2ID= (int*)malloc(sizeof(int)*T_knum);
+    JrTk = (double*)malloc(sizeof(double)*T_knum);
+    JiTk = (double*)malloc(sizeof(double)*T_knum);
+    T_KGrids1 = (double*)malloc(sizeof(double)*T_knum);
+    T_KGrids2 = (double*)malloc(sizeof(double)*T_knum);
+    T_KGrids3 = (double*)malloc(sizeof(double)*T_knum);
+    arpo = (int*)malloc(sizeof(int)*numprocs);
+
+    /* set T_KGrids1,2,3 and T_k_op */
+
+    T_knum = 0;
+    for (i=0; i<=(knum_i-1); i++){
+      for (j=0; j<=(knum_j-1); j++){
+	for (k=0; k<=(knum_k-1); k++){
+
+	  T_KGrids1[T_knum] = KGrids1[i];
+	  T_KGrids2[T_knum] = KGrids2[j];
+	  T_KGrids3[T_knum] = KGrids3[k];
+	  JrTk[T_knum]=0.0; 
+	  JiTk[T_knum]=0.0; 
+
+          T_knum++;
+	}
+      }
     }
-    else{
 
-      /****************************************************
-       calculation of J
-      ****************************************************/
+    /****  allocate k-point  into proccessors *****/
 
-      /* loop for k-grids */
+    if (T_knum<=myid){
+      S_knum = -10;
+      E_knum = -100;
+      num_kloop0 = 1;
+    }
+    else if (T_knum<numprocs) {
+      S_knum = myid;
+      E_knum = myid;
+      num_kloop0 = 1;
+    }
+    else {
+      tmp4 = (double)T_knum/(double)numprocs; 
+      num_kloop0 = (int)tmp4 + 1;
+      S_knum = (int)((double)myid*(tmp4+0.0001)); 
+      E_knum = (int)((double)(myid+1)*(tmp4+0.0001)) - 1;
+      if (myid==(numprocs-1)) E_knum = T_knum - 1;
+      if (E_knum<0)           E_knum = 0;
+    }
 
-      Jr = 0.0;
-      Ji = 0.0;
+    /****************************************************
+                         start calc J
+    ****************************************************/
 
-      for (i=0; i<=(knum_i-1); i++){
-	k1 = KGrids1[i];
+    /** printf("\n"); **/
 
-	for (j=0; j<=(knum_j-1); j++){
-	  k2 = KGrids2[j] ;
+    end_switch = 0;
+    do {
 
-	  for (k=0; k<=(knum_k-1); k++){
-	    k3 = KGrids3[k] ;
+      /* specify two atoms */ 
+      if (myid==Host_ID){
 
-	    /* if (k_op[i][j][k]<=0) goto NEXT_Chem;  */
+	printf("\n Specify two atoms (e.g 1 2, quit: 0 0) \n");
+	scanf("%d %d",&atmij[1],&atmij[2]);
+
+	if (atmij[1] > 0){
+	  printf(" Atom:i,j = %d, %d \n",atmij[1], atmij[2]);
+	} 
+	else{ 
+          printf(" ******* exit \n");
+        }
+      }
+
+      MPI_Bcast(atmij, 3, MPI_INT, 0, comm1); 
+      MPI_Barrier(comm1); 
+      First_Atom  = atmij[1];
+      Second_Atom = atmij[2];
+
+      if      (First_Atom==0 && Second_Atom==0) {
+        end_switch = 1;       
+        MPI_Finalize();
+        exit(1);
+      }
+      else if (First_Atom==0 && Second_Atom!=0 || 
+	       First_Atom!=0 && Second_Atom==0){
+	printf(" Invalid atom\n");
+      }
+      else if (atomnum<First_Atom || atomnum<Second_Atom){
+	printf(" Invalid atom\n");
+      }
+      else{
+
+	/****************************************************
+                          calculation of J
+	****************************************************/
+
+	/* loop for k-grids */
+
+	Jr = 0.0;
+	Ji = 0.0;
+
+	for (kloop0=0; kloop0<num_kloop0; kloop0++){
+
+	  kloop = S_knum + kloop0;
+
+	  arpo[myid] = -1;
+	  if (S_knum<=kloop && kloop<=E_knum) arpo[myid] = kloop;
+	  for (ID=0; ID<numprocs; ID++){
+	    MPI_Bcast(&arpo[ID], 1, MPI_INT, ID, comm1);
+	  }
+	  if (myid==Host_ID){
+          }
+
+	  kloop = arpo[myid];
+
+	  for (ID=0; ID<numprocs; ID++){
+	    Tkmesh2ID[kloop] = ID;
+	    MPI_Bcast(Tkmesh2ID, T_knum, MPI_INT, ID, comm1);
+	  }
+
+	  if (0<=kloop){
+
+	    k1 = T_KGrids1[kloop];
+	    k2 = T_KGrids2[kloop];
+	    k3 = T_KGrids3[kloop];
+	    JrTk[kloop]=0.0;           
+	    JiTk[kloop]=0.0;           
 
 	    /****************************************************
 		 MPF indicates the starting position of
 			 atom i in arraies H and S
 	    ****************************************************/
 
-
 	    Overlap_Band(OLP,S,MPF,k1,k2,k3);
 
 	    n = S[0][0].r;
@@ -422,7 +563,6 @@ void jx_band1(int argc, char *argv[])
 	      }
 
 
-	    NEXT_Chem: ; 
 	    } /* spin */
 
 	    /************************************************************************* 
@@ -436,28 +576,46 @@ void jx_band1(int argc, char *argv[])
 		3) calculation of J from V'
 	    **************************************************************************/
 
-           /* calculation of J */ 
+	    /* calculation of J */ 
 
-           calc_J_band1(First_Atom, Second_Atom, MPF, Coes, ko,
-                        n1, F_TNumOrbs3, Jmat);
+	    calc_J_band1(First_Atom, Second_Atom, MPF, Coes, ko,
+			 n1, F_TNumOrbs3, Jmat);
 
-           Jr += Jmat[0]; 
-           Ji += Jmat[1]; 
+	    JrTk[kloop] += Jmat[0];
+	    JiTk[kloop] += Jmat[1];
 
-	  } /* k */
-	} /* j */
-      } /* i */
+	    /*  printf("\n calculate kloop, Jr=%d, %f10.5 \n",kloop, JrTk[kloop]); */
+	  } /* if(kloop =>0) */
+        } /* kloop0  for MPI */
 
-      Jr = Jr/(double)(knum_i*knum_j*knum_k);
 
-      printf(" J_ij between %ith atom and %ith atom is %15.12f cm^{-1}\n",
-	     First_Atom, Second_Atom, Jr);
+  
+	MPI_Barrier(comm1); 
+	kloop=0;
+	for (i=0; i<=(knum_i-1); i++){
+	  for (j=0; j<=(knum_j-1); j++){
+	    for (k=0; k<=(knum_k-1); k++){
+	      ID1 = Tkmesh2ID[kloop];
+	      MPI_Bcast(&JrTk[kloop], 1, MPI_DOUBLE, ID1, comm1);
+	      MPI_Bcast(&JiTk[kloop], 1, MPI_DOUBLE, ID1, comm1);
+	      kloop++;
+	    }
+	  }
+	}
+     
 
-    } /* else */
+	for (i=0; i<T_knum; i++){
+          Jr += JrTk[i];
+	}
+	Jr = Jr/(double)(knum_i*knum_j*knum_k);
+	if (myid==Host_ID){
+	  printf("\n J_ij between %ith atom and %ith atom is %15.12f cm^{-1}\n",
+		 First_Atom, Second_Atom, Jr);
+	}
+      } /* else */
 
-  } while(end_switch==0);
-
-  /*********************************************
+    } while(end_switch==0);
+    /*********************************************
     freeing of arrays:
 
     int *Full_atom;
@@ -469,55 +627,72 @@ void jx_band1(int argc, char *argv[])
     double **B;
     double ***C;
     double **D;
-  *********************************************/
+    *********************************************/
 
-  free(Full_atom);
-  free(MPF);
+    free(Full_atom);
+    free(MPF);
 
-  for (spin=0; spin<=SpinP_switch; spin++){
-    free(ko[spin]);
-  }
-  free(ko);
+    for (spin=0; spin<=SpinP_switch; spin++){
+      free(ko[spin]);
+    }
+    free(ko);
 
-  for (j=0; j<F_TNumOrbs3; j++){
-    free(H[j]);
-  }
-  free(H);
-
-  for (i=0; i<F_TNumOrbs3; i++){
-    free(S[i]);
-  }
-  free(S);
-
-  free(M1);
-
-  for (j=0; j<F_TNumOrbs3; j++){
-    free(C[j]);
-  }
-  free(C);
-
-  for (spin=0; spin<=SpinP_switch; spin++){
     for (j=0; j<F_TNumOrbs3; j++){
-      free(Coes[spin][j]);
+      free(H[j]);
     }
-    free(Coes[spin]);
-  }
-  free(Coes);
+    free(H);
 
-  free(KGrids1);
-  free(KGrids2);
-  free(KGrids3);
-
-  for (i=0;i<knum_i; i++) {
-    for (j=0;j<knum_j; j++) {
-      free(k_op[i][j]);
+    for (i=0; i<F_TNumOrbs3; i++){
+      free(S[i]);
     }
-    free(k_op[i]);
+    free(S);
+
+    free(M1);
+
+    for (j=0; j<F_TNumOrbs3; j++){
+      free(C[j]);
+    }
+    free(C);
+
+    for (spin=0; spin<=SpinP_switch; spin++){
+      for (j=0; j<F_TNumOrbs3; j++){
+	free(Coes[spin][j]);
+      }
+      free(Coes[spin]);
+    }
+    free(Coes);
+
+    free(KGrids1);
+    free(KGrids2);
+    free(KGrids3);
+
+    for (i=0;i<knum_i; i++) {
+      for (j=0;j<knum_j; j++) {
+	free(k_op[i][j]);
+      }
+      free(k_op[i]);
+    }
+    free(k_op);
   }
-  free(k_op);
 
-}  
+  /* print message */
 
+  MPI_Barrier(comm1); 
+
+  dtime(&TEtime); 
+  /* if (myid==Host_ID){ */
+  printf(" \nElapsed time = %lf (s) for myid=%3d\n",TEtime-TStime,myid);fflush(stdout);
+  /* printf(" \nElapsed time = %lf (s) with processors=%3d\n",TEtime-TStime,numprocs);fflush(stdout); */
+  /*  } */
+  MPI_Barrier(comm1); 
+  /** printf(" \nThe calculation was finished normally in myid=%2d.\n",myid);fflush(stdout); **/
+
+  /* MPI_Finalize */
+
+  MPI_Finalize();
+
+  exit(1);
+}
 
 
 void jx_cluster1(int argc, char *argv[]) 
@@ -549,6 +724,19 @@ void jx_cluster1(int argc, char *argv[])
   static double **B, ***C, **D;
   static double sum,sum1;                                                      
 
+  int myid,numprocs;
+
+  MPI_Comm_size(comm1,&numprocs);
+  MPI_Comm_rank(comm1,&myid);
+
+  if (1<numprocs){
+    if (myid==Host_ID){
+      printf("\nMPI parallelization is not supported for the cluster mode.\n\n");
+    }
+    MPI_Finalize();
+    exit(1);
+  }
+
   /************************************************************************* 
      Calculation flow in the evaluation of J:
 
@@ -705,9 +893,9 @@ void jx_cluster1(int argc, char *argv[])
   Eigen_lapack(FullOLP,ko[0],F_TNumOrbs);
 
   /*
-  for (l=1; l<=F_TNumOrbs; l++){
+    for (l=1; l<=F_TNumOrbs; l++){
     printf("l ko %2d %15.12f\n",l,ko[0][l]);
-  }
+    }
   */
 
   /* check ill-conditioned eigenvalues */
@@ -830,6 +1018,7 @@ void jx_cluster1(int argc, char *argv[])
     /* specify two atoms */ 
     printf(" Specify two atoms (e.g 1 2, quit: 0 0)  ");
     scanf("%d %d",&First_Atom,&Second_Atom);
+
     if      (First_Atom==0 && Second_Atom==0) end_switch = 1;       
     else if (First_Atom==0 && Second_Atom!=0 || 
              First_Atom!=0 && Second_Atom==0){
@@ -1169,10 +1358,10 @@ void calc_J_band1(int First_Atom, int Second_Atom, int *MPF, dcomplex ***C, doub
 	      IVi_i[ii] = sum_i ;
 	    }
 	    sum1_r += IVi_r[ll]*C[spin2][(ll-1+MPF[Choo_atom[0]])][j].r
-                    - IVi_i[ll]*C[spin2][(ll-1+MPF[Choo_atom[0]])][j].i ;
+	      - IVi_i[ll]*C[spin2][(ll-1+MPF[Choo_atom[0]])][j].i ;
 
 	    sum1_i += IVi_r[ll]*C[spin2][(ll-1+MPF[Choo_atom[0]])][j].i
-                    + IVi_i[ll]*C[spin2][(ll-1+MPF[Choo_atom[0]])][j].r ;
+	      + IVi_i[ll]*C[spin2][(ll-1+MPF[Choo_atom[0]])][j].r ;
 	  }
 	  VVi_r[spin1][spin2][i][j] = sum1_r ;
 	  VVi_i[spin1][spin2][i][j] = sum1_i ;
@@ -1203,12 +1392,12 @@ void calc_J_band1(int First_Atom, int Second_Atom, int *MPF, dcomplex ***C, doub
 	    }
 
 	    sum1_r = sum1_r
-                    + IVj_r[ll]*C[spin2][(ll-1+MPF[Choo_atom[1]])][j].r 
-	            - IVj_i[ll]*C[spin2][(ll-1+MPF[Choo_atom[1]])][j].i ; 
+	      + IVj_r[ll]*C[spin2][(ll-1+MPF[Choo_atom[1]])][j].r 
+	      - IVj_i[ll]*C[spin2][(ll-1+MPF[Choo_atom[1]])][j].i ; 
 
 	    sum1_i = sum1_i
-                    + IVj_r[ll]*C[spin2][(ll-1+MPF[Choo_atom[1]])][j].i 
-                    + IVj_i[ll]*C[spin2][(ll-1+MPF[Choo_atom[1]])][j].r ;
+	      + IVj_r[ll]*C[spin2][(ll-1+MPF[Choo_atom[1]])][j].i 
+	      + IVj_i[ll]*C[spin2][(ll-1+MPF[Choo_atom[1]])][j].r ;
 	  }
 	  VVj_r[spin1][spin2][i][j] = sum1_r ;
 	  VVj_i[spin1][spin2][i][j] = sum1_i ;
@@ -1244,12 +1433,12 @@ void calc_J_band1(int First_Atom, int Second_Atom, int *MPF, dcomplex ***C, doub
       dFftn = Fftn[0][i] - Fftn[1][j] ;
       dko = ko[1][j] - ko[0][i] ;
       J_ij_r = J_ij_r + 0.5*dFftn *
-               (  VVj_r[0][1][i][j] * VVi_r[1][0][j][i]
-                - VVj_i[0][1][i][j] * VVi_i[1][0][j][i] ) / dko ;
+	(  VVj_r[0][1][i][j] * VVi_r[1][0][j][i]
+	   - VVj_i[0][1][i][j] * VVi_i[1][0][j][i] ) / dko ;
 
       J_ij_i = J_ij_i + 0.5*dFftn *
-               (  VVj_r[0][1][i][j] * VVi_i[1][0][j][i]
-                + VVj_i[0][1][i][j] * VVi_r[1][0][j][i] ) / dko ;
+	(  VVj_r[0][1][i][j] * VVi_i[1][0][j][i]
+	   + VVj_i[0][1][i][j] * VVi_r[1][0][j][i] ) / dko ;
     }
   }
 
@@ -1657,7 +1846,7 @@ void calc_J_cluster1(int First_Atom, int Second_Atom, int *MPF, double ***C, dou
   /* printing the final result : J_ij*/
 
   printf(" J_ij between %ith atom and %ith atom is %15.12f cm^{-1}\n",
-          Choo_atom[0], Choo_atom[1], J_ij);
+	 Choo_atom[0], Choo_atom[1], J_ij);
 
   /*********************************************
     freeing of arrays:
@@ -1780,9 +1969,9 @@ void k_inversion(int i,  int j,  int k,
                  int mi, int mj, int mk, 
                  int *ii, int *ij, int *ik )
 {
-       *ii= mi-i-1;
-       *ij= mj-j-1;
-       *ik= mk-k-1;
+  *ii= mi-i-1;
+  *ij= mj-j-1;
+  *ik= mk-k-1;
 }
 
 
@@ -1974,11 +2163,11 @@ void Hamiltonian_Band(double ****RH, dcomplex **H, int *MP,
 
 void Eigen_lapack(double **a, double *ko, int n)
 {
- /* input:  n;
-    input:  a[n][n];  matrix A
+  /* input:  n;
+     input:  a[n][n];  matrix A
 
-    output: a[n][n]; eigevectors
-    output: ko[n];   eigenvalues  */
+     output: a[n][n]; eigevectors
+     output: ko[n];   eigenvalues  */
     
   static char *name="Eigen_lapack";
 
@@ -2012,7 +2201,7 @@ void Eigen_lapack(double **a, double *ko, int n)
 
   for (i=0;i<n;i++) {
     for (j=0;j<n;j++) {
-       A[i*n+j]= a[i+1][j+1];
+      A[i*n+j]= a[i+1][j+1];
     }
   }
 
@@ -2020,7 +2209,7 @@ void Eigen_lapack(double **a, double *ko, int n)
   printf("A=\n");
   for (i=0;i<n;i++) {
     for (j=0;j<n;j++) {
-       printf("%f ",A[i*n+j]);
+      printf("%f ",A[i*n+j]);
     }
     printf("\n");
   }
@@ -2028,14 +2217,14 @@ void Eigen_lapack(double **a, double *ko, int n)
 #endif
 
   F77_NAME(dsyevx,DSYEVX)( JOBZ, RANGE, UPLO, &n, A, &LDA, &VL, &VU, &IL, &IU,
-           &ABSTOL, &M, ko, Z, &LDZ, WORK, &LWORK, IWORK,
-           IFAIL, &INFO ); 
+			   &ABSTOL, &M, ko, Z, &LDZ, WORK, &LWORK, IWORK,
+			   IFAIL, &INFO ); 
 
   /* store eigenvectors */
   for (i=0;i<n;i++) {
     for (j=0;j<n;j++) {
-     /*  a[i+1][j+1]= Z[i*n+j]; */
-       a[j+1][i+1]= Z[i*n+j];
+      /*  a[i+1][j+1]= Z[i*n+j]; */
+      a[j+1][i+1]= Z[i*n+j];
 
     }
   }
@@ -2046,11 +2235,11 @@ void Eigen_lapack(double **a, double *ko, int n)
   }
 
   if (INFO>0) {
-     printf("\n%s: error in dsyevx_, info=%d\n\n",name,INFO);
+    printf("\n%s: error in dsyevx_, info=%d\n\n",name,INFO);
   }
   if (INFO<0) {
-     printf("%s: info=%d\n",name,INFO);
-     exit(10);
+    printf("%s: info=%d\n",name,INFO);
+    exit(10);
   }
    
   free(IFAIL); free(IWORK); free(WORK); free(Z); free(A);
@@ -2062,8 +2251,8 @@ void Eigen_lapack(double **a, double *ko, int n)
 
 
 /*
-void EigenBand_lapack(dcomplex **A, double *W, int N)
-{
+  void EigenBand_lapack(dcomplex **A, double *W, int N)
+  {
   static char *JOBZ="V";
   static char *RANGE="A";
   static char *UPLO="L";
@@ -2087,9 +2276,9 @@ void EigenBand_lapack(dcomplex **A, double *W, int N)
   A0=(dcomplex*)malloc(sizeof(dcomplex)*N*N);
 
   for (i=1;i<=N;i++) {
-    for (j=1;j<=N;j++) {
-      A0[(j-1)*N+i-1] = A[i][j];
-    }
+  for (j=1;j<=N;j++) {
+  A0[(j-1)*N+i-1] = A[i][j];
+  }
   }
 
   LWORK=3*N; 
@@ -2101,29 +2290,29 @@ void EigenBand_lapack(dcomplex **A, double *W, int N)
 
 
   F77_NAME(zheevx,ZHEEVX)( JOBZ, RANGE, UPLO, &N, A0, &LDA, &VL, &VU, &IL, &IU,
-           &ABSTOL, &M, W, Z, &LDZ, WORK, &LWORK, RWORK, IWORK,
-           IFAIL, &INFO );
+  &ABSTOL, &M, W, Z, &LDZ, WORK, &LWORK, RWORK, IWORK,
+  IFAIL, &INFO );
 
   if (INFO!=0) {
-     printf("************************************************************\n");
-     printf("  EigenBand_lapack: zheevx_()=%d\n",INFO);
-     printf("************************************************************\n");
-     exit(10);
+  printf("************************************************************\n");
+  printf("  EigenBand_lapack: zheevx_()=%d\n",INFO);
+  printf("************************************************************\n");
+  exit(10);
   }
 
   for (i=1;i<=N;i++) {
-    for (j=1;j<=N;j++) {
-      A[i][j].r = Z[(j-1)*N+i-1].r;
-      A[i][j].i = Z[(j-1)*N+i-1].i;
-    }
+  for (j=1;j<=N;j++) {
+  A[i][j].r = Z[(j-1)*N+i-1].r;
+  A[i][j].i = Z[(j-1)*N+i-1].i;
+  }
   }
 
   for (i=N;i>=1;i--) {
-    W[i] =W[i-1];
+  W[i] =W[i-1];
   }
 
   for (i=1; i<=N; i++) {
-    printf("i=%2d W=%15.12f\n",i,W[i]);
+  printf("i=%2d W=%15.12f\n",i,W[i]);
   }
 
   free(A0); 
@@ -2132,7 +2321,7 @@ void EigenBand_lapack(dcomplex **A, double *W, int N)
   free(RWORK); 
   free(IWORK); 
   free(IFAIL); 
-}
+  }
 */
 
 
@@ -2163,10 +2352,10 @@ void EigenBand_lapack(dcomplex **A, double *W, int N)
   F77_NAME(zheev,ZHEEV)(JOBZ,UPLO, &N, A0, &N, W, WORK, &LWORK, RWORK, &INFO  );
 
   if (INFO!=0) {
-     printf("************************************************************\n");
-     printf("  EigenBand_lapack: cheev_()=%d\n",INFO);
-     printf("************************************************************\n");
-     exit(10);
+    printf("************************************************************\n");
+    printf("  EigenBand_lapack: cheev_()=%d\n",INFO);
+    printf("************************************************************\n");
+    exit(10);
   }
 
   for (i=1;i<=N;i++) {
@@ -2182,3 +2371,24 @@ void EigenBand_lapack(dcomplex **A, double *W, int N)
 
   free(A0); free(RWORK); free(WORK); 
 }
+
+void dtime(double *t)
+{
+  /* real time */
+  struct timeval timev;
+  gettimeofday(&timev, NULL);
+  *t = timev.tv_sec + (double)timev.tv_usec*1e-6;
+
+  /* user time + system time */
+  /*
+    float tarray[2];
+    clock_t times(), wall;
+    struct tms tbuf;
+    wall = times(&tbuf);
+    tarray[0] = (float) (tbuf.tms_utime / (float)CLOCKS_PER_SEC);
+    tarray[1] = (float) (tbuf.tms_stime / (float)CLOCKS_PER_SEC);
+    *t = (double) (tarray[0]+tarray[1]);
+    printf("dtime: %lf\n",*t);
+  */
+}
+
