@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-  Ver. 3.5 (June/20/2008)
+  Ver. 3.7 (15/May/2013)
 
   OpenMX (Open source package for Material eXplorer) is a program package
   for linear scaling density functional calculations of large-scale materials.
@@ -33,13 +33,12 @@
 
      Taisuke Ozaki
 
-     Present (Sep./01/2009) official address
+     Present (15/May/2013) official address
 
-       Research Center for Integrated Sciences (RCIS), 
        Japan Advanced Institute of Science and Technology (JAIST)
        Asahidai 1-1, Nomi, Ishikawa 923-1292, Japan
 
-       e-mail: t-ozaki@jaist.go.jp
+       e-mail: t-ozaki@jaist.ac.jp
     **************************************************************
  
 *****************************************************************************/
@@ -66,18 +65,8 @@
 #include <unistd.h>
 /*  end stat section */
 #include "openmx_common.h"
-
-#ifdef nompi
-#include "mimic_mpi.h"
-#else
 #include "mpi.h"
-#endif
-
-#ifdef noomp
-#include "mimic_omp.h"
-#else
 #include <omp.h>
-#endif
   
 #include "tran_prototypes.h"
 #include "tran_variables.h"
@@ -90,23 +79,18 @@ int main(int argc, char *argv[])
   static char fileE[YOUSO10] = ".ene"; 
   static char fileDRC[YOUSO10] = ".md";
   static char fileMemory[YOUSO10]; 
-  static char fileRestart[YOUSO10];
-  static char operate[200];
   double TStime,TEtime;
-  static struct stat statbuf; 
-
-  /* for idle CPUs */
-  int tag;
-  int complete;
-  MPI_Request request;
-  MPI_Status  status;
 
   /* MPI initialize */
 
   mpi_comm_level1 = MPI_COMM_WORLD; 
+  MPI_COMM_WORLD1 = MPI_COMM_WORLD; 
+
   MPI_Init(&argc,&argv);
-  MPI_Comm_size(mpi_comm_level1,&numprocs);
-  MPI_Comm_rank(mpi_comm_level1,&myid);
+  MPI_Comm_size(MPI_COMM_WORLD,&numprocs);
+  MPI_Comm_rank(MPI_COMM_WORLD,&myid);
+  NUMPROCS_MPI_COMM_WORLD = numprocs;
+  MYID_MPI_COMM_WORLD = myid;
   Num_Procs = numprocs;
 
   /* for measuring elapsed time */
@@ -144,8 +128,8 @@ int main(int argc, char *argv[])
     }
   }
 
-  MPI_Bcast(&po, 1, MPI_INT, Host_ID, mpi_comm_level1);
-  MPI_Bcast(&ip, 1, MPI_INT, Host_ID, mpi_comm_level1);
+  MPI_Bcast(&po, 1, MPI_INT, Host_ID, MPI_COMM_WORLD1);
+  MPI_Bcast(&ip, 1, MPI_INT, Host_ID, MPI_COMM_WORLD1);
 
   if ( (argc-1)<(ip+1) ){
     if (myid==Host_ID){
@@ -175,10 +159,10 @@ int main(int argc, char *argv[])
   }
 
   /****************************************************
-    ./openmx -show 
+    ./openmx -show directory 
 
-    making of *.out files in order to check whether 
-    OpenMX normally runs on many platforms or not.
+    showing PAOs and VPS used in input files stored in 
+    "directory".
   ****************************************************/
 
   if (strcmp(argv[1],"-show")==0){
@@ -223,6 +207,19 @@ int main(int argc, char *argv[])
     exit(1);
   }
 
+  /****************************************************
+   ./openmx -maketestL2
+
+    making of *.out files in order to check whether 
+    OpenMX normally runs for large systems on many 
+    platforms or not
+  ****************************************************/
+
+  if ( (argc==2 || argc==3) && strcmp(argv[1],"-maketestL2")==0){
+    Maketest("L2",argc,argv);
+    exit(1);
+  }
+
   /****************************************************
    ./openmx -runtestL
 
@@ -235,6 +232,18 @@ int main(int argc, char *argv[])
     Runtest("L",argc,argv);
   }
 
+  /****************************************************
+   ./openmx -runtestL2
+
+   check whether OpenMX normally runs for large systems 
+   on many platforms or not by comparing the stored *.out 
+   and generated *.out on your machine.
+  ****************************************************/
+
+  if (strcmp(argv[1],"-runtestL2")==0){
+    Runtest("L2",argc,argv);
+  }
+
   /*******************************************************
    check memory leak by monitoring actual used memory size
   *******************************************************/
@@ -316,6 +325,7 @@ int main(int argc, char *argv[])
 
   if (strcmp(argv[1],"-runtestNEGF")==0){
     Runtest("NEGF",argc,argv);
+    MPI_Finalize();
     exit(1);
   }
 
@@ -342,19 +352,29 @@ int main(int argc, char *argv[])
     Force_test(argc,argv);
     exit(1);
   }
+  
+  /*******************************************************
+    check the NEB calculation or not, and if yes, go to 
+    the NEB calculation.
+  *******************************************************/
+
+  if (neb_check(argv)) neb(argc,argv);
+
+  /*******************************************************
+   allocation of CompTime and show the greeting message 
+  *******************************************************/
 
-  /* allocation of CompTime */
   CompTime = (double**)malloc(sizeof(double*)*numprocs); 
   for (i=0; i<numprocs; i++){
-    CompTime[i] = (double*)malloc(sizeof(double)*20); 
-    for (j=0; j<20; j++) CompTime[i][j] = 0.0;
+    CompTime[i] = (double*)malloc(sizeof(double)*30); 
+    for (j=0; j<30; j++) CompTime[i][j] = 0.0;
   }
 
   if (myid==Host_ID){  
     printf("\n*******************************************************\n"); 
     printf("*******************************************************\n"); 
     printf(" Welcome to OpenMX   Ver. %s                           \n",Version_OpenMX); 
-    printf(" Copyright (C), 2002-2009, T. Ozaki                    \n"); 
+    printf(" Copyright (C), 2002-2013, T. Ozaki                    \n"); 
     printf(" OpenMX comes with ABSOLUTELY NO WARRANTY.             \n"); 
     printf(" This is free software, and you are welcome to         \n"); 
     printf(" redistribute it under the constitution of the GNU-GPL.\n");
@@ -370,24 +390,25 @@ int main(int argc, char *argv[])
                    Read the input file
   ****************************************************/
 
-  /* setup CPU group */
-  setup_CPU_group(argv[1]);
-  if (myid>=atomnum)  goto LAST_PROC;  /*  to cut off CPUs */
-
   init_alloc_first();
+
   CompTime[myid][1] = readfile(argv);
 
-  MPI_Barrier(mpi_comm_level1);
+  MPI_Barrier(MPI_COMM_WORLD1);
 
   /* initialize PrintMemory routine */
 
   sprintf(fileMemory,"%s%s.memory%i",filepath,filename,myid);
   PrintMemory(fileMemory,0,"init"); 
   PrintMemory_Fix();
- 
+
   /* initialize */
-  
+
   init();
+
+  /* for DFTD-vdW by okuno */
+  if(dftD_switch==1) DFTDvdW_init();
+
   fnjoint(filepath,filename,fileE);
   fnjoint(filepath,filename,fileDRC);
 
@@ -403,8 +424,8 @@ int main(int argc, char *argv[])
     }
   }
 
-  MPI_Bcast(&po, 1, MPI_INT, Host_ID, mpi_comm_level1);
-  MPI_Bcast(&ip, 1, MPI_INT, Host_ID, mpi_comm_level1);
+  MPI_Bcast(&po, 1, MPI_INT, Host_ID, MPI_COMM_WORLD1);
+  MPI_Bcast(&ip, 1, MPI_INT, Host_ID, MPI_COMM_WORLD1);
 
   if ( po==1 ) ML_flag = 1;
   else         ML_flag = 0;
@@ -421,22 +442,27 @@ int main(int argc, char *argv[])
     }
   }
 
-  MPI_Bcast(&po, 1, MPI_INT, Host_ID, mpi_comm_level1);
-  MPI_Bcast(&ip, 1, MPI_INT, Host_ID, mpi_comm_level1);
+  MPI_Bcast(&po, 1, MPI_INT, Host_ID, MPI_COMM_WORLD1);
+  MPI_Bcast(&ip, 1, MPI_INT, Host_ID, MPI_COMM_WORLD1);
 
   if ( po==1 ){
     force_flag = atoi(argv[ip+1]);
     ForceConsistency_flag = 1;
   }
 
-  /* check force consistency */
+  /* check force consistency 
+     the number of processes 
+     should be less than 2.
+  */
 
   if (ForceConsistency_flag==1){
+
     Check_Force(argv);
-    OutData(argv[1]);
+    CompTime[myid][20] = OutData(argv[1]);
     Merge_LogFile(argv[1]);
     Free_Arrays(0);
     MPI_Finalize();
+    exit(0); 
     return 1;
   }
 
@@ -449,8 +475,12 @@ int main(int argc, char *argv[])
 
   do {
 
-    CompTime[myid][2] += truncation(MD_iter,Solver==6,1);
-    if (ML_flag==1 && myid==Host_ID) Get_VSZ(MD_iter);  
+    if (MD_switch==12)
+      CompTime[myid][2] += truncation(1,1);  /* EvsLC */
+    else 
+      CompTime[myid][2] += truncation(MD_iter,1);
+
+    if (ML_flag==1 && myid==Host_ID) Get_VSZ(MD_iter);
 
     if (Solver==4) {
       TRAN_Calc_GridBound( mpi_comm_level1, atomnum, WhatSpecies, Spe_Atom_Cut1,
@@ -461,7 +491,7 @@ int main(int argc, char *argv[])
 
     CompTime[myid][3] += DFT(MD_iter,(MD_iter-1)%orbitalOpt_per_MDIter+1);
 
-    if (myid==Host_ID) iterout(MD_iter,MD_TimeStep*(MD_iter-1),fileE,fileDRC);
+    iterout(MD_iter+MD_Current_Iter,MD_TimeStep*(MD_iter+MD_Current_Iter-1),fileE,fileDRC);
 
     /* MD or geometry optimization */
 
@@ -509,7 +539,10 @@ int main(int argc, char *argv[])
                   Making of output files
   ****************************************************/
 
-  OutData(argv[1]);
+  if (OutData_bin_flag) 
+    CompTime[myid][20] = OutData_Binary(argv[1]);
+  else 
+    CompTime[myid][20] = OutData(argv[1]);
 
   /****************************************************
     write connectivity, Hamiltonian, overlap, density
@@ -544,11 +577,14 @@ int main(int argc, char *argv[])
 
   Free_Arrays(0);
 
-LAST_PROC: 
+  /* print memory */
 
-  if (myid<atomnum) PrintMemory("total",0,"sum");
+  PrintMemory("total",0,"sum");
 
-  printf("\nThe calculation was normally finished. (proc=%3d)\n",myid);
+  MPI_Barrier(MPI_COMM_WORLD);
+  if (myid==Host_ID){
+    printf("\nThe calculation was normally finished.\n");
+  }
 
   MPI_Finalize();
 
