@@ -1,6 +1,6 @@
 /**********************************************************************
   openmx_common.c:
-
+  
      openmx_common.c is a collective routine of subroutines
      which are often used.
 
@@ -14,6 +14,8 @@
 #include <stdlib.h>
 #include <string.h>
 #include <math.h>
+#include <ctype.h>
+
 #include "openmx_common.h"
 
 static int imaxarg1,imaxarg2;
@@ -307,27 +309,20 @@ double rnd(double width)
   double result;
 
   result = rand();
-
-  while (width<result){
-    result = result/2.0;
-  }
-  result = result - width*0.75;
+  result = result*width/(double)RAND_MAX - 0.5*width;
   return result;
 }
 
 double rnd0to1()
 {
-
   /****************************************************
    This rnd() function generates random number 0 to 1
   ****************************************************/
 
   double result;
+
   result = rand();
-  while (2.0<result){
-    result = result/2.0;
-  }
-  result = result - 1.0;
+  result /= (double)RAND_MAX;
   return result;
 }
 
@@ -885,3 +880,17 @@ void k_inversion(int i,  int j,  int k,
        *ij= mj-j-1;
        *ik= mk-k-1;
 }
+
+
+char *string_tolower(char *buf, char *buf1)
+{
+  char *c=buf;
+  char *c1=buf1;
+
+  while (*c){
+    *c1=tolower(*c);
+    c++;
+    c1++;
+  }
+ return buf;
+}
