@@ -565,79 +565,6 @@ void Input( FILE *fp )
     printf("error in making temporal_12345.input\n"); 
   }
 
-  /****************************************************
-      exchange splitting matrix
-  ****************************************************/
-
-  fread(i_vec, sizeof(int), 1, fp);
-  Num_ESM = i_vec[0];
-
-  /*
-  printf("Num_ESM=%2d\n",Num_ESM);
-  */
-
-  if (Num_ESM!=0){
-
-    ESM_Atoms = (int*)malloc(sizeof(int)*Num_ESM);      
-    fread(ESM_Atoms, sizeof(int), Num_ESM, fp);
-
-    for (i=0; i<Num_ESM; i++){
-      printf("ESM_Atoms %i %i\n",i,ESM_Atoms[i]);
-    }
-
-    /****************************************************
-     global allocation of arrays:
-     static double ESM[Num_ESM][FNAN][FNAN][tno0][tno1]
-    ****************************************************/
-
-    ESM = (double*****)malloc(sizeof(double****)*Num_ESM);
-    for (k=0; k<Num_ESM; k++){
-
-      Gc_AN = ESM_Atoms[k];
-      ESM[k] = (double****)malloc(sizeof(double***)*(FNAN[Gc_AN]+1));
-
-      for (h_AN=0; h_AN<=FNAN[Gc_AN]; h_AN++){
-	Gh_AN = natn[Gc_AN][h_AN];
-	TNO1 = Total_NumOrbs[Gh_AN];
-
-	ESM[k][h_AN] = (double***)malloc(sizeof(double**)*(FNAN[Gc_AN]+1));
-
-	for (q_AN=0; q_AN<=FNAN[Gc_AN]; q_AN++){
-	  Gq_AN = natn[Gc_AN][q_AN];
-	  TNO2 = Total_NumOrbs[Gq_AN];
-	  ESM[k][h_AN][q_AN] = (double**)malloc(sizeof(double*)*TNO1);
-
-	  for (i=0; i<TNO1; i++){
-	    ESM[k][h_AN][q_AN][i] = (double*)malloc(sizeof(double)*TNO2);
-	  }
-	}
-      }
-    }
-
-    /* read ESM */    
-    for (k=0; k<Num_ESM; k++){
-      Gc_AN = ESM_Atoms[k];
-      for (h_AN=0; h_AN<=FNAN[Gc_AN]; h_AN++){
-	Gh_AN = natn[Gc_AN][h_AN];
-	TNO1 = Total_NumOrbs[Gh_AN];
-	for (q_AN=0; q_AN<=FNAN[Gc_AN]; q_AN++){
-	  Gq_AN = natn[Gc_AN][q_AN];
-	  TNO2 = Total_NumOrbs[Gq_AN];
-	  for (i=0; i<TNO1; i++){
-	    fread(ESM[k][h_AN][q_AN][i],sizeof(double),TNO2,fp);
-
-	    for (j=0; j<TNO2; j++){
-              printf("%10.6f ",ESM[k][h_AN][q_AN][i][j]);
-	    }            
-            printf("\n");
- 
-	  }
-	}
-      }
-    }
-
-  }  
-
 }
 
 
