@@ -14,17 +14,10 @@
 #include <math.h>
 #include <time.h>
 #include "openmx_common.h"
-
-#ifdef nompi
-#include "mimic_mpi.h"
-#else
 #include "mpi.h"
-#endif
-
 #include "tran_prototypes.h"
 
 
-
 double readfile(char *argv[])
 { 
   double time0;
@@ -36,8 +29,8 @@ double readfile(char *argv[])
 
   dtime(&TStime);
 
-  MPI_Comm_size(mpi_comm_level1,&numprocs);
-  MPI_Comm_rank(mpi_comm_level1,&myid);
+  MPI_Comm_size(MPI_COMM_WORLD1,&numprocs);
+  MPI_Comm_rank(MPI_COMM_WORLD1,&myid);
 
   /****************************************************
            Read a input file or restart file.
@@ -50,7 +43,6 @@ double readfile(char *argv[])
 #endif
 
     Input_std(argv[1]);
-
     fclose(fp);
   }
   else{
@@ -58,52 +50,42 @@ double readfile(char *argv[])
     exit(0);
   }
 
+  Allocate_Arrays(2);
+  Set_Allocate_Atom2CPU(0,1,0); /* for species */
+  SetPara_DFT();
 
-  if (myid<atomnum) {
+  if ( TRAN_Check_Region_Lead(atomnum, WhatSpecies, Spe_Atom_Cut1, Gxyz, tv)==0 ) {
+    printf("\n\nERROR: PAOs of lead atoms can overlap only to the next nearest region.\n\n");
+    MPI_Finalize();
+    exit(1);
+  }
 
-    Allocate_Arrays(2);
-    Set_Allocate_Atom2CPU(0,1,0); /* for species */
-
-    SetPara_DFT();
-
-    if ( TRAN_Check_Region_Lead(atomnum, WhatSpecies, Spe_Atom_Cut1, Gxyz, tv)==0 ) {
-      printf("\n\nERROR: PAOs of lead atoms can overlap only to the next nearest region.\n\n");
+  if ( Solver==4 ) {
+    if ( TRAN_Check_Region(atomnum, WhatSpecies, Spe_Atom_Cut1, Gxyz)==0 ) {
+      printf("\n\nERROR: PAOs of atoms of L|C|R can overlap only to the next nearest region.\n\n");
       MPI_Finalize();
       exit(1);
     }
+  }
 
-    if ( Solver==4 ) {
-      if ( TRAN_Check_Region(atomnum, WhatSpecies, Spe_Atom_Cut1, Gxyz)==0 ) {
-	printf("\n\nERROR: PAOs of atoms of L|C|R can overlap only to the next nearest region.\n\n");
-	MPI_Finalize();
-	exit(1);
-      }
-    }
+  /*****************************
+    last input
+    0: atomnum, 
+    1: elapsed time 
+  *****************************/    
 
-    Set_Allocate_Atom2CPU(0,0,0); /* a simple division for atoms (get Matomnum) */
-
-    if (Solver!=6) { /*  except for GDC */
-
-    /*****************************
-      final input
-      0: atomnum, 
-      1: the neighbor 
-      2: elapsed time 
-    *****************************/    
-
-      Set_Allocate_Atom2CPU(1,0,0); 
-    }
-  } 
+  Set_Allocate_Atom2CPU(1,0,0); 
 
   /***************************************************************
    NEGF:
    check the consistency between the current and previous inputs 
   ***************************************************************/
 
-  TRAN_Check_Input(mpi_comm_level1, Solver);
+  TRAN_Check_Input(MPI_COMM_WORLD1, Solver);
 
   dtime(&TEtime);
   time0 = TEtime - TStime;
+
   return time0;
 }
 
